unit CBuf;

interface

type
  TCBuf = class
    constructor Create(ASize : integer);
    destructor Destroy; override;
  private
    FBuf  : pchar;
    FSize : integer;
    H     : integer;
    T     : integer;
    Mtx   : THandle;
    function GetDataSize : integer;
    function GetFreeSpace : integer;
  public
    property DataSize  : integer read GetDataSize;
    property FreeSpace : integer read GetFreeSpace;
    function Put(c : pchar; ALen : integer) : integer;
    function Get(c : pchar; ALen : integer) : integer;
  end;

implementation

uses
  Windows;

// ========== TCBuf ============================================================

constructor TCBuf.Create(ASize : integer);
begin
  inherited Create;
  FSize:=ASize;
  GetMem(FBuf,ASize+1);
  H:=0; // Points to the beginning of free space
  T:=0; // Points to the beginning of data
  Mtx:=CreateMutex(nil,false,nil);
end;

// -----------------------------------------------------------------------------

destructor TCBuf.Destroy;
begin
  CloseHandle(Mtx);
  FreeMem(FBuf);
  inherited Destroy;
end;

// -----------------------------------------------------------------------------

function TCBuf.GetDataSize : integer;
begin
  WaitForSingleObject(Mtx,10000);
  try
    if H >= T then
      result:=H-T
    else
      result:=H+(FSize-T);
  finally
    ReleaseMutex(Mtx);
  end;
end;

// -----------------------------------------------------------------------------

function TCBuf.GetFreeSpace : integer;
begin
  WaitForSingleObject(Mtx,10000);
  try
    if H >= T then
      result:=T+(FSize-H)-1
    else
      result:=T-H-1;
  finally
    ReleaseMutex(Mtx);
  end;
end;

// -----------------------------------------------------------------------------

function TCBuf.Put(c : pchar; ALen : integer) : integer;
begin
  WaitForSingleObject(Mtx,10000);
  try
    if H >= T then
      result:=T+(FSize-H)-1
    else
      result:=T-H-1;
    if result > ALen then
      result:=ALen;
    if H >= T then
    begin
      if result <= (FSize-H) then
      begin
        CopyMemory(@FBuf[H],c,result);
        inc(H,result);
        if H = FSize then H:=0;
      end
      else
      begin
        CopyMemory(@FBuf[H],c,FSize-H);
        CopyMemory(FBuf,@c[FSize-H],result-(FSize-H));
        H:=result-(FSize-H);
      end;
    end
    else
    begin
      CopyMemory(@FBuf[H],c,result);
      inc(H,result);
    end;
  finally
    ReleaseMutex(Mtx);
  end;
end;

// -----------------------------------------------------------------------------

function TCBuf.Get(c : pchar; ALen : integer) : integer;
begin
  WaitForSingleObject(Mtx,10000);
  try
    if H >= T then
      result:=H-T
    else
      result:=H+(FSize-T);
    if result > ALen then result:=ALen;
    // Assume that there is enough space in the output buffer
    if H >= T then
    begin
      CopyMemory(c,@FBuf[T],result);
      inc(T,result);
    end
    else
    begin
      if result <= (FSize-T) then
      begin
        CopyMemory(c,@FBuf[T],result);
        inc(T,result);
        if T = FSize then T:=0;
      end
      else
      begin
        CopyMemory(c,@FBuf[T],FSize-T);
        CopyMemory(@c[FSize-H],FBuf,result-(FSize-T));
        T:=result-(FSize-T);
      end;
    end;
  finally
    ReleaseMutex(Mtx);
  end;
end;

// -----------------------------------------------------------------------------

end.
