unit CharEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, CharTable, ComCtrls;

type
  TfrmCharEdit = class(TForm)
    btClose: TButton;
    btSave: TButton;
    sb: TStatusBar;
    procedure FormPaint(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
  private
    CA : array [0..255] of byte;
    CALen : integer;
    MDown : boolean;
    FMaxCharSize : integer;
    procedure SetupArray;
    procedure UpdateChar;
    function SetBitAtXY(X,Y : integer; SetFlag : boolean) : boolean;
  public
    Character : TCharacter;
    Modified : boolean;
  end;

var
  frmCharEdit: TfrmCharEdit;

procedure OpenCharEditor(AChar : TCharacter; FeedbackFrm : THandle; MaxCharSize : integer);

implementation

{$R *.dfm}

// =============================================================================

procedure OpenCharEditor(AChar : TCharacter; FeedbackFrm : THandle; MaxCharSize : integer);
begin
  frmCharEdit.DoubleBuffered := true;
  frmCharEdit.MDown := false;
  frmCharEdit.Character := AChar;
  frmCharEdit.FMaxCharSize := MaxCharSize;
  frmCharEdit.ClientWidth := MaxCharSize*20;
  frmCharEdit.SetupArray;
  frmCharEdit.Modified := false;
  if byte(AChar.Code) < $20 then
    frmCharEdit.sb.Panels[0].Text := format('%2.2x',[byte(AChar.Code)])
  else
  if byte(AChar.Code) = $20 then
    frmCharEdit.sb.Panels[0].Text := format('%2.2x = SPACE',[byte(AChar.Code)])
  else
    frmCharEdit.sb.Panels[0].Text := format('%2.2x = %s',[byte(AChar.Code),AChar.Code]);
  if frmCharEdit.ShowModal = mrOk then
  begin
    frmCharEdit.UpdateChar;
    frmCharEdit.Modified := false;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmCharEdit.FormPaint(Sender: TObject);
var
  ix,iy : integer;
  ww,hh : integer;
  Rect  : TRect;
  rr2   : TRect;
  CC    : TCanvas;
  dsz   : integer;
begin
  CC := Canvas;
  CC.Lock;
  try
    CC.Brush.Color := RGB(100,0,0);
    CC.Pen.Color := 0;
    CC.Pen.Style := psSolid;
    CC.Pen.Width := 1;
    dsz := (ClientWidth-20) div FMaxCharSize;
    if ((ClientHeight-70) div 5) < dsz then
      dsz := ((ClientHeight-70) div 5);
    ww := FMaxCharSize*dsz;
    hh := 5*dsz; // Char height is 5
    Rect.Left := 10;
    Rect.Top := 10;
    Rect.Right := Rect.Left+ww;
    Rect.Bottom := Rect.Top+hh;
    CC.Rectangle(Rect);
    for ix:=0 to FMaxCharSize-1 do
    begin
      for iy:=4 downto 0 do
      begin
        if ((CA[ix] and (1 shl iy)) = 0) then
          CC.Brush.Color := RGB(100,0,0)
        else
          CC.Brush.Color := RGB(255,140,140);
        rr2.Left := Rect.Left+ix*dsz;
        rr2.Right := rr2.Left+dsz;
        rr2.Top := Rect.Top+(4-iy)*dsz;
        rr2.Bottom := rr2.Top+dsz;
        inc(rr2.Left); inc(rr2.Top); dec(rr2.Right); dec(rr2.Bottom);
        CC.Ellipse(rr2);
      end;
    end;
  finally
    CC.Unlock;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmCharEdit.SetupArray;
var
  ii : integer;
begin
  CALen := length(Character.Data);
  for ii:=0 to FMaxCharSize-1 do
  begin
    if ii < CALen then
      CA[ii] := Character.Data[ii]
    else
      CA[ii] := 0;
  end;
  Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmCharEdit.UpdateChar;
var
  ii : integer;
begin
  CALen := FMaxCharSize;
  while (CALen > 1) and (CA[CALen-1] = 0) do dec(CALen);
  if CALen <> length(Character.Data) then setlength(Character.Data,CALen);
  for ii:=0 to CALen-1 do
    Character.Data[ii] := CA[ii];
end;

// -----------------------------------------------------------------------------

procedure TfrmCharEdit.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    SetBitAtXY(X,Y,true)
  else
    SetBitAtXY(X,Y,false);
  MDown := true;
  Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmCharEdit.FormMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  refr : boolean;
begin
  if not MDown then exit;
  if ssLeft in Shift then
    refr := SetBitAtXY(X,Y,true)
  else
    refr := SetBitAtXY(X,Y,false);
  if refr then Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmCharEdit.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MDown := false;
end;

// -----------------------------------------------------------------------------

function TfrmCharEdit.SetBitAtXY(X,Y : integer; SetFlag : boolean) : boolean;
var
  ix,iy : integer;
  ww,hh : integer;
  Rect  : TRect;
  rr2   : TRect;
  dsz   : integer;
  cstate: boolean;
begin
  result := false;
  dsz := (ClientWidth-20) div FMaxCharSize;
  if ((ClientHeight-70) div 5) < dsz then
    dsz := ((ClientHeight-70) div 5);
  ww := FMaxCharSize*dsz;
  hh := 5*dsz; // Char height is 5
  Rect.Left := 10;
  Rect.Top := 10;
  Rect.Right := Rect.Left+ww;
  Rect.Bottom := Rect.Top+hh;
  for ix:=0 to FMaxCharSize-1 do
  begin
    for iy:=4 downto 0 do
    begin
      rr2.Left := Rect.Left+ix*dsz;
      rr2.Right := rr2.Left+dsz;
      rr2.Top := Rect.Top+(4-iy)*dsz;
      rr2.Bottom := rr2.Top+dsz;
      inc(rr2.Left); inc(rr2.Top); dec(rr2.Right); dec(rr2.Bottom);
      if (X > rr2.Left) and (X < rr2.Right) and (Y > rr2.Top) and (Y < rr2.Bottom) then
      begin
        cstate := ((CA[ix] and (1 shl iy)) <> 0);
        if cstate <> SetFlag then
        begin
          if SetFlag then
            CA[ix] := CA[ix] or (1 shl iy)
          else
            CA[ix] := CA[ix] and not ($1 shl iy);
          Modified := true;
          result := true;
        end;
        exit;
      end;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmCharEdit.FormResize(Sender: TObject);
begin
  btSave.Top := ClientHeight-50;
  btClose.Top := ClientHeight-50;
  Invalidate;
end;

// -----------------------------------------------------------------------------

end.

