unit CharTable;

interface

uses
  Windows, Classes, Graphics;

type
  TCharData = array of byte;

type
  TCharacter = class
    constructor Create(ACode : char; var AData; Len : integer);
    destructor Destroy; override;
  public
    Data : TCharData;
    Code : char;
    procedure SetData(var AData; Len : integer);
    function GetRect : TRect;
    procedure DrawTo(CC : TCanvas; At : TPoint; Highlighted : boolean);
  end;

type
  TCharTable = class (TList)
    procedure Clear; override;
  private
    function GetCharacter(Code : byte) : TCharacter;
  public
    property Chars[Code : byte] : TCharacter read GetCharacter; default;
  end;

implementation

uses
  SysUtils;

// ========== TCharacter =======================================================

constructor TCharacter.Create(ACode : char; var AData; Len : integer);
var
  ii : integer;
begin
  inherited Create;
  Code := ACode;
  setlength(Data,Len);
  for ii:=0 to Len-1 do
    Data[ii] := TByteArray(AData)[ii];
end;

// -----------------------------------------------------------------------------

destructor TCharacter.Destroy;
begin
  setlength(Data,0);
  inherited Destroy;
end;

// -----------------------------------------------------------------------------

procedure TCharacter.SetData(var AData; Len : integer);
var
  ii : integer;
begin
  setlength(Data,Len);
  for ii:=0 to Len-1 do
    Data[ii] := TByteArray(AData)[ii];
end;

// -----------------------------------------------------------------------------

function TCharacter.GetRect : TRect;
var
  sx,sy : integer;
begin
  sx := length(Data);
  sy := 5;
  result.Left := 0;
  result.Top := 0;
  result.Right := (sx+1)*8+20;
  if result.Right < 80 then result.Right := 80;
  result.Bottom := (sy+1)*8+40;
end;

// -----------------------------------------------------------------------------

procedure TCharacter.DrawTo(CC : TCanvas; At : TPoint; Highlighted : boolean);
var
  ix,iy : integer;
  sx,sy : integer;
  ww    : integer;
  Rect  : TRect;
  rr1,rr2 : TRect;
  ss    : string;
begin
  CC.Lock;
  try
    if Highlighted then
      CC.Brush.Color := RGB(255,220,220)
    else
      CC.Brush.Color := RGB(220,220,220);
    CC.Pen.Color := 0;
    CC.Pen.Style := psSolid;
    CC.Pen.Width := 1;
    sx := length(Data);
    ww := (sx+1)*8;
    sy := 5;
    Rect.Left := At.X;
    Rect.Top := At.Y;
    Rect.Right := ww+20;
    if Rect.Right < 80 then Rect.Right := 80;
    Rect.Right := Rect.Right+At.X;
    Rect.Bottom := At.Y+(sy+1)*8+40;
    CC.Rectangle(Rect);
    rr1.Top := Rect.Top+10;
    rr1.Bottom := rr1.Top+(sy+1)*8;
    rr1.Left := Rect.Left+((Rect.Right-Rect.Left)-ww) div 2;
    rr1.Right := rr1.Left+ww;
    if sx > 0 then
    begin
      CC.Brush.Color := RGB(100,0,0);
      CC.Rectangle(rr1);
      for ix:=0 to length(Data)-1 do
      begin
        for iy:=4 downto 0 do
        begin
          if (Data[ix] and (1 shl iy)) = 0 then
            CC.Brush.Color := RGB(100,0,0)
          else
            CC.Brush.Color := RGB(255,140,140);
          rr2.Left := rr1.Left+ix*8+4;
          rr2.Right := rr2.Left+7;
          rr2.Top := rr1.Top+(4-iy)*8+4;
          rr2.Bottom := rr2.Top+7;
          CC.Ellipse(rr2);
        end;
      end;
    end;
    CC.Brush.Style := bsClear;
    CC.Font.Name := 'Arial';
    CC.Font.Size := 8;
    CC.Font.Style := [fsBold];
    if byte(Code) < $20 then
      ss := format('%2.2x',[byte(Code)])
    else
    if byte(Code) = $20 then
      ss := format('%2.2x = SPACE',[byte(Code)])
    else
      ss := format('%2.2x = %s',[byte(Code),Code]);
    rr2.Top := rr1.Bottom+10;
    rr2.Bottom := Rect.Bottom-5;
    rr2.Left := Rect.Left+5;
    rr2.Right := Rect.Right-5;
    CC.TextRect(rr2,(rr2.Left+rr2.Right-CC.TextWidth(ss)) div 2,rr2.Top,ss);
  finally
    CC.Unlock;
  end;
end;

// ========== TCharTable =======================================================

procedure TCharTable.Clear;
var
  ii : integer;
begin
  for ii:=0 to Count-1 do Chars[ii].Free;
  inherited Clear;
end;

// -----------------------------------------------------------------------------

function TCharTable.GetCharacter(Code : byte) : TCharacter;
var
  ii : integer;
begin
  if (Code >  $7f) then
    result := nil
  else
  begin
    while (Code >= Count) do Add(TCharacter.Create(char(Count),ii,0));
    result := items[Code];
  end;
end;

// -----------------------------------------------------------------------------

end.

