unit ColEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, GenericCommand;

type
  TfrmColEditor = class(TForm)
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FCmd : TGenericCommand;
    FCol : integer;
    InsEdit : boolean;
    hMouseDown : TMouseEvent;
    hMouseMove : TMouseMoveEvent;
    procedure LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  public
    procedure OpenEditor(ACmd : TGenericCommand; ACol : integer);
  end;

var
  frmColEditor: TfrmColEditor;

implementation

uses
  main, SerialIO, PCB;

{$R *.dfm}

// =============================================================================

procedure TfrmColEditor.OpenEditor(ACmd : TGenericCommand; ACol : integer);
var
  Row,Col : integer;
  bb  : byte;
  bb1 : byte;
  ss  : string;
  ii  : integer;
begin
  if not assigned(frmMain.Dev) then exit;
  FCmd := ACmd;
  FCol := ACol;
  if (FCol < 0) or (FCol >= frmMain.Dev.PCB.ColCount) then exit;
  if FCmd.Parameters.IndexOfName('Col') >= 0 then
  begin
    InsEdit := true;
    bb := UnhexByte(FCmd.Parameters['Col']);
    bb1 := 1;
    for Row:=frmMain.Dev.PCB.RowCount-1 downto 0 do
    begin
      frmMain.Dev.PCB[FCol,Row].Active := ((bb and bb1) <> 0);
      frmMain.Dev.PCB[FCol,Row].Brightness := 7;
      bb1 := bb1 shl 1;
    end;
  end
  else
  begin
    InsEdit := false;
    ss := FCmd.Parameters['Value'];
    while length(ss) < frmMain.Dev.PCB.RowCount do ss := '0'+ss;
    ii := length(ss);
    for Row:=frmMain.Dev.PCB.RowCount-1 downto 0 do
    begin
      for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
      begin
        if Col = FCol then
          frmMain.Dev.PCB[Col,Row].Active := (ss[ii] = '1')
        else
        begin
          frmMain.Dev.PCB[Col,Row].Active := false;
          frmMain.Dev.PCB[Col,Row].Brightness := 7;
        end;
      end;
      dec(ii);
    end;
  end;
  frmMain.Invalidate;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmColEditor.FormShow(Sender: TObject);
begin
  DisableAPP;
  hMouseDown := frmMain.OnMouseDown;
  hMouseMove := frmMain.OnMouseMove;
  frmMain.OnMouseDown := LocalMouseDown;
  frmMain.OnMouseMove := LocalMouseMove;
end;

// -----------------------------------------------------------------------------

procedure TfrmColEditor.FormHide(Sender: TObject);
var
  Row : integer;
  Col : integer;
begin
  frmMain.OnMouseDown := hMouseDown;
  frmMain.OnMouseMove := hMouseMove;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
       frmMain.Dev.PCB[Col,Row].Highlighted := false;
  EnableAPP;
  frmMain.SavePositions(self);
end;

// -----------------------------------------------------------------------------

procedure TfrmColEditor.LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Row : integer;
  Md  : boolean;
begin
  Md := false;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    if (X > frmMain.Dev.PCB[FCol,Row].LastRect.Left) and (X < frmMain.Dev.PCB[FCol,Row].LastRect.Right) and
       (Y > frmMain.Dev.PCB[FCol,Row].LastRect.Top) and (Y < frmMain.Dev.PCB[FCol,Row].LastRect.Bottom) then
    begin
      frmMain.Dev.PCB[FCol,Row].Active := not frmMain.Dev.PCB[FCol,Row].Active;
      Md := true;
    end;
  end;
  if Md then frmMain.Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmColEditor.LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  Row : integer;
  Md  : boolean;
begin
  Md := false;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    if (X > frmMain.Dev.PCB[FCol,Row].LastRect.Left) and (X < frmMain.Dev.PCB[FCol,Row].LastRect.Right) and
       (Y > frmMain.Dev.PCB[FCol,Row].LastRect.Top) and (Y < frmMain.Dev.PCB[FCol,Row].LastRect.Bottom) then
    begin
      if not frmMain.Dev.PCB[FCol,Row].Highlighted then
      begin
        Md := true;
        frmMain.Dev.PCB[FCol,Row].Highlighted := true;
      end;
    end
    else
    begin
      if frmMain.Dev.PCB[FCol,Row].Highlighted then
      begin
        Md := true;
        frmMain.Dev.PCB[FCol,Row].Highlighted := false;
      end;
    end;
  end;
  if Md then frmMain.Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmColEditor.Button2Click(Sender: TObject);
begin
  Close;
end;

// -----------------------------------------------------------------------------

procedure TfrmColEditor.Button1Click(Sender: TObject);
var
  Row : integer;
  bb  : byte;
  bb1 : byte;
  ss  : string;
begin
  bb := 0;
  bb1 := 1;
  ss := '';
  for Row:=frmMain.Dev.PCB.RowCount-1 downto 0 do
  begin
    if InsEdit then
    begin
      if frmMain.Dev.PCB[FCol,Row].Active then bb := bb or bb1;
      bb1 := bb1 shl 1;
    end
    else
    begin
      if frmMain.Dev.PCB[FCol,Row].Active then
        ss := '1'+ss
      else
        ss := '0'+ss;
    end;
  end;
  if InsEdit then
    FCmd.Parameters['Col'] := HexByte(bb)
  else
    FCmd.Parameters['Value'] := ss;
  Close;
end;

// -----------------------------------------------------------------------------

end.
