unit Constants;

interface

uses
  Windows, Graphics;

var
  SoftwareVersion : string = 'Detecting...'; // Will be filled in on application start
                                             // See the initialization section below
var
  IniFN : string = '';

type
  THexCharSet = set of char;

const
  HexCharSet : THexCharSet = ['0'..'9','a'..'f','A'..'F'];

const
  PCB_SCREEN_COLOR    = clMoneyGreen;
  PCB_COLOR           = clRed;
  PCB_BORDER          = clBlack;

  INP_SCREEN_COLOR    = clMoneyGreen;
  INP_COLOR           = clBlue;
  INP_BORDER          = clBlack;
  INP_BTN_BORDER      = clBlue;

  LED_SCREEN_COLOR    = clRed;
  LED_HL_SCREEN_COLOR = clRed;

  CMD_SCREEN_COLOR    = clYellow;

const
  SLV01_DELAY_INC     = 40;
  SLV01_DELAY11_INC   = 1;

const
  CALL_MSK            = $2000;
  GOTO_MSK            = $4000;

const
  ED_NO_EDIT          = 00;
  ED_TOP_ROW          = 01;
  ED_BOTTOM_ROW       = 02;
  ED_SET_PIXEL        = 03;
  ED_CLR_PIXEL        = 04;
  ED_BRIGHTNESS       = 05;
  ED_ROW_LEFT         = 06;
  ED_ROW_RIGHT        = 07;
  ED_INS_FROM_LEFT    = 08;
  ED_INS_FROM_RIGHT   = 09;
  ED_SIMPLE_MSG       = 10;
  ED_SET_DELAY        = 11;
  ED_SPACING          = 12;
  ED_DIRECTION        = 13;
  ED_MIRRORING        = 14;
  ED_HBUF             = 15;
  ED_VBUF             = 16;
  ED_CALL             = 17;
  ED_GOTO             = 18;
  ED_SIMPLE_MSG_256   = 19;
  ED_SET_DELAY_256    = 20;
  ED_ONOFF_PIXEL      = 21;
  ED_SET_DELAY_2560   = 22;

function Dark(AColor : TColorRef; factor : double) : TColorRef;

procedure ShowProgress(min,max,position : integer; msg : string);
procedure HideProgress;

function BitStr(b : byte) : string;
function StrBit(s : string) : byte;

function UnescapeChars(ss : string) : string;
function EscapeChars(ss : string) : string;

implementation

uses
  Forms, StdCtrls, ComCtrls, Classes, SysUtils, SerialIO;

// =============================================================================

function Dark(AColor : TColorRef; factor : double) : TColorRef;
var
  R,G,B : byte;
begin
  R := round(GetRValue(AColor)*factor);
  G := round(GetGValue(AColor)*factor);
  B := round(GetBValue(AColor)*factor);
  result := RGB(R,G,B);
end;

// -----------------------------------------------------------------------------

var
  ProgressForm    : TForm;
  ProgressLabel   : TLabel;
  ProgressBar     : TProgressBar;

//----------------------------------------------------------------------------

procedure ShowProgress(min,max,position : integer; msg : string);
begin
  if not assigned(ProgressForm) then
  begin
    ProgressForm := TForm.CreateNew(Application);
    with ProgressForm do
    begin
      Width := 400;
      Height := 90;
      Position := poScreenCenter;
//        FormStyle := fsStayOnTop;
      BorderIcons := [];
      BorderStyle := bsSingle;
    end;
    ProgressLabel := TLabel.Create(ProgressForm);
    with ProgressLabel do
    begin
      Parent := ProgressForm;
      Top := 10;
      Left := 10;
      Width := ProgressForm.ClientWidth-20;
      AutoSize := false;
      Alignment := taCenter;
      Caption := '---';
    end;
    ProgressBar := TProgressBar.Create(ProgressForm);
    with ProgressBar do
    begin
      Parent := ProgressForm;
      Top := 30;
      Left := 10;
      Width := ProgressForm.ClientWidth-20;
      Height := 18;
    end;
    ProgressForm.Show;
  end;
  ProgressBar.Min := min;
  ProgressBar.Max := max;
  ProgressBar.Position := position;
  ProgressLabel.Caption := msg;
  ProgressLabel.Invalidate;
  ProgressBar.Invalidate;
  Application.ProcessMessages;
end;

//----------------------------------------------------------------------------

procedure HideProgress;
begin
  if assigned(ProgressForm) then FreeAndNil(ProgressForm);
end;

//----------------------------------------------------------------------------

function BitStr(b : byte) : string;
var
  b1 : byte;
begin
  b1 := $80;
  result := '';
  while b1 <> 0 do
  begin
    if (b1 and b) = 0 then
      result := result+'0'
    else
      result := result+'1';
    b1 := b1 shr 1;
  end;
end;

//----------------------------------------------------------------------------

function StrBit(s : string) : byte;
var
  ii : integer;
  b1 : byte;
begin
  while length(s) < 8 do s := '0'+s;
  result := 0;
  b1 := $80;
  for ii:=1 to 8 do
  begin
    if s[ii] = '1' then
      result := result or b1;
    b1 := b1 shr 1;
  end;
end;

//----------------------------------------------------------------------------

function EscapeChars(ss : string) : string;
var
  ii : integer;
begin
  result := '';
  for ii:=1 to length(ss) do
  begin
    if byte(ss[ii]) < 32 then
      result := result+'0x'+HexByte(byte(ss[ii]))
    else
      result := result+ss[ii];
  end;
end;

//----------------------------------------------------------------------------

function UnescapeChars(ss : string) : string;
var
  ii : integer;
begin
 result := '';
 ii := 1;
 while ii <= length(ss) do
 begin
   if ((length(ss)-ii) > 2) and (copy(ss,ii,2) = '0x') and (ss[ii+2] in HexCharSet) and (ss[ii+3] in HexCharSet) then
   begin
     result := result+char(UnhexByte(copy(ss,ii+2,2)));
     inc(ii,3);
   end
   else
     result := result+ss[ii];
   inc(ii);
 end;
end;

//----------------------------------------------------------------------------

var
  ii : integer;
  dw,FVLen    : dword;
  Pp          : pointer;
  FileVersion : PVSFixedFileInfo;

initialization

  FVLen := GetFileVersionInfoSize(PChar(paramstr(0)),dw);
  GetMem(Pp,FVLen+10);
  SoftwareVersion := '???';
  try
    if GetFileVersionInfo(pchar(paramstr(0)),dw,FVLen,Pp) then
    begin
      if VerQueryValue(Pp,'\',pointer(FileVersion),FVLen) then
      begin
        SoftwareVersion := format('%u.%u.%u.%u',[
          HiWord(FileVersion.dwFileVersionMS),LoWord(FileVersion.dwFileVersionMS),
          HiWord(FileVersion.dwFileVersionLS),LoWord(FileVersion.dwFileVersionLS)]);
      end;
    end;
  finally
    FreeMem(Pp);
  end;

  IniFN := paramstr(0);
  ii := length(IniFN);
  while (ii > 0) and (IniFN[ii] <> '.') do dec(ii);
  IniFN := copy(IniFN,1,ii)+'ini';

end.

