unit FontEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GenericDevice, StdCtrls;

type
  TfrmFontEditor = class(TForm)
    sb: TScrollBar;
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure sbChange(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    Dev   : TGenericDevice;
    CChar : integer;
  public
    procedure OpenEditor(ADev : TGenericDevice);
    procedure EditChar(AChar : integer);
  end;

var
  frmFontEditor: TfrmFontEditor;

implementation

uses
  CharEditor;

{$R *.dfm}

// =============================================================================

procedure TfrmFontEditor.OpenEditor(ADev : TGenericDevice);
begin
  if Visible then
  begin
    SetFocus;
    exit;
  end;
  Dev := ADev;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmFontEditor.FormPaint(Sender: TObject);
var
  ii : integer;
  pt : TPoint;
  r1 : TRect;
begin
  pt.X := 10;
  pt.Y := 10-sb.Position;
  for ii:=Dev.GetCharTblLimits.X to Dev.GetCharTblLimits.Y do
  begin
    r1 := Dev.CharTable[ii].GetRect;
    if (pt.X+r1.Right-r1.Left) > ClientWidth then
    begin
      pt.X := 10;
      inc(pt.Y,r1.Bottom-r1.Top+10);
      if pt.Y > ClientHeight then break;
    end;
    Dev.CharTable[ii].DrawTo(Canvas,pt,false);
    inc(pt.X,r1.Right-r1.Left+10);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmFontEditor.FormResize(Sender: TObject);
var
  ii : integer;
  pt : TPoint;
  r1 : TRect;
begin
  pt.X := 10;
  pt.Y := 10;
  for ii:=Dev.GetCharTblLimits.X to Dev.GetCharTblLimits.Y do
  begin
    r1 := Dev.CharTable[ii].GetRect;
    if (pt.X+r1.Right-r1.Left) > ClientWidth then
    begin
      pt.X := 10;
      inc(pt.Y,r1.Bottom-r1.Top+10);
    end;
    inc(pt.X,r1.Right-r1.Left+10);
  end;
  inc(pt.Y,r1.Bottom-r1.Top+10);
  if pt.Y > ClientHeight then
  begin
    sb.Visible := true;
    sb.Min := 0;
    sb.Max := pt.Y-ClientHeight;
    sb.Position := 0;
    sb.SmallChange := (r1.Bottom-r1.Top+10) div 2;
    sb.LargeChange := ClientHeight;
  end
  else
  begin
    sb.Visible := false;
    sb.Min := 0;
    sb.Max := 1;
    sb.Position := 0;
  end;
  Repaint;
end;

// -----------------------------------------------------------------------------

procedure TfrmFontEditor.sbChange(Sender: TObject);
begin
  Repaint;
end;

// -----------------------------------------------------------------------------

procedure TfrmFontEditor.FormMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  ii : integer;
  pt : TPoint;
  r1 : TRect;
  r2 : TRect;
  rOld,rNew : TRect;
  ncc: integer;
begin
  pt.X := 10;
  pt.Y := 10;
  ncc := -1;
  for ii:=Dev.GetCharTblLimits.X to Dev.GetCharTblLimits.Y do
  begin
    r1 := Dev.CharTable[ii].GetRect;
    if (pt.X+r1.Right-r1.Left) > ClientWidth then
    begin
      pt.X := 10;
      inc(pt.Y,r1.Bottom-r1.Top+10);
    end;
    r2 := r1;
    inc(r2.Left,pt.X);
    inc(r2.Top,pt.Y-sb.Position);
    inc(r2.Right,pt.X);
    inc(r2.Bottom,pt.Y-sb.Position);
    if (X > r2.Left) and (X < r2.Right) and (Y > r2.Top) and (Y < r2.Bottom) then
    begin
      ncc := ii;
      rNew := r2;
    end;
    if ii = CChar then rOld := r2;
    inc(pt.X,r1.Right-r1.Left+10);
  end;
  if ncc <> CChar then
  begin
    if CChar >= 0 then
    begin
      pt.X := rOld.Left; pt.Y := rOld.Top;
      Dev.CharTable[CChar].DrawTo(Canvas,pt,false);
    end;
    if ncc >= 0 then
    begin
      pt.X := rNew.Left; pt.Y := rNew.Top;
      Dev.CharTable[ncc].DrawTo(Canvas,pt,true);
    end;
    CChar := ncc;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmFontEditor.FormCreate(Sender: TObject);
begin
  DoubleBuffered := true;
  CChar := -1;
end;

// -----------------------------------------------------------------------------

procedure TfrmFontEditor.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if CChar >= 0 then
  begin
    EditChar(CChar);
    Dev.Modified := true;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmFontEditor.EditChar(AChar : integer);
begin
  OpenCharEditor(Dev.CharTable[AChar],Handle,Dev.GetMaxCharSize);
  Invalidate;
end;

// -----------------------------------------------------------------------------

end.

