unit LEDs;

interface

uses
  Windows, Graphics;

type
  TLED = class
    constructor Create;
  public
    Color : TColorRef;
    Brightness : byte;
    Active : boolean;
    Highlighted : boolean;
    LastRect : TRect;
    procedure DrawTo(CC : TCanvas; Rect : TRect; Highlighted : boolean = false);
    procedure Assign(ALED : TLED);
  end;

implementation

uses
  Constants, Types;

// ========== TLED =============================================================

constructor TLED.Create;
begin
  inherited Create;
  Color := clRed;
  Active := false;
  Highlighted := false;
  Brightness := $04;  // Medium
end;

// -----------------------------------------------------------------------------

procedure TLED.DrawTo(CC : TCanvas; Rect : TRect; Highlighted : boolean = false);
var
  ident1 : integer;
  ident2 : integer;
  ident3 : integer;
  rect1  : TRect;
  rect2  : TRect;
  rect3  : TRect;
  // ---------------------------------------------------------
  function IdentRect(ARect : TRect; AIdent : integer) : TRect;
  begin
    result.Top := ARect.Top+AIdent;
    result.Bottom := ARect.Bottom-AIdent;
    result.Left := ARect.Left+AIdent;
    result.Right := ARect.Right-AIdent;
  end;
  // ---------------------------------------------------------
begin
  LastRect := Rect;
  ident1 := round((Rect.Right-Rect.Left)*0.05);
  ident2 := round((Rect.Right-Rect.Left)*0.1);
  ident3 := round((Rect.Right-Rect.Left)*0.18);
  rect1 := IdentRect(Rect,ident1);
  rect2 := IdentRect(Rect,ident2);
  rect3 := IdentRect(Rect,ident3);
  CC.Pen.Color := 0;
  if Highlighted or self.Highlighted then
  begin
    CC.Brush.Color := Dark(LED_HL_SCREEN_COLOR,0.4);
    CC.FillRect(Rect);
    CC.Pen.Width := 0;
    CC.Pen.Style := psDot;
    CC.Pen.Color := RGB(255,255,255);
    CC.Rectangle(rect1);
  end
  else
  begin
    CC.Brush.Color := Dark(LED_SCREEN_COLOR,0.2);
    CC.FillRect(Rect);
  end;
  CC.Pen.Style := psSolid;
  CC.Pen.Color := clBlack;
  CC.Pen.Width := 0;
  CC.Brush.Color := Dark(Color,0.4);
  CC.Ellipse(rect2);
  CC.Pen.Style := psDash;
  if Active then
  begin
    case Brightness of
      0: CC.Brush.Color := Dark(Color,0.2);
      1: CC.Brush.Color := Dark(Color,0.4);
      2: CC.Brush.Color := Dark(Color,0.5);
      3: CC.Brush.Color := Dark(Color,0.6);
      4: CC.Brush.Color := Dark(Color,0.7);
      5: CC.Brush.Color := Dark(Color,0.8);
      6: CC.Brush.Color := Dark(Color,0.9);
      7: CC.Brush.Color := Color;
    end;
  end
  else
  begin
    CC.Brush.Color := Dark(Color,0.2);
  end;
  CC.Ellipse(rect3);
end;

// -----------------------------------------------------------------------------

procedure TLED.Assign(ALED : TLED);
begin
  Color := ALED.Color;
  Brightness := ALED.Brightness;
  Active := ALED.Active;
end;

// -----------------------------------------------------------------------------

end.
