unit Palette;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, GenericCommand;

type
  TDragCmd = class (TDragControlObjectEx)
    constructor Create(AControl : TControl; ACmd : TGenericCommand); reintroduce; virtual;
    destructor Destroy; override;
  public
    FDragImages : TDragImageList;
    Cmd : TGenericCommand;
    function GetDragImages : TDragImageList; override;
  end;

type
  TfrmPalette = class(TForm)
    dg: TDrawGrid;
    procedure dgDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
    procedure dgMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure dgMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure dgMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure dgStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    CS : TCommandSet;
    StartX : integer;
    StartY : integer;
  public
    procedure OpenPalette(ACS : TCommandSet);
  end;

var
  frmPalette: TfrmPalette;

implementation

uses
  ImgList, Main, IniFiles, Constants, SerialIO;

{$R *.dfm}

// ========== TDragCmd =========================================================

constructor TDragCmd.Create(AControl : TControl; ACmd : TGenericCommand);
begin
  inherited Create(AControl);
  FDragImages := nil;
  Cmd := ACmd;
end;

// -----------------------------------------------------------------------------

destructor TDragCmd.Destroy;
begin
  if assigned(FDragImages) then FDragImages.Free;
  inherited Destroy;
end;

// -----------------------------------------------------------------------------

function TDragCmd.GetDragImages : TDragImageList;
var
  ii : integer;
begin
  FDragImages := TDragImageList.CreateSize(32,32);
  ii := FDragImages.Add(Cmd.Image,Cmd.Image);
  FDragImages.SetDragImage(ii,28,28);
  result := nil;
end;

// =============================================================================

procedure TfrmPalette.OpenPalette(ACS : TCommandSet);
var
  cfg : TIniFile;
begin
  CS := ACS;
  dg.RowCount := CS.Count;
  cfg := TIniFile.Create(IniFN);
  try
    Left := UnhexWord(cfg.ReadString('Palette','PositionX','008C'));
    Top := UnhexWord(cfg.ReadString('Palette','PositionY','00AA'));
    Width := UnhexWord(cfg.ReadString('Palette','Width','00F0'));
    Height := UnhexWord(cfg.ReadString('Palette','Height','0190'));
  finally
    cfg.Free;
  end;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmPalette.dgDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  tw : integer;
begin
  if ACol = 0 then
    dg.Canvas.Draw(Rect.Left+1,Rect.Top+1,CS[ARow].Image)
  else
  begin
    tw := dg.Canvas.TextWidth(CS[ARow].Description);
    if dg.ColWidths[ACol] < (tw+8) then
      dg.ColWidths[ACol] := tw+8
    else
      dg.Canvas.TextRect(Rect,Rect.Left+2,Rect.Top+10,CS[ARow].Description);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmPalette.dgMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Row,Col : integer;
  Sel : TGridRect;
begin
  dg.MouseToCell(X,Y,Col,Row);
  sel.Top := Row;
  sel.Bottom := Row;
  sel.Left := 0;
  sel.Right := 1;
  dg.Selection := sel;
  dg.Invalidate;
  StartX := X;
  StartY := Y;
end;

// -----------------------------------------------------------------------------

procedure TfrmPalette.FormShow(Sender: TObject);
begin
  StartX := -1;
  StartY := -1;
end;

// -----------------------------------------------------------------------------

procedure TfrmPalette.dgMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  StartX := -1;
  StartY := -1;
end;

// -----------------------------------------------------------------------------

procedure TfrmPalette.dgMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if (StartX < 0) or (StartY < 0) then exit;
  if ((X-StartX)*(X-StartX)+(Y-StartY)*(Y-StartY)) > 25 then
  begin
    StartX := -1;
    StartY := -1;
    dg.BeginDrag(true);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmPalette.dgStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  if (dg.Selection.Top < 0) or (dg.Selection.Top >= CS.Count) then
  begin
    EndDrag(false);
    exit;
  end;
  DragObject := TDragCmd.Create(dg,CS[dg.Selection.Top]);
end;

// -----------------------------------------------------------------------------

procedure TfrmPalette.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PostMessage(frmMain.Handle,MSG_CLOSE_EDITOR,0,0);
  frmMain.SavePositions(self);
end;

// -----------------------------------------------------------------------------

end.
