unit PixelEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, GenericCommand, PCB;

type
  TPixelAction = (paOn, paOff, paFlip);

type
  TfrmPixelEditor = class(TForm)
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;
    Label2: TLabel;
    procedure Button2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FCmd  : TGenericCommand;
    FAct  : TPixelAction;
    FUndo : TUndo;
    X     : byte;
    Y     : byte;
    hMouseDown : TMouseEvent;
    hMouseMove : TMouseMoveEvent;
    procedure LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  public
    procedure OpenEditor(ACmd : TGenericCommand; AAct : TPixelAction; AUndo : PUndo);
  end;

var
  frmPixelEditor: TfrmPixelEditor;

implementation

uses
  main, SerialIO;

{$R *.dfm}

// =============================================================================

procedure TfrmPixelEditor.OpenEditor(ACmd : TGenericCommand; AAct : TPixelAction; AUndo : PUndo);
var
  row,col : integer;
begin
  if not assigned(frmMain.Dev) then exit;
  FCmd := ACmd;
  FAct := AAct;
  for row:=0 to frmMain.Dev.PCB.RowCount-1 do
    for col:=0 to frmMain.Dev.PCB.ColCount-1 do
    begin
      FUndo[col,row] := AUndo[col,row];
      frmMain.Dev.PCB[col,row].Brightness := 7;
    end;
  X := UnhexByte(FCmd.Parameters['X']);
  Y := UnhexByte(FCmd.Parameters['Y']);
  if X >= frmMain.Dev.PCB.ColCount then X := 0;
  if Y >= frmMain.Dev.PCB.RowCount then Y := 0;
  if FAct = paOn then
  begin
    Label1.Caption := 'Click on a pixel to SET';
    frmMain.Dev.PCB[X,Y].Active := true;
  end
  else
  if FAct = paOff then
  begin
    Label1.Caption := 'Click on a pixel to CLEAR';
    frmMain.Dev.PCB[X,Y].Active := false;
  end
  else
  begin
    Label1.Caption := 'Click on a pixel to change its state';
    frmMain.Dev.PCB[X,Y].Active := not frmMain.Dev.PCB[X,Y].Active;
  end;
  Label2.Caption := 'X = '+IntToStr(X)+';  Y = '+IntToStr(Y);
  frmMain.Invalidate;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmPixelEditor.Button2Click(Sender: TObject);
begin
  Close;
end;

// -----------------------------------------------------------------------------

procedure TfrmPixelEditor.FormShow(Sender: TObject);
begin
  DisableAPP;
  hMouseDown := frmMain.OnMouseDown;
  hMouseMove := frmMain.OnMouseMove;
  frmMain.OnMouseDown := LocalMouseDown;
  frmMain.OnMouseMove := LocalMouseMove;
end;

// -----------------------------------------------------------------------------

procedure TfrmPixelEditor.FormHide(Sender: TObject);
var
  Row : integer;
  Col : integer;
begin
  frmMain.OnMouseDown := hMouseDown;
  frmMain.OnMouseMove := hMouseMove;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
      frmMain.Dev.PCB[Col,Row].Highlighted := false;
  EnableAPP;
  frmMain.SavePositions(self);
end;

// -----------------------------------------------------------------------------

procedure TfrmPixelEditor.LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Row : integer;
  Col : integer;
  Md  : boolean;
begin
  Md := false;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
    begin
      if (X > frmMain.Dev.PCB[Col,Row].LastRect.Left) and (X < frmMain.Dev.PCB[Col,Row].LastRect.Right) and
         (Y > frmMain.Dev.PCB[Col,Row].LastRect.Top) and (Y < frmMain.Dev.PCB[Col,Row].LastRect.Bottom) then
      begin
        if FAct = paOn then
        begin
          if not frmMain.Dev.PCB[Col,Row].Active then
          begin
            frmMain.Dev.PCB[Col,Row].Active := true;
            Md := true;
            self.X := Col;
            self.Y := Row;
          end;
        end
        else
        if FAct = paOff then
        begin
          if frmMain.Dev.PCB[Col,Row].Active then
          begin
            frmMain.Dev.PCB[Col,Row].Active := false;
            Md := true;
            self.X := Col;
            self.Y := Row;
          end;
        end
        else
        begin
          if frmMain.Dev.PCB[Col,Row].Active <> (not frmMain.Dev.PCB[Col,Row].Active) then
          begin
            frmMain.Dev.PCB[Col,Row].Active := not frmMain.Dev.PCB[Col,Row].Active;
            Md := true;
            self.X := Col;
            self.Y := Row;
          end;
        end;
      end
      else
      begin
        if frmMain.Dev.PCB[Col,Row].Active <> FUndo[Col,Row] then
        begin
          frmMain.Dev.PCB[Col,Row].Active := FUndo[Col,Row];
          Md := true;
        end;
      end;
    end;
  end;
  if Md then frmMain.Invalidate;
  Label2.Caption := 'X = '+IntToStr(self.X)+';  Y = '+IntToStr(self.Y);
end;

// -----------------------------------------------------------------------------

procedure TfrmPixelEditor.LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  Row : integer;
  Col : integer;
  Md  : boolean;
begin
  Md := false;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
    begin
      if (X > frmMain.Dev.PCB[Col,Row].LastRect.Left) and (X < frmMain.Dev.PCB[Col,Row].LastRect.Right) and
         (Y > frmMain.Dev.PCB[Col,Row].LastRect.Top) and (Y < frmMain.Dev.PCB[Col,Row].LastRect.Bottom) then
      begin
        if not frmMain.Dev.PCB[Col,Row].Highlighted then
        begin
          frmMain.Dev.PCB[Col,Row].Highlighted := true;
          Md := true;
        end;
      end
      else
      begin
        if frmMain.Dev.PCB[Col,Row].Highlighted then
        begin
          frmMain.Dev.PCB[Col,Row].Highlighted := false;
          Md := true;
        end;
      end;
    end;
  end;
  if Md then frmMain.Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmPixelEditor.Button1Click(Sender: TObject);
begin
  FCmd.Parameters['X'] := HexByte(X);
  FCmd.Parameters['Y'] := HexByte(Y);
  close;
end;

// -----------------------------------------------------------------------------

end.
