unit RowEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, GenericCommand;

type
  TfrmRowEditor = class(TForm)
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FCmd : TGenericCommand;
    FRow : integer;
    InsEdit : boolean;
    hMouseDown : TMouseEvent;
    hMouseMove : TMouseMoveEvent;
    procedure LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  public
    procedure OpenEditor(ACmd : TGenericCommand; ARow : integer);
  end;

var
  frmRowEditor: TfrmRowEditor;

implementation

{$R *.dfm}

uses
  main, PCB, LEDs;

// =============================================================================

procedure TfrmRowEditor.FormShow(Sender: TObject);
begin
  DisableAPP;
  hMouseDown := frmMain.OnMouseDown;
  hMouseMove := frmMain.OnMouseMove;
  frmMain.OnMouseDown := LocalMouseDown;
  frmMain.OnMouseMove := LocalMouseMove;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowEditor.FormHide(Sender: TObject);
var
  Row : TLEDRow;
  Col : integer;
begin
  frmMain.OnMouseDown := hMouseDown;
  frmMain.OnMouseMove := hMouseMove;
  Row := frmMain.Dev.PCB.LEDRows[FRow];
  for Col:=0 to Row.ColCount-1 do Row[Col].Highlighted := false;
  EnableAPP;
  frmMain.SavePositions(self);
end;

// -----------------------------------------------------------------------------

procedure TfrmRowEditor.OpenEditor(ACmd : TGenericCommand; ARow : integer);
var
  Row : TLEDRow;
  Col : integer;
  ss  : string;
begin
  if not assigned(frmMain.Dev) then exit;
  FCmd := ACmd;
  FRow := ARow;
  if (FRow < 0) or (FRow >= frmMain.Dev.PCB.RowCount) then exit;
  Row := frmMain.Dev.PCB.LEDRows[FRow];
  ss := trim(FCmd.Parameters['Row']);
  while length(ss) < Row.ColCount do ss := ss+'0';
  for Col:=0 to Row.ColCount-1 do
  begin
    Row[Col].Active := (ss[Col+1] = '1');
    Row[Col].Brightness := 7;
  end;
  frmMain.Invalidate;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowEditor.LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Row : TLEDRow;
  Col : integer;
  Md  : boolean;
begin
  Row := frmMain.Dev.PCB.LEDRows[FRow];
  Md := false;
  for Col:=0 to Row.ColCount-1 do
  begin
    if (X > Row[Col].LastRect.Left) and (X < Row[Col].LastRect.Right) and
       (Y > Row[Col].LastRect.Top) and (Y < Row[Col].LastRect.Bottom) then
    begin
      Row[Col].Active := not Row[Col].Active;
      Md := true;
    end;
  end;
  if Md then frmMain.Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowEditor.LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  Row : TLEDRow;
  Col : integer;
  Md  : boolean;
begin
  Row := frmMain.Dev.PCB.LEDRows[FRow];
  Md := false;
  for Col:=0 to Row.ColCount-1 do
  begin
    if (X > Row[Col].LastRect.Left) and (X < Row[Col].LastRect.Right) and
       (Y > Row[Col].LastRect.Top) and (Y < Row[Col].LastRect.Bottom) then
    begin
      if not Row[Col].Highlighted then
      begin
        Md := true;
        Row[Col].Highlighted := true;
      end;
    end
    else
    begin
      if Row[Col].Highlighted then
      begin
        Md := true;
        Row[Col].Highlighted := false;
      end;
    end;
  end;
  if Md then frmMain.Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowEditor.Button2Click(Sender: TObject);
begin
  Close;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowEditor.Button1Click(Sender: TObject);
var
  Row : TLEDRow;
  Col : integer;
  ss  : string;
begin
  Row := frmMain.Dev.PCB.LEDRows[FRow];
  ss := '';
  for Col:=0 to Row.ColCount-1 do
  begin
    if Row[Col].Active then
      ss := ss+'1'
    else
      ss := ss+'0';
  end;
  FCmd.Parameters['Row'] := ss;
  Close;
end;

// -----------------------------------------------------------------------------

end.
