unit RowShiftEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, GenericCommand, PCB;

type
  TfrmRowShiftEditor = class(TForm)
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FCmd       : TGenericCommand;
    FRow       : byte;
    FUndo      : PUndo;
    FToTheLeft : boolean;
    hMouseDown : TMouseEvent;
    hMouseMove : TMouseMoveEvent;
    procedure LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure SetupDisplay;
  public
    procedure OpenEditor(ACmd : TGenericCommand; AToTheLeft : boolean; AUndo : PUndo);
  end;

var
  frmRowShiftEditor: TfrmRowShiftEditor;

implementation

uses
  main, SerialIO;

{$R *.dfm}

// =============================================================================

procedure TfrmRowShiftEditor.OpenEditor(ACmd : TGenericCommand; AToTheLeft : boolean; AUndo : PUndo);
begin
  if not assigned(frmMain.Dev) then exit;
  FCmd := ACmd;
  FUndo := AUndo;
  FToTheLeft := AToTheLeft;
  FRow := UnhexByte(FCmd.Parameters['Row']);
  if (FRow >= frmMain.Dev.PCB.RowCount) then FRow := 0;
  SetupDisplay;
  frmMain.Invalidate;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowShiftEditor.SetupDisplay;
var
  Row : integer;
  Col : integer;
begin
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    if Row = FRow then
    begin
      if FToTheLeft then
      begin
        for Col:=0 to frmMain.Dev.PCB.ColCount-2 do
        begin
          frmMain.Dev.PCB[Col,Row].Active := FUndo[Col+1,Row];
          frmMain.Dev.PCB[Col,Row].Highlighted := false;
          if frmMain.Dev.PCB[Col,Row].Brightness = 0 then
            frmMain.Dev.PCB[Col,Row].Brightness := 7;
        end;
        frmMain.Dev.PCB[frmMain.Dev.PCB.ColCount-1,Row].Active := false;
        frmMain.Dev.PCB[frmMain.Dev.PCB.ColCount-1,Row].Highlighted := false;
        if frmMain.Dev.PCB[frmMain.Dev.PCB.ColCount-1,Row].Brightness = 0 then
          frmMain.Dev.PCB[frmMain.Dev.PCB.ColCount-1,Row].Brightness := 7;
      end
      else
      begin
        for Col:=frmMain.Dev.PCB.ColCount-1 downto 1 do
        begin
          frmMain.Dev.PCB[Col,Row].Active := FUndo[Col-1,Row];
          frmMain.Dev.PCB[Col,Row].Highlighted := false;
          if frmMain.Dev.PCB[Col,Row].Brightness = 0 then
            frmMain.Dev.PCB[Col,Row].Brightness := 7;
        end;
        frmMain.Dev.PCB[0,Row].Active := false;
        frmMain.Dev.PCB[0,Row].Highlighted := false;
        if frmMain.Dev.PCB[0,Row].Brightness = 0 then frmMain.Dev.PCB[0,Row].Brightness := 7;
      end;
    end
    else
    begin
      for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
      begin
        frmMain.Dev.PCB[Col,Row].Active := FUndo[Col,Row];
        frmMain.Dev.PCB[Col,Row].Highlighted := false;
        if frmMain.Dev.PCB[Col,Row].Brightness = 0 then
          frmMain.Dev.PCB[Col,Row].Brightness := 7;
      end;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowShiftEditor.FormShow(Sender: TObject);
begin
  DisableAPP;
  hMouseDown := frmMain.OnMouseDown;
  hMouseMove := frmMain.OnMouseMove;
  frmMain.OnMouseDown := LocalMouseDown;
  frmMain.OnMouseMove := LocalMouseMove;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowShiftEditor.FormHide(Sender: TObject);
var
  Row : integer;
  Col : integer;
begin
  frmMain.OnMouseDown := hMouseDown;
  frmMain.OnMouseMove := hMouseMove;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
      frmMain.Dev.PCB[Col,Row].Highlighted := false;
  frmMain.Invalidate;
  EnableAPP;
  frmMain.SavePositions(self);
end;

// -----------------------------------------------------------------------------

procedure TfrmRowShiftEditor.LocalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Row,Col : integer;
begin
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
    begin
      if (X > frmMain.Dev.PCB[Col,Row].LastRect.Left) and (X < frmMain.Dev.PCB[Col,Row].LastRect.Right) and
         (Y > frmMain.Dev.PCB[Col,Row].LastRect.Top) and (Y < frmMain.Dev.PCB[Col,Row].LastRect.Bottom) then
      begin
        FRow := Row;
        SetupDisplay;
        frmMain.Invalidate;
        exit;
      end;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowShiftEditor.LocalMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  Row  : integer;
  Col  : integer;
  Col1 : integer;
  Md   : boolean;
begin
  Md := false;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
    begin
      if frmMain.Dev.PCB[Col,Row].Highlighted then
      begin
        Md := true;
        for Col1:=0 to frmMain.Dev.PCB.ColCount-1 do
          frmMain.Dev.PCB[Col1,Row].Highlighted := false;
      end;
    end;
  end;
  for Row:=0 to frmMain.Dev.PCB.RowCount-1 do
  begin
    for Col:=0 to frmMain.Dev.PCB.ColCount-1 do
    begin
      if (X > frmMain.Dev.PCB[Col,Row].LastRect.Left) and (X < frmMain.Dev.PCB[Col,Row].LastRect.Right) and
         (Y > frmMain.Dev.PCB[Col,Row].LastRect.Top) and (Y < frmMain.Dev.PCB[Col,Row].LastRect.Bottom) then
      begin
        if not frmMain.Dev.PCB[Col,Row].Highlighted then
        begin
          Md := true;
          for Col1:=0 to frmMain.Dev.PCB.ColCount-1 do
            frmMain.Dev.PCB[Col1,Row].Highlighted := true;
        end;
        break;
      end;
    end;
  end;
  if Md then frmMain.Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowShiftEditor.Button2Click(Sender: TObject);
begin
  Close;
end;

// -----------------------------------------------------------------------------

procedure TfrmRowShiftEditor.Button1Click(Sender: TObject);
begin
  FCmd.Parameters['Row'] := HexByte(FRow);
  Close;
end;

// -----------------------------------------------------------------------------

end.
