unit SLV01Commands;

interface

uses
  GenericCommand, PCB, CharTable;

// 00000000 - Stop execution
type
  TSLV01Cmd_Stop = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

// 00000001 - Restart
type
  TSLV01Cmd_Restart = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

// 00000010 - Set all
type
  TSLV01Cmd_SetAll = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

// 00000011 - Clear all
type
  TSLV01Cmd_ClearAll = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

// 00000100 - Insert next 2 bytes horisontally from the top side
type
  TSLV01Cmd_InsFromTop = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 00000101 - Insert next 2 bytes horisontally from the bottom side
type
  TSLV01Cmd_InsFromBottom = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 00000110 - Set pixel. Next byte - pixel address xxxxyyyy
type
  TSLV01Cmd_SetPixel = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 00000111 - Clear pixel. Next byte - pixel address xxxxyyyy
type
  TSLV01Cmd_ClearPixel = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 00001xxx - Brightness level (000 - drivers off)
type
  TSLV01Cmd_SetBrightness = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 00010xxx - Scroll a row left (3 lsb = row number)
type
  TSLV01Cmd_ScrollRowLeft = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 00011xxx - Scroll a row right (3 lsb = row number)
type
  TSLV01Cmd_ScrollRowRight = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 010xxxxx - Insert from left
type
  TSLV01Cmd_InsFromLeft = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 011xxxxx - Insert from right
type
  TSLV01Cmd_InsFromRight = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

// 100xxxxx - Scroll next xxxxx characters
type
  TSLV01Cmd_ScrollMsg = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    procedure Reset; override;
    function GetEditType : integer; override;
  public
    CurrentChar : integer;
    CharCol : integer;
  end;

// 11xxxxxx - Delay (xxxxxx by 0.1) sec
type
  TSLV01Cmd_Delay = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    procedure Reset; override;
    function GetEditType : integer; override;
  public
    CValue : byte;
  end;

function TSLV01_LoadCommand(var Data; var Index : integer) : TGenericCommand;
function TSLV01_CreateCommand(ACmdText : string) : TGenericCommand;

// =============================================================================
// =============================================================================
// =============================================================================

implementation

uses
  SysUtils, SerialIO, Constants;

// =============================================================================

function TSLV01_CreateCommand(ACmdText : string) : TGenericCommand;
var
  ii : integer;
  bb : byte;
begin
  if ACmdText = '[STOP]' then
    result := TSLV01Cmd_Stop.Create
  else
  if ACmdText = '[RESTART]' then
    result := TSLV01Cmd_Restart.Create
  else
  if ACmdText = '[SETALL]' then
    result := TSLV01Cmd_SetAll.Create
  else
  if ACmdText = '[CLEARALL]' then
    result := TSLV01Cmd_ClearAll.Create
  else
  if copy(ACmdText,1,13) = '[INSFROMTOP]=' then
  begin
    result := TSLV01Cmd_InsFromTop.Create;
    ACmdText := copy(ACmdText,14,255);
    while length(ACmdText) < 10 do ACmdText := ACmdText+'0';
    result.Parameters['Row'] := copy(ACmdText,1,10);
  end
  else
  if copy(ACmdText,1,16) = '[INSFROMBOTTOM]=' then
  begin
    result := TSLV01Cmd_InsFromBottom.Create;
    ACmdText := copy(ACmdText,17,255);
    while length(ACmdText) < 10 do ACmdText := ACmdText+'0';
    result.Parameters['Row'] := copy(ACmdText,1,10);
  end
  else
  if copy(ACmdText,1,11) = '[SETPIXEL]=' then
  begin
    result := TSLV01Cmd_SetPixel.Create;
    ACmdText := copy(ACmdText,12,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['X'] := IntToStr((bb shr 4) and $f);
    result.Parameters['Y'] := IntToStr(bb and $f);
  end
  else
  if copy(ACmdText,1,13) = '[CLEARPIXEL]=' then
  begin
    result := TSLV01Cmd_ClearPixel.Create;
    ACmdText := copy(ACmdText,14,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['X'] := IntToStr((bb shr 4) and $f);
    result.Parameters['Y'] := IntToStr(bb and $f);
  end
  else
  if copy(ACmdText,1,13) = '[BRIGHTNESS]=' then
  begin
    result := TSLV01Cmd_SetBrightness.Create;
    ACmdText := copy(ACmdText,14,255);
    ii := StrToIntDef(ACmdText,6);
    if ii < 0 then ii := 0;
    if ii > 7 then ii := 7;
    result.Parameters['Level'] := IntToStr(ii);
  end
  else
  if copy(ACmdText,1,16) = '[SCROLLROWLEFT]=' then
  begin
    result := TSLV01Cmd_ScrollRowLeft.Create;
    ACmdText := copy(ACmdText,17,255);
    ii := StrToIntDef(ACmdText,0);
    if ii < 0 then ii := 0;
    if ii > 4 then ii := 4;
    result.Parameters['Row'] := IntToStr(ii);
  end
  else
  if copy(ACmdText,1,17) = '[SCROLLROWRIGHT]=' then
  begin
    result := TSLV01Cmd_ScrollRowRight.Create;
    ACmdText := copy(ACmdText,18,255);
    ii := StrToIntDef(ACmdText,0);
    if ii < 0 then ii := 0;
    if ii > 4 then ii := 4;
    result.Parameters['Row'] := IntToStr(ii);
  end
  else
  if copy(ACmdText,1,14) = '[INSFROMLEFT]=' then
  begin
    result := TSLV01Cmd_InsFromLeft.Create;
    ACmdText := copy(ACmdText,15,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Col'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,15) = '[INSFROMRIGHT]=' then
  begin
    result := TSLV01Cmd_InsFromRight.Create;
    ACmdText := copy(ACmdText,16,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Col'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,12) = '[SCROLLMSG]=' then
  begin
    result := TSLV01Cmd_ScrollMsg.Create;
    ACmdText := copy(ACmdText,13,length(ACmdText));
    result.Parameters['Msg'] := copy(ACmdText,1,31);
  end
  else
  if copy(ACmdText,1,8) = '[DELAY]=' then
  begin
    result := TSLV01Cmd_Delay.Create;
    ACmdText := copy(ACmdText,9,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Value'] := HexByte(bb);
  end
  else
    result := TSLV01Cmd_Restart.Create;
end;

// -----------------------------------------------------------------------------

function TSLV01_LoadCommand(var Data; var Index : integer) : TGenericCommand;
begin
  if TByteArray(Data)[Index] = $00 then
    result := TSLV01Cmd_Stop.Create
  else
  if TByteArray(Data)[Index] = $01 then
    result := TSLV01Cmd_Restart.Create
  else
  if TByteArray(Data)[Index] = $02 then
    result := TSLV01Cmd_SetAll.Create
  else
  if TByteArray(Data)[Index] = $03 then
    result := TSLV01Cmd_ClearAll.Create
  else
  if TByteArray(Data)[Index] = $04 then
    result := TSLV01Cmd_InsFromTop.Create
  else
  if TByteArray(Data)[Index] = $05 then
    result := TSLV01Cmd_InsFromBottom.Create
  else
  if TByteArray(Data)[Index] = $06 then
    result := TSLV01Cmd_SetPixel.Create
  else
  if TByteArray(Data)[Index] = $07 then
    result := TSLV01Cmd_ClearPixel.Create
  else
  if (TByteArray(Data)[Index] and $f8) = $08 then
    result := TSLV01Cmd_SetBrightness.Create
  else
  if (TByteArray(Data)[Index] and $f8) = $10 then
    result := TSLV01Cmd_ScrollRowLeft.Create
  else
  if (TByteArray(Data)[Index] and $f8) = $18 then
    result := TSLV01Cmd_ScrollRowRight.Create
  else
  if (TByteArray(Data)[Index] and $e0) = $40 then
    result := TSLV01Cmd_InsFromLeft.Create
  else
  if (TByteArray(Data)[Index] and $e0) = $60 then
    result := TSLV01Cmd_InsFromRight.Create
  else
  if (TByteArray(Data)[Index] and $e0) = $80 then
    result := TSLV01Cmd_ScrollMsg.Create
  else
  if (TByteArray(Data)[Index] and $c0) = $c0 then
    result := TSLV01Cmd_Delay.Create
  else
    result := TSLV01Cmd_Restart.Create;
  result.Load(Data,Index);
end;

// ========== TSLV01Cmd_Stop ===================================================

constructor TSLV01Cmd_Stop.Create;
begin
  inherited Create;
  ImgName := 'CI_STOP';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command stops the script''s execution flow.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Stop.GetCmdText : string;
begin
  result := '[STOP]';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Stop.Load(var Data; var Index : integer) : integer;
begin
  // No additional bytes are needed
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_Stop.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $00;
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Stop.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Stop.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_Stop.Create;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Stop.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV01Cmd_Stop then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Stop.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  // Script will be stuck on this command
  result := 1;
end;

// ========== TSLV01Cmd_Restart ================================================

constructor TSLV01Cmd_Restart.Create;
begin
  inherited Create;
  ImgName := 'CI_RESTART';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command restarts the script''s so the execution continues from the first script''s command.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Restart.GetCmdText : string;
begin
  result := '[RESTART]';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Restart.Load(var Data; var Index : integer) : integer;
begin
  // No additional bytes are needed
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_Restart.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $01;
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Restart.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Restart.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_Restart.Create;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Restart.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV01Cmd_Restart then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Restart.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  // The following result forces the TCommandSet object to reset all the commands
  // and restart the execution flow from the first one
  result := -1;
end;

// ========== TSLV01Cmd_SetAll =================================================

constructor TSLV01Cmd_SetAll.Create;
begin
  inherited Create;
  ImgName := 'CI_SETALL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command switches all the LEDs to the "ON" state.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetAll.GetCmdText : string;
begin
  result := '[SETALL]';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetAll.Load(var Data; var Index : integer) : integer;
begin
  // No additional bytes are needed
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_SetAll.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $02; // 00000010
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetAll.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetAll.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_SetAll.Create;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetAll.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV01Cmd_SetAll then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetAll.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : integer;
begin
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-1 do
      PCB[col,row].Active := true;
  // This command has finished
  result := 2;
end;

// ========== TSLV01Cmd_ClearAll ===============================================

constructor TSLV01Cmd_ClearAll.Create;
begin
  inherited Create;
  ImgName := 'CI_CLEARALL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command switches all the LEDs to the "OFF" state.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearAll.GetCmdText : string;
begin
  result := '[CLEARALL]';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearAll.Load(var Data; var Index : integer) : integer;
begin
  // No additional bytes are needed
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_ClearAll.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $03; // 00000011
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearAll.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearAll.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_ClearAll.Create;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearAll.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV01Cmd_ClearAll then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearAll.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : integer;
begin
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-1 do
      PCB[col,row].Active := false;
  // This command has finished
  result := 2;
end;

// ========== TSLV01Cmd_InsFromTop =============================================

constructor TSLV01Cmd_InsFromTop.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMTOP';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts the image down one row and then sets the top row according to the commnd''s data.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromTop.GetCmdText : string;
var
  ss  : string;
begin
  ss := trim(Parameters['Row']);
  while length(ss) < 10 do ss := ss+'0';
  result := '[INSFROMTOP]='+ss;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromTop.Load(var Data; var Index : integer) : integer;
var
  Row : word;
  ii  : integer;
  ss  : string;
begin
  inc(Index);
  Row := (TByteArray(Data)[Index] shl 8);
  inc(Index);
  Row := (Row or TByteArray(Data)[Index]);
  inc(Index);
  ss := '';
  for ii:=0 to 9 do
  begin
    if (($8000 shr ii) and Row) = 0 then
      ss := ss+'0'
    else
      ss := ss+'1';
  end;
  Parameters['Row'] := ss;
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_InsFromTop.Save(var Data; var Index : integer);
var
  Row : word;
  ss  : string;
  ii  : integer;
begin
  Row := 0;
  ss := trim(Parameters['Row']);
  while length(ss) < 10 do ss := ss+'0';
  for ii:=0 to 9 do
    if ss[ii+1] = '1' then
      Row := Row or ($8000 shr ii);
  TByteArray(Data)[Index] := $04; // 00000100
  inc(Index);
  TByteArray(Data)[Index] := hi(Row);
  inc(Index);
  TByteArray(Data)[Index] := lo(Row);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromTop.Size : integer;
begin
  result := 3;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromTop.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_InsFromTop.Create;
  TSLV01Cmd_InsFromTop(result).Parameters['Row'] := Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromTop.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_InsFromTop then
  begin
    if ((ACmd as TSLV01Cmd_InsFromTop).Parameters['Row'] = Parameters['Row']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromTop.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : word;
  ss  : string;
  ii  : integer;
begin
  for row:=PCB.RowCount-1 downto 1 do
    PCB.LEDRows[row].Assign(PCB.LEDRows[row-1]);
  Row := 0;
  ss := trim(Parameters['Row']);
  while length(ss) < 10 do ss := ss+'0';
  for ii:=0 to 9 do
    if ss[ii+1] = '1' then
      Row := Row or ($8000 shr ii);
  for col:=0 to PCB.ColCount-1 do
  begin
    PCB[col,0].Active := ((Row and ($8000 shr col)) <> 0)
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromTop.GetEditType : integer;
begin
  result := ED_TOP_ROW;
end;

// ========== TSLV01Cmd_InsFromBottom ==========================================

constructor TSLV01Cmd_InsFromBottom.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMBOTTOM';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts the image up one row and then sets the bottom row according to the commnd''s data.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromBottom.GetCmdText : string;
var
  ss  : string;
begin
  ss := trim(Parameters['Row']);
  while length(ss) < 10 do ss := ss+'0';
  result := '[INSFROMBOTTOM]='+ss;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromBottom.Load(var Data; var Index : integer) : integer;
var
  Row : word;
  ii  : integer;
  ss  : string;
begin
  inc(Index);
  Row := (TByteArray(Data)[Index] shl 8);
  inc(Index);
  Row := (Row or TByteArray(Data)[Index]);
  inc(Index);
  ss := '';
  for ii:=0 to 9 do
  begin
    if (($8000 shr ii) and Row) = 0 then
      ss := ss+'0'
    else
      ss := ss+'1';
  end;
  Parameters['Row'] := ss;
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_InsFromBottom.Save(var Data; var Index : integer);
var
  Row : word;
  ss  : string;
  ii  : integer;
begin
  Row := 0;
  ss := trim(Parameters['Row']);
  while length(ss) < 10 do ss := ss+'0';
  for ii:=0 to 9 do
    if ss[ii+1] = '1' then
      Row := Row or ($8000 shr ii);
  TByteArray(Data)[Index] := $05; // 00000101
  inc(Index);
  TByteArray(Data)[Index] := hi(Row);
  inc(Index);
  TByteArray(Data)[Index] := lo(Row);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromBottom.Size : integer;
begin
  result := 3;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromBottom.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_InsFromBottom.Create;
  TSLV01Cmd_InsFromBottom(result).Parameters['Row'] := Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromBottom.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_InsFromBottom then
  begin
    if ((ACmd as TSLV01Cmd_InsFromBottom).Parameters['Row'] = Parameters['Row']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromBottom.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : integer;
  ss  : string;
  ii  : integer;
begin
  for row:=0 to PCB.RowCount-2 do
    PCB.LEDRows[row].Assign(PCB.LEDRows[row+1]);
  Row := 0;
  ss := trim(Parameters['Row']);
  while length(ss) < 10 do ss := ss+'0';
  for ii:=0 to 9 do
    if ss[ii+1] = '1' then
      Row := Row or ($8000 shr ii);
  for col:=0 to PCB.ColCount-1 do
  begin
    PCB[col,PCB.RowCount-1].Active := ((Row and ($8000 shr col)) <> 0)
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromBottom.GetEditType : integer;
begin
  result := ED_BOTTOM_ROW;
end;

// ========== TSLV01Cmd_SetPixel ===============================================

constructor TSLV01Cmd_SetPixel.Create;
begin
  inherited Create;
  ImgName := 'CI_SETPIXEL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets an individual pixel to the "ON" state.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetPixel.GetCmdText : string;
var
  b : byte;
  X,Y : byte;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  b := ((X shl 4) and $f0)+(Y and $0f);
  result := '[SETPIXEL]='+HexByte(b);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetPixel.Load(var Data; var Index : integer) : integer;
var
  b : byte;
begin
  inc(Index);
  b := TByteArray(Data)[Index];
  inc(Index);
  Parameters['X'] := HexByte((b shr 4) and $0f);
  Parameters['Y'] := HexByte(b and $0f);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_SetPixel.Save(var Data; var Index : integer);
var
  b : byte;
  X,Y : byte;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  b := ((X shl 4) and $f0)+(Y and $0f);
  TByteArray(Data)[Index] := $06; // 00000110
  inc(Index);
  TByteArray(Data)[Index] := b;
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetPixel.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetPixel.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_SetPixel.Create;
  TSLV01Cmd_SetPixel(result).Parameters['X'] := Parameters['X'];
  TSLV01Cmd_SetPixel(result).Parameters['Y'] := Parameters['Y'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetPixel.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_SetPixel then
  begin
    if ((ACmd as TSLV01Cmd_SetPixel).Parameters['X'] = Parameters['X']) and
       ((ACmd as TSLV01Cmd_SetPixel).Parameters['Y'] = Parameters['Y']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetPixel.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  X,Y : byte;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  if assigned(PCB[X,Y]) then PCB[X,Y].Active := true;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetPixel.GetEditType : integer;
begin
  result := ED_SET_PIXEL;
end;

// ========== TSLV01Cmd_ClearPixel =============================================

constructor TSLV01Cmd_ClearPixel.Create;
begin
  inherited Create;
  ImgName := 'CI_CLEARPIXEL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets an individual pixel to the "OFF" state.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearPixel.GetCmdText : string;
var
  b : byte;
  X,Y : byte;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  b := ((X shl 4) and $f0)+(Y and $0f);
  result := '[CLEARPIXEL]='+HexByte(b);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearPixel.Load(var Data; var Index : integer) : integer;
var
  b : byte;
begin
  inc(Index);
  b := TByteArray(Data)[Index];
  inc(Index);
  Parameters['X'] := HexByte((b shr 4) and $0f);
  Parameters['Y'] := HexByte(b and $0f);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_ClearPixel.Save(var Data; var Index : integer);
var
  b : byte;
  X,Y : byte;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  b := ((X shl 4) and $f0)+(Y and $0f);
  TByteArray(Data)[Index] := $07; // 00000111
  inc(Index);
  TByteArray(Data)[Index] := b;
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearPixel.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearPixel.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_ClearPixel.Create;
  TSLV01Cmd_ClearPixel(result).Parameters['X'] := Parameters['X'];
  TSLV01Cmd_ClearPixel(result).Parameters['Y'] := Parameters['Y'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearPixel.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_ClearPixel then
  begin
    if ((ACmd as TSLV01Cmd_ClearPixel).Parameters['X'] = Parameters['X']) and
       ((ACmd as TSLV01Cmd_ClearPixel).Parameters['Y'] = Parameters['Y']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearPixel.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  X,Y : byte;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  if assigned(PCB[X,Y]) then PCB[X,Y].Active := false;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ClearPixel.GetEditType : integer;
begin
  result := ED_CLR_PIXEL;
end;

// ========== TSLV01Cmd_SetBrightness ==========================================

constructor TSLV01Cmd_SetBrightness.Create;
begin
  inherited Create;
  ImgName := 'CI_BRIGHTNESS';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets the brightness level for all the pixels.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetBrightness.GetCmdText : string;
begin
  result := '[BRIGHTNESS]='+Parameters['Level'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetBrightness.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Level'] := HexByte(TByteArray(Data)[Index] and $07);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_SetBrightness.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($08 or (UnhexByte(Parameters['Level']) and $07)); // 00001xxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetBrightness.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetBrightness.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_SetBrightness.Create;
  TSLV01Cmd_SetBrightness(result).Parameters['Level'] := Parameters['Level'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetBrightness.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_SetBrightness then
  begin
    if ((ACmd as TSLV01Cmd_SetBrightness).Parameters['Level'] = Parameters['Level']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetBrightness.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
begin
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-1 do
      PCB[col,row].Brightness := (UnhexByte(Parameters['Level']) and $07);
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_SetBrightness.GetEditType : integer;
begin
  result := ED_BRIGHTNESS;
end;

// ========== TSLV01Cmd_ScrollRowLeft ==========================================

constructor TSLV01Cmd_ScrollRowLeft.Create;
begin
  inherited Create;
  ImgName := 'CI_SCROLLROWLEFT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display row one pixel to the left.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowLeft.GetCmdText : string;
begin
  result := '[SCROLLROWLEFT]='+Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowLeft.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Row'] := HexByte(TByteArray(Data)[Index] and $07);
  if UnhexByte(Parameters['Row']) > 4 then Parameters['Row'] := '04';
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_ScrollRowLeft.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($10 or (UnhexByte(Parameters['Row']) and $07)); // 00010xxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowLeft.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowLeft.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_ScrollRowLeft.Create;
  TSLV01Cmd_ScrollRowLeft(result).Parameters['Row'] := Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowLeft.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_ScrollRowLeft then
  begin
    if ((ACmd as TSLV01Cmd_ScrollRowLeft).Parameters['Row'] = Parameters['Row']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowLeft.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col : integer;
  Row : byte;
begin
  Row := UnhexByte(Parameters['Row']);
  if assigned(PCB.LEDRows[Row]) then
  begin
    for col:=0 to PCB.ColCount-2 do
      PCB.LEDRows[Row][col].Active := PCB.LEDRows[Row][col+1].Active;
    PCB.LEDRows[Row][PCB.ColCount-1].Active := false;
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowLeft.GetEditType : integer;
begin
  result := ED_ROW_LEFT;
end;

// ========== TSLV01Cmd_ScrollRowRight =========================================

constructor TSLV01Cmd_ScrollRowRight.Create;
begin
  inherited Create;
  ImgName := 'CI_SCROLLROWRIGHT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display row one pixel to the right.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowRight.GetCmdText : string;
begin
  result := '[SCROLLROWRIGHT]='+Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowRight.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Row'] := HexByte(TByteArray(Data)[Index] and $07);
  if UnhexByte(Parameters['Row']) > 4 then Parameters['Row'] := '04';
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_ScrollRowRight.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($18 or (UnhexByte(Parameters['Row']) and $07)); // 00011xxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowRight.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowRight.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_ScrollRowRight.Create;
  TSLV01Cmd_ScrollRowRight(result).Parameters['Row'] := Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowRight.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_ScrollRowRight then
  begin
    if ((ACmd as TSLV01Cmd_ScrollRowRight).Parameters['Row'] = Parameters['Row']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowRight.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col : integer;
  Row : byte;
begin
  Row := UnhexByte(Parameters['Row']);
  if assigned(PCB.LEDRows[Row]) then
  begin
    for col:=PCB.ColCount-1 downto 1 do
      PCB.LEDRows[Row][col].Active := PCB.LEDRows[Row][col-1].Active;
    PCB.LEDRows[Row][0].Active := false;
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollRowRight.GetEditType : integer;
begin
  result := ED_ROW_RIGHT;
end;

// ========== TSLV01Cmd_InsFromLeft ============================================

constructor TSLV01Cmd_InsFromLeft.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMLEFT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display to the right and inserts a column, defined by te script''s data into the leftmost display column.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromLeft.GetCmdText : string;
begin
  result := '[INSFROMLEFT]='+Parameters['Col'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromLeft.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Col'] := HexByte(TByteArray(Data)[Index] and $1f);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_InsFromLeft.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($40 or (UnhexByte(Parameters['Col']) and $1f)); // 010xxxxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromLeft.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromLeft.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_InsFromLeft.Create;
  TSLV01Cmd_InsFromLeft(result).Parameters['Col'] := Parameters['Col'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromLeft.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_InsFromLeft then
  begin
    if ((ACmd as TSLV01Cmd_InsFromLeft).Parameters['Col'] = Parameters['Col']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromLeft.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
  b : byte;
begin
  // Shift the display
  for row:=0 to PCB.RowCount-1 do
    for col:=PCB.ColCount-1 downto 1 do
      PCB[col,row].Active := PCB[col-1,row].Active;
  // Set the leftmost column
  b := 1;
  for row:=PCB.RowCount-1 downto 0 do
  begin
    PCB[0,Row].Active := ((UnhexByte(Parameters['Col']) and b) <> 0);
    b := b shl 1;
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromLeft.GetEditType : integer;
begin
  result := ED_INS_FROM_LEFT;
end;

// ========== TSLV01Cmd_InsFromRight ===========================================

constructor TSLV01Cmd_InsFromRight.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMRIGHT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display to the left and inserts a column, defined by te script''s data into the rightmost display column.';
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromRight.GetCmdText : string;
begin
  result := '[INSFROMRIGHT]='+Parameters['Col'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromRight.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Col'] := HexByte(TByteArray(Data)[Index] and $1f);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_InsFromRight.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($60 or (UnhexByte(Parameters['Col']) and $1f)); // 011xxxxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromRight.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromRight.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_InsFromRight.Create;
  TSLV01Cmd_InsFromRight(result).Parameters['Col'] := Parameters['Col'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromRight.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_InsFromRight then
  begin
    if ((ACmd as TSLV01Cmd_InsFromRight).Parameters['Col'] = Parameters['Col']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromRight.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
  b : byte;
begin
  // Shift the display
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-2 do
      PCB[col,row].Active := PCB[col+1,row].Active;
  // Set the rightmost column
  b := 1;
  for row:=PCB.RowCount-1 downto 0 do
  begin
    PCB[PCB.ColCount-1,Row].Active := ((UnhexByte(Parameters['Col']) and b) <> 0);
    b := b shl 1;
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_InsFromRight.GetEditType : integer;
begin
  result := ED_INS_FROM_RIGHT;
end;

// ========== TSLV01Cmd_ScrollMsg ==============================================

constructor TSLV01Cmd_ScrollMsg.Create;
begin
  inherited Create;
  ImgName := 'CI_SCROLLMSG';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command scrolls a message.';
  Parameters['Msg'] := '';
  CurrentChar := 1;
  CharCol := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollMsg.GetCmdText : string;
begin
  result := '[SCROLLMSG]='+Parameters['Msg'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollMsg.Load(var Data; var Index : integer) : integer;
var
  len : byte;
  ss  : string;
begin
  len := (TByteArray(Data)[Index] and $1f);
  inc(Index);
  ss := '';
  while len > 0 do
  begin
    ss := ss+char(TByteArray(Data)[Index]);
    inc(Index);
    dec(len);
  end;
  CurrentChar := 1;
  CharCol := 0;
  Parameters['Msg'] := EscapeChars(ss);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_ScrollMsg.Save(var Data; var Index : integer);
var
  len : byte;
  ii  : byte;
  ss  : string;
begin
  ss := UnescapeChars(Parameters['Msg']);
  len := length(ss);
  TByteArray(Data)[Index] := ($80 or (len and $1f)); // 100xxxxx
  inc(Index);
  for ii:=1 to len do
  begin
    TByteArray(Data)[Index] := byte(ss[ii]);
    inc(Index);
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollMsg.Size : integer;
begin
  result := length(UnescapeChars(Parameters['Msg']))+1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollMsg.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_ScrollMsg.Create;
  TSLV01Cmd_ScrollMsg(result).Parameters['Msg'] := Parameters['Msg'];
  TSLV01Cmd_ScrollMsg(result).CurrentChar := 1;
  TSLV01Cmd_ScrollMsg(result).CharCol := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollMsg.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_ScrollMsg then
  begin
    if ((ACmd as TSLV01Cmd_ScrollMsg).Parameters['Msg'] = Parameters['Msg']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollMsg.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
  b1,b2 : byte;
  ss : string;
begin
  result := 3;
  ss := UnescapeChars(Parameters['Msg']);
  if CurrentChar > length(ss) then
  begin
    result := 0;
    exit;
  end;
  if not NoDelay then sleep(lo(PCB.Tag1)*SLV01_DELAY_INC);
  // Shift the display
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-2 do
      PCB[col,row].Active := PCB[col+1,row].Active;
  if not assigned(CharTable[byte(ss[CurrentChar])]) or (CharCol >= length(CharTable[byte(ss[CurrentChar])].Data)) then
  begin
    b2 := 0;
    inc(CurrentChar);
    CharCol := 0;
    if CurrentChar > length(ss) then
    begin
      CharCol := 0;
      CurrentChar := 1;
      result := 2;
    end;
  end
  else
  begin
    b2 := CharTable[byte(ss[CurrentChar])].Data[CharCol];
    inc(CharCol);
  end;
  // Set the rightmost column
  b1 := 1;
  for row:=PCB.RowCount-1 downto 0 do
  begin
    PCB[PCB.ColCount-1,Row].Active := ((b2 and b1) <> 0);
    b1 := b1 shl 1;
  end;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_ScrollMsg.Reset;
begin
  CharCol := 0;
  CurrentChar := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_ScrollMsg.GetEditType : integer;
begin
  result := ED_SIMPLE_MSG;
end;

// ========== TSLV01Cmd_Delay ==================================================

constructor TSLV01Cmd_Delay.Create;
begin
  inherited Create;
  ImgName := 'CI_DELAY';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command pauses the script for the specified time.';
  Parameters['Value'] := '06';
  CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Delay.GetCmdText : string;
begin
  result := '[DELAY]='+Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Delay.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Value'] := HexByte(TByteArray(Data)[Index] and $3f);
  CValue := 0;
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_Delay.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($c0 or (UnhexByte(Parameters['Value']) and $3f)); // 11xxxxxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Delay.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Delay.Clone : TGenericCommand;
begin
  result := TSLV01Cmd_Delay.Create;
  TSLV01Cmd_Delay(result).Parameters['Value'] := Parameters['Value'];
  TSLV01Cmd_Delay(result).CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Delay.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV01Cmd_Delay then
  begin
    if ((ACmd as TSLV01Cmd_Delay).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Delay.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  bb : byte;
begin
  PCB.Tag1 := UnhexByte(Parameters['Value']);
  if not NoDelay then
  begin
    bb := UnhexByte(Parameters['Value']);
    sleep(bb*SLV01_DELAY_INC);
  end;
  result := 0;
end;

// -----------------------------------------------------------------------------

procedure TSLV01Cmd_Delay.Reset;
begin
  CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV01Cmd_Delay.GetEditType : integer;
begin
  result := ED_SET_DELAY;
end;

// -----------------------------------------------------------------------------

end.

