unit SLV11;

// SL0011000228002305

interface

uses
  GenericDevice, GenericCommand, PCB, Types;

type
  TSLV11 = class(TGenericDevice)
    constructor Create(APort : string; AIDString : string);
  private
    OldState : byte;
  public
    function ReadDevice : integer; override;
    function WriteDevice : integer; override;
    function ReadInfo : integer; override;
    procedure Reset; override;
    procedure UpdateActiveScript; override;
    function Run(NoDelay : boolean; CS : TCommandSet;  Cmd : TGenericCommand) : TUndo; override;
    function WriteTo(FileName : string; ID : integer) : integer; overload; override;
    function WriteTo(FileName : string; Scr : TCommandSet) : integer; overload; override;
    function ReadFrom(FileName : string; ID : integer) : integer; overload; override;
    function ReadFrom(FileName : string; Scr : TCommandSet) : integer; overload; override;
    function GetCharTblLimits : TPoint; override;
    function GetMaxCharSize : integer; override;
    function CmdFromText(ACmdText : string) : TGenericCommand; override;
  end;

implementation

uses
  SysUtils, SerialIO, Dialogs, Constants, SLV11Commands, CharTable, Inputs,
  Main, Classes, Windows;

// Default scripts
var
  // Normal operations script - no signals detected
  Script_00000 : array [1..10] of byte =
    ($01,$ce,$ff,$ce,$ff,$ce,$ff,$c2,$06,$00);

  // Stop script
  Script_00001 : array[1..54] of byte =
   ($68,$01,$42,$cd,$01,$44,$46,$cb,$05,byte('S'),byte('T'),byte('O'),byte('P'),
    byte(' '),$6f,$cc,$08,$68,$cc,$10,$6f,$cc,$08,$68,$cc,$10,$6f,$cc,$08,$68,
    $cc,$10,$6f,$cc,$08,$68,$cc,$10,$6f,$cc,$08,$68,$cc,$10,$6f,$cc,$08,$68,$cc,
    $10,$02,$6d,$c3,$00);

  // Left turn script
  Script_00010 : array [1..192] of byte =
    ($01,$6a,$c0,$04,$04,$cc,$01,$c0,$0e,$04,$cc,$01,$c0,$1f,$04,
     $cc,$01,$c0,$0e,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,
     $01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,
     $01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$c0,$00,$04,$cc,$01,
     $04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,
     $04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,
     $04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,
     $04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$6b,$cc,$0a,$6c,$cc,
     $0a,$6d,$cc,$0a,$6e,$cc,$0a,$6f,$ce,$05,$6e,$cc,$14,$6d,$cc,
     $14,$6c,$cc,$14,$6b,$cc,$14,$6a,$04,$cc,$01,$04,$cc,$01,$04,
     $cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,
     $cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$04,
     $cc,$01,$04,$cc,$01,$04,$cc,$01,$04,$cc,$01,$00);

  // Right turn script
  Script_00011 : array [1..192] of byte =
    ($01,$6a,$c0,$04,$05,$cc,$01,$c0,$0e,$05,$cc,$01,$c0,$1f,$05,
     $cc,$01,$c0,$0e,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,
     $01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,
     $01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$c0,$00,$05,$cc,$01,
     $05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,
     $05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,
     $05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,
     $05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$6b,$cc,$0a,$6c,$cc,
     $0a,$6d,$cc,$0a,$6e,$cc,$0a,$6f,$ce,$05,$6e,$cc,$14,$6d,$cc,
     $14,$6c,$cc,$14,$6b,$cc,$14,$6a,$05,$cc,$01,$05,$cc,$01,$05,
     $cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,
     $cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$05,
     $cc,$01,$05,$cc,$01,$05,$cc,$01,$05,$cc,$01,$00);

  // Emergency signal script
  Script_00100 : array [1..56] of byte =
    ($6f,$02,$ce,$05,$01,$ce,$03,$02,$ce,$05,$01,$ce,$03,$02,$ce,
     $05,$01,$ce,$0a,$02,$ce,$0a,$01,$ce,$03,$02,$ce,$0a,$01,$ce,
     $03,$02,$ce,$0a,$01,$ce,$0a,$02,$ce,$05,$01,$ce,$03,$02,$ce,
     $05,$01,$ce,$03,$02,$ce,$05,$01,$ce,$96,$00);

  // Reverse script
  Script_00101 : array [1..46] of byte =
    ($68,$01,$41,$cd,$01,$44,$46,$cb,$06,byte('W'),byte('A'),byte('T'),
     byte('C'),byte('H'),byte(' '),$6f,$cc,$30,$6d,$ce,$14,$68,$01,$41,
     $cd,$01,$44,$46,$cb,$05,byte('O'),byte('U'),byte('T'),byte(' '),
     byte(' '),$c0,$00,$04,$04,$6f,$cc,$30,$6d,$ce,$28,$00);

  // Li'l advertising
  Script_00110 : array [1..77] of byte =
   ($6f,$cd,$0c,$41,$44,$46,$cb,$2f,byte('C'),byte('U'),byte('S'),byte('T'),
    byte('O'),byte('M'),byte('I'),byte('Z'),byte('A'),byte('B'),byte('L'),
    byte('E'),byte(' '),byte('L'),byte('I'),byte('G'),byte('T'),byte('S'),
    byte(' '),byte('-'),byte(' '),byte('W'),byte('W'),byte('W'),byte('.'),
    byte('B'),byte('O'),byte('L'),byte('D'),byte('L'),byte('I'),byte('N'),
    byte('E'),byte('.'),byte('O'),byte('R'),byte('G'),byte(' '),byte(' '),
    byte(' '),byte(' '),byte(' '),byte(' '),byte(' '),byte(' '),byte(' '),
    byte(' '),$68,$ce,$ff,$ce,$ff,$ce,$ff,$ce,$ff,$ce,$ff,$ce,$ff,$ce,$ff,$ce,
    $ff,$ce,$ff,$ce,$ff,$00);

  Script_00111 : array [1..2] of byte = ($00,$00);

  Script_01000 : array [1..2] of byte = ($00,$00);

  Script_01001 : array [1..2] of byte = ($00,$00);

  Script_01010 : array [1..2] of byte = ($00,$00);

  Script_01011 : array [1..2] of byte = ($00,$00);

  Script_01100 : array [1..2] of byte = ($00,$00);

  Script_01101 : array [1..2] of byte = ($00,$00);

  Script_01110 : array [1..2] of byte = ($00,$00);

  Script_01111 : array [1..2] of byte = ($01,$c3);

  Script_10000 : array [1..26] of byte =
    ($6f,$02,$ce,$06,$6e,$ce,$06,$6d,$ce,$06,$6c,$ce,$06,$6b,$ce,$06,$6a,$ce,
     $06,$69,$ce,$64,$01,$6d,$c2,$00);

  Script_10001 : array [1..2] of byte = ($00,$00);

  Script_10010 : array [1..2] of byte = ($00,$00);

  Script_10011 : array [1..2] of byte = ($00,$00);

  Script_10100 : array [1..2] of byte = ($00,$00);

  Script_10101 : array [1..2] of byte = ($00,$00);

  Script_10110 : array [1..2] of byte = ($00,$00);

  Script_10111 : array [1..2] of byte = ($00,$00);

  Script_11000 : array [1..2] of byte = ($00,$00);

  Script_11001 : array [1..2] of byte = ($00,$00);

  Script_11010 : array [1..2] of byte = ($00,$00);

  Script_11011 : array [1..2] of byte = ($00,$00);

  Script_11100 : array [1..2] of byte = ($00,$00);

  Script_11101 : array [1..2] of byte = ($00,$00);

  Script_11110 : array [1..2] of byte = ($00,$00);

  Script_11111 : array [1..2] of byte = ($00,$00);


// ========== TSLV11 ===========================================================

constructor TSLV11.Create(APort : string; AIDString : string);
begin
  inherited Create(APort,AIDString);
  OldState := $ff;
  if ReadInfo = 0 then
  begin
    TopMem := $2800;
    PCB.RowCount := 5;
    PCB.ColCount := 35;
  end
  else
  begin
    TopMem := UnhexWord(copy(InfoString,11,4));
    PCB.RowCount := UnhexByte(copy(InfoString,17,2));
    PCB.ColCount := UnhexByte(copy(InfoString,15,2));
  end;
  SupportedCommands.Add(TSLV11Cmd_Restart.Create);
  SupportedCommands.Add(TSLV11Cmd_ClearAll.Create);
  SupportedCommands.Add(TSLV11Cmd_SetAll.Create);
  SupportedCommands.Add(TSLV11Cmd_InsFromRight.Create);
  SupportedCommands.Add(TSLV11Cmd_InsFromLeft.Create);
  SupportedCommands.Add(TSLV11Cmd_FillVBuf.Create);
  SupportedCommands.Add(TSLV11Cmd_Call.Create);
  SupportedCommands.Add(TSLV11Cmd_GoTo.Create);
  SupportedCommands.Add(TSLV11Cmd_Stop.Create);
  SupportedCommands.Add(TSLV11Cmd_InsFromTop.Create);
  SupportedCommands.Add(TSLV11Cmd_InsFromBottom.Create);
  SupportedCommands.Add(TSLV11Cmd_SetHBuf.Create);
  SupportedCommands.Add(TSLV11Cmd_ClearHBuf.Create);
  SupportedCommands.Add(TSLV11Cmd_ScrollMsg.Create);
  SupportedCommands.Add(TSLV11Cmd_Delay.Create);
  SupportedCommands.Add(TSLV11Cmd_CharDelay.Create);
  SupportedCommands.Add(TSLV11Cmd_BigDelay.Create);
  SupportedCommands.Add(TSLV11Cmd_SetSpacing.Create);
  SupportedCommands.Add(TSLV11Cmd_SetDirection.Create);
  SupportedCommands.Add(TSLV11Cmd_SetMirroring.Create);
  SupportedCommands.Add(TSLV11Cmd_SetBrightness.Create);
  SupportedCommands.Add(TSLV11Cmd_ScrollRowLeft.Create);
  SupportedCommands.Add(TSLV11Cmd_ScrollRowRight.Create);
  SupportedCommands.Add(TSLV11Cmd_FillHBuf.Create);
  SupportedCommands.Add(TSLV11Cmd_SetPixel.Create);
  Inputs.Add(TPinInput.Create('Input #1 (Left Turn)'));
  Inputs.Add(TPinInput.Create('Input #2 (Stop)'));
  Inputs.Add(TPinInput.Create('Input #3 (Right Turn)'));
  Inputs.Add(TPinInput.Create('Input #4 (Reverse)'));
  Inputs.Add(TPinInput.Create('Input #5 (Accessory Power)'));
end;

// -----------------------------------------------------------------------------

function TSLV11.ReadDevice : integer;
var
  Buf  : PByteArray;
  ss   : string;
  tout : integer;
  blk  : integer;
  ipos : integer;
  ii,jj: integer;
  CharSizeTbl : array [0..127] of byte;
  cb   : array [0..31] of byte;
  scr  : TCommandSet;
  cmd  : TGenericCommand;
begin
  result := 0;
  GetMem(Buf,TopMem*2);
  SIO := TSerialIO.Create(Port,SIOCallback);
  try
    tout := 0;
    while (tout < 4) and (SIO.Error = SIO_ERR_CLOSED) do
    begin
      sleep(50);
      inc(tout);
    end;
    if SIO.Error <> SIO_ERR_NO_ERROR then exit;

    blk := 0;
    repeat
      if (blk mod 8) = 0 then
        ShowProgress(0,TopMem,blk*32,'Reading device...');
      ss := format('%2.2x%2.2x%s',[byte('R'),blk,#13]);
      Resp := '';
      SIO.Send(pchar(ss),length(ss));
      if WaitForEOL = 0 then exit;
      ss := GetRespStr;
      if (length(ss) <> 64) then
      begin
        ShowMessage('Communication error. Please try again.');
        exit;
      end;
      for ii:=0 to 31 do
        Buf[(integer(blk)*32)+ii] := UnhexByte(ss[(ii*2)+1]+ss[(ii*2)+2]);
      inc(blk);
    until (dword(blk*32) > TopMem);

    // Loading in the character table
    CharTable.Clear;
    for ii:=0 to 127 do
    begin
      CharSizeTbl[ii] := Buf[ii];
    end;
    ipos := 128;
    for ii:=0 to 127 do
    begin
      if (CharSizeTbl[ii] > 32) then
      begin
        ShowMessage('Incorrect or damaged data.');
        exit;
      end;
      if CharSizeTbl[ii] <> 0 then
      begin
        for jj:=0 to (CharSizeTbl[ii]-1) do cb[jj] := Buf[ipos+jj];
      end;
      CharTable[ii].SetData(cb,CharSizeTbl[ii]);
      inc(ipos,CharSizeTbl[ii]);
    end;

    // Loading scripts
    ii := 0;
    ScriptSet.Clear;
    scr := TCommandSet.Create;
    try
      while (dword(ipos) < (TopMem-1)) do
      begin
        if (Buf[ipos] = $aa) and (Buf[ipos+1] = $55) then
        begin
          inc(ipos,2);
          if assigned(scr) then
          begin
            ScriptSet.Add(scr);
            inc(ii);
            if ii >= 32 then break;
          end;
          scr := TCommandSet.Create;
        end
        else
        if assigned(scr) then
        begin
          cmd := TSLV11_LoadCommand(Buf^,ipos);
          scr.Add(cmd);
          if (cmd is TSLV11Cmd_Stop) or (cmd is TSLV11Cmd_Restart) then
          begin
            ScriptSet.Add(scr);
            scr := nil;
            inc(ii);
            if ii >= 32 then break;
          end;
        end
        else
          inc(ipos);
      end;
    except
    end;
    result := 1;
  finally
    SIO.Terminate;
    SIO := nil;
    FreeMem(buf);
    HideProgress;
  end;
end;

// -----------------------------------------------------------------------------

//function TGenericDevice.WriteBlock(ABlockNumber : integer) : boolean;
//var
//  ss   : string;
//  ii   : integer;
//begin
//  result := false;
//  if not assigned(SIO) then exit;
//  if DevType = dtSLV0703 then
//  begin
//    ss := format('%2.2x%2.2x',[byte('W'),ABlockNumber]);
//    for ii:=0 to 63 do
//      ss := ss+format('%2.2x',[Data[(integer(ABlockNumber)*64)+ii]]);
//    ss := ss+#13;
//    Resp := '';
//    SIOSend(pchar(ss),length(ss));
//    if not WaitForEOL then exit;
//    ss := GetRespStr;
//    result := (copy(ss,1,2) = 'OK');
//  end;
//end;

// -----------------------------------------------------------------------------

function TSLV11.WriteDevice : integer;
var
  ss  : string;
  Buf : PByteArray;
  bpos: integer;
  ii  : integer;
  jj  : integer;
  ch  : TCharacter;
  blk : integer;
  tout: integer;
  // ------------------------------------
  procedure DumpScript(ScrNo : integer);
  var
    Cmd : TGenericCommand;
    ii : integer;
  begin
    for ii:=0 to ScriptSet[ScrNo].Count-1 do
    begin
      Cmd := ScriptSet[ScrNo][ii];
      if dword(Cmd.Size+bpos) > TopMem then exit;
      cmd.Save(Buf^,bpos);
      if (cmd is TSLV11Cmd_Stop) or (cmd is TSLV11Cmd_Restart) then break;
    end;
    Buf[bpos] := $aa; inc(bpos);
    Buf[bpos] := $55; inc(bpos);
  end;
  // ------------------------------------
begin
  result := 0;
  try
    GetMem(Buf,TopMem*2);
    try
      for ii:=0 to TopMem*2-1 do Buf[ii] := $ff;
      bpos := 0;
      // Char Table
      for ii:=0 to 127 do
      begin
        Buf[bpos] := length(CharTable[ii].Data);
        inc(bpos);
      end;
      for ii:=0 to 127 do
      begin
        ch := CharTable[ii];
        for jj:=0 to length(ch.Data)-1 do
        begin
          Buf[bpos] := CharTable[ii].Data[jj];
          inc(bpos);
        end;
      end;
      for ii:=0 to 31 do DumpScript(ii);

      SIO := TSerialIO.Create(Port,SIOCallback);
      try
        tout := 0;
        while (tout < 4) and (SIO.Error = SIO_ERR_CLOSED) do
        begin
          sleep(50);
          inc(tout);
        end;
        if SIO.Error <> SIO_ERR_NO_ERROR then exit;

        blk := 0;
        repeat
          ShowProgress(0,(bpos div 64)+1,blk,'Writing device');
          
          ss := format('%2.2x%2.2x',[byte('W'),blk]);
          for ii:=0 to 63 do
            ss := ss+format('%2.2x',[Buf[(integer(blk)*64)+ii]]);
          ss := ss+#13;
          Resp := '';
          SIO.Send(pchar(ss),length(ss));
          if WaitForEOL = 0 then
          begin
            ShowMessage('Write timeout');
            exit;
          end;
          ss := GetRespStr;
          if (copy(ss,1,2) <> 'OK') then
          begin
            ShowMessage('Write error: '+ss);
            exit;
          end;
          inc(blk);
        until (blk*64) > bpos;
      finally
        SIO.Terminate;
        SIO := nil;
        HideProgress;
      end;
      result := 1;
    finally
      FreeMem(Buf);
    end;
  except
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11.ReadInfo : integer;
var
  tout : integer;
  ss   : string;
begin
  result := 0;
  InfoString := '';
  SIO := TSerialIO.Create(Port,SIOCallback);
  try
    tout := 0;
    while (tout < 4) and (SIO.Error = SIO_ERR_CLOSED) do
    begin
      sleep(50);
      inc(tout);
    end;
    if SIO.Error <> SIO_ERR_NO_ERROR then exit;
    resp := '';
    SIO.Send('49'#13,3);
    if WaitForEOL = 0 then exit;
    ss := GetRespStr;
    if (length(ss) < 18) then exit;
    InfoString := ss;
    result := 1;
  finally
    SIO.Terminate;
    SIO := nil;
  end;
end;

// -----------------------------------------------------------------------------

procedure TSLV11.Reset;
  // ------------------------------------------------
  procedure AddChar(len,Code,b0,b1,b2,b3,b4,b5 : byte); overload;
  var
    ch : TCharacter;
    ii : integer;
  begin
    ch := CharTable[Code];
    setlength(ch.Data,Len);
    if Len = 0 then exit;
    for ii:=0 to Len-1 do
    begin
      case ii of
        0: ch.Data[ii] := b0;
        1: ch.Data[ii] := b1;
        2: ch.Data[ii] := b2;
        3: ch.Data[ii] := b3;
        4: ch.Data[ii] := b4;
        5: ch.Data[ii] := b5;
      end;
    end;
  end;
  // ------------------------------------------------
  procedure AddChar(Code,Data1,Data2,Data3 : byte); overload;
  begin
    AddChar(3,Code,Data1,Data2,Data3,0,0,0);
  end;
  // ------------------------------------------------
  procedure AddChar(Code,Data1,Data2,Data3,Data4 : byte); overload;
  begin
    AddChar(4,Code,Data1,Data2,Data3,Data4,0,0);
  end;
  // ------------------------------------------------
  procedure AddChar(Code,Data1,Data2,Data3,Data4,Data5 : byte); overload;
  begin
    AddChar(5,Code,Data1,Data2,Data3,Data4,Data5,0);
  end;
  // ------------------------------------------------
  procedure AddChar(Code,Data1,Data2,Data3,Data4,Data5,Data6 : byte); overload;
  begin
    AddChar(6,Code,Data1,Data2,Data3,Data4,Data5,Data6);
  end;
  // ------------------------------------------------
  procedure AddScript(Code : byte; var Data; Len : integer);
  var
    cs : TCommandSet;
    ii : integer;
  begin
    while ScriptSet.Count <= Code do ScriptSet.Add(TCommandSet.Create);
    cs := ScriptSet[Code];
    ii := 0;
    while ii < Len do
      cs.Add(TSLV11_LoadCommand(Data,ii));
  end;
  // ------------------------------------------------
begin
  CharTable.Clear;
	AddChar(032,$00,$00,$00);
	AddChar(033,$00,$1d,$00);
	AddChar(034,$00,$18,$00,$18);
	AddChar(035,$0b,$0e,$1b,$0e,$1a);
	AddChar(036,$08,$15,$1f,$15,$02);
	AddChar(037,$19,$1a,$04,$0b,$13);
	AddChar(038,$0a,$15,$17,$0b);
	AddChar(039,$00,$18,$00);
	AddChar(040,$0e,$11,$00);
	AddChar(041,$11,$0e,$00);
	AddChar(042,$05,$06,$1c,$06,$05);
	AddChar(043,$04,$04,$1f,$04,$04);
	AddChar(044,$01,$02,$00);
	AddChar(045,$04,$04,$04,$04);
	AddChar(046,$00,$02,$00);
	AddChar(047,$01,$02,$04,$08,$10);
	AddChar(048,$0e,$11,$11,$0e);
	AddChar(049,$04,$08,$1f);
	AddChar(050,$11,$13,$15,$09);
	AddChar(051,$11,$15,$15,$0a);
	AddChar(052,$1c,$04,$04,$1f);
	AddChar(053,$1d,$15,$15,$17);
	AddChar(054,$0e,$15,$15,$02);
	AddChar(055,$11,$12,$14,$18);
	AddChar(056,$0a,$15,$15,$0a);
	AddChar(057,$1d,$15,$15,$1f);
	AddChar(058,$00,$00,$00);
	AddChar(059,$00,$00,$00);
	AddChar(060,$00,$00,$00);
	AddChar(061,$00,$00,$00);
	AddChar(062,$00,$00,$00);
	AddChar(063,$00,$00,$00);
	AddChar(064,$00,$00,$00);
	AddChar(065,$07,$0a,$12,$0a,$07);
	AddChar(066,$1f,$15,$1f,$0a);
	AddChar(067,$0e,$11,$11,$11);
	AddChar(068,$1f,$11,$11,$0e);
	AddChar(069,$1f,$15,$15,$11);
	AddChar(070,$1f,$14,$14,$10);
	AddChar(071,$0e,$11,$15,$17);
	AddChar(072,$1f,$04,$04,$1f);
	AddChar(073,$11,$1f,$11);
	AddChar(074,$12,$11,$1e);
	AddChar(075,$1f,$04,$0a,$11);
	AddChar(076,$1f,$01,$01);
	AddChar(077,$1f,$08,$04,$08,$1f);
	AddChar(078,$1f,$08,$04,$02,$1f);
	AddChar(079,$0e,$11,$11,$0e);
	AddChar(080,$1f,$14,$14,$08);
	AddChar(081,$0e,$11,$12,$0d);
	AddChar(082,$1f,$14,$16,$09);
	AddChar(083,$09,$15,$15,$12);
	AddChar(084,$10,$1f,$10);
	AddChar(085,$1e,$01,$01,$1e);
	AddChar(086,$1c,$02,$01,$02,$1c);
	AddChar(087,$1f,$02,$04,$02,$1f);
	AddChar(088,$11,$0a,$04,$0a,$11);
	AddChar(089,$10,$08,$07,$08,$10);
	AddChar(090,$11,$13,$15,$19,$11);
	AddChar(091,$00,$00,$00);
	AddChar(092,$00,$00,$00);
	AddChar(093,$00,$00,$00);
	AddChar(094,$00,$00,$00);
	AddChar(095,$00,$00,$00);
	AddChar(096,$1a,$19,$01,$01,$19,$1a);
	AddChar(097,$07,$0a,$12,$0a,$07);
	AddChar(098,$1f,$15,$1f,$0a);
	AddChar(099,$0e,$11,$11,$11);
	AddChar(100,$1f,$11,$11,$0e);
	AddChar(101,$1f,$15,$15,$11);
	AddChar(102,$1f,$14,$14,$10);
	AddChar(103,$0e,$11,$15,$17);
	AddChar(104,$1f,$04,$04,$1f);
	AddChar(105,$11,$1f,$11);
	AddChar(106,$12,$11,$1e);
	AddChar(107,$1f,$04,$0a,$11);
	AddChar(108,$1f,$01,$01);
	AddChar(109,$1f,$08,$04,$08,$1f);
	AddChar(110,$1f,$08,$04,$02,$1f);
	AddChar(111,$0e,$11,$11,$0e);
	AddChar(112,$1f,$14,$14,$08);
	AddChar(113,$0e,$11,$12,$0d);
	AddChar(114,$1f,$14,$16,$09);
	AddChar(115,$09,$15,$15,$12);
	AddChar(116,$10,$1f,$10);
	AddChar(117,$1e,$01,$01,$1e);
	AddChar(118,$1c,$02,$01,$02,$1c);
	AddChar(119,$1f,$02,$04,$02,$1f);
	AddChar(120,$11,$0a,$04,$0a,$11);
	AddChar(121,$10,$08,$07,$08,$10);
	AddChar(122,$11,$13,$15,$19,$11);
	AddChar(123,$00,$00,$00);
	AddChar(124,$00,$00,$00);
	AddChar(125,$00,$00,$00);
	AddChar(126,$00,$00,$00);
	AddChar(127,$00,$00,$00);

  // Creating default scripts
  ScriptSet.Clear;
  AddScript(00,Script_00000,sizeof(Script_00000));
  AddScript(01,Script_00001,sizeof(Script_00001));
  AddScript(02,Script_00010,sizeof(Script_00010));
  AddScript(03,Script_00011,sizeof(Script_00011));
  AddScript(04,Script_00100,sizeof(Script_00100));
  AddScript(05,Script_00101,sizeof(Script_00101));
  AddScript(06,Script_00110,sizeof(Script_00110));
  AddScript(07,Script_00111,sizeof(Script_00111));
  AddScript(08,Script_01000,sizeof(Script_01000));
  AddScript(09,Script_01001,sizeof(Script_01001));
  AddScript(10,Script_01010,sizeof(Script_01010));
  AddScript(11,Script_01011,sizeof(Script_01011));
  AddScript(12,Script_01100,sizeof(Script_01100));
  AddScript(13,Script_01101,sizeof(Script_01101));
  AddScript(14,Script_01110,sizeof(Script_01110));
  AddScript(15,Script_01111,sizeof(Script_01111));
  AddScript(16,Script_10000,sizeof(Script_10000));
  AddScript(17,Script_10001,sizeof(Script_10001));
  AddScript(18,Script_10010,sizeof(Script_10010));
  AddScript(19,Script_10011,sizeof(Script_10011));
  AddScript(20,Script_10100,sizeof(Script_10100));
  AddScript(21,Script_10101,sizeof(Script_10101));
  AddScript(22,Script_10110,sizeof(Script_10110));
  AddScript(23,Script_10111,sizeof(Script_10111));
  AddScript(24,Script_11000,sizeof(Script_11000));
  AddScript(25,Script_11001,sizeof(Script_11001));
  AddScript(26,Script_11010,sizeof(Script_11010));
  AddScript(27,Script_11011,sizeof(Script_11011));
  AddScript(28,Script_11100,sizeof(Script_11100));
  AddScript(29,Script_11101,sizeof(Script_11101));
  AddScript(30,Script_11110,sizeof(Script_11110));
  AddScript(31,Script_11111,sizeof(Script_11111));
end;

// -----------------------------------------------------------------------------

procedure TSLV11.UpdateActiveScript;
var
  b : byte;
begin
  if TPinInput(Inputs[1]).Value then
    b := $01
  else
  if TPinInput(Inputs[0]).Value and not TPinInput(Inputs[2]).Value then
    b := $02
  else
  if TPinInput(Inputs[2]).Value and not TPinInput(Inputs[0]).Value then
    b := $03
  else
  if TPinInput(Inputs[2]).Value and TPinInput(Inputs[0]).Value then
    b := $04
  else
  if TPinInput(Inputs[3]).Value then
    b := $05
  else
  begin
    if TPinInput(Inputs[4]).Value then
      b := $0f
    else
      b := $00;
  end;
  if b <> OldState then
  begin
    OldState := b;
    ActiveScript := b;
  end;
  ScriptSet[ActiveScript].Reset;
end;

// -----------------------------------------------------------------------------

function TSLV11.Run(NoDelay : boolean; CS : TCommandSet;  Cmd : TGenericCommand) : TUndo;
var
  ii : integer;
  ascr : integer;
  RCmd : TGenericCommand;
  Row,Col : integer;
begin
  if NoDelay then
  begin
    ascr := ActiveScript;
    ActiveScript := ScriptSet.Add(CS);
    try
      ScriptSet[ActiveScript].ActiveCommand := 0;
      ii := 0;
      RCmd := nil;
      repeat
        if ii > ScriptSet[ActiveScript].ActiveCommand then break; // Loop
        if (ScriptSet[ActiveScript][ScriptSet[ActiveScript].ActiveCommand] is TSLV11Cmd_Stop) or
           (ScriptSet[ActiveScript][ScriptSet[ActiveScript].ActiveCommand] is TSLV11Cmd_GoTo) or
           (ScriptSet[ActiveScript][ScriptSet[ActiveScript].ActiveCommand] is TSLV11Cmd_Restart) then break;
        if (ScriptSet[ActiveScript][ScriptSet[ActiveScript].ActiveCommand] is TSLV11Cmd_Call) then
        begin
          inc(ScriptSet[ActiveScript].ActiveCommand);
          if ScriptSet[ActiveScript].ActiveCommand >= ScriptSet[ActiveScript].Count then
            ScriptSet[ActiveScript].Reset;
        end
        else
        begin
          RCmd := ScriptSet[ActiveScript][ScriptSet[ActiveScript].ActiveCommand];
          for Row:=0 to PCB.RowCount-1 do
            for Col:=0 to PCB.ColCount-1 do
              result[Col,Row] := PCB[Col,Row].Active;
          if ScriptSet[ActiveScript].Run(PCB,CharTable,NoDelay) < 0 then break;
          PostMessage(frmMain.Handle,MSG_INVALIDATE,0,0);
        end;
        ii := ScriptSet[ActiveScript].ActiveCommand;
      until (RCmd = Cmd) and (RCmd <> ScriptSet[ActiveScript][ScriptSet[ActiveScript].ActiveCommand]);
    finally
      ScriptSet.Delete(ActiveScript);
      ActiveScript := ascr;
    end;
  end
  else
  begin
    result[0,0] := false;
    ii := ScriptSet[ActiveScript].Run(PCB,CharTable,NoDelay);
    case ii of
      -1: UpdateActiveScript;
       0: result[0,0] := false;
       1: result[0,0] := false;
       2: result[0,0] := true;
       3: result[0,0] := true;
    else
      if (ii and CALL_MSK) <> 0 then
      begin
      end
      else
      if (ii and GOTO_MSK) <> 0 then
      begin
        ActiveScript := lo(ii);
        UpdateActiveScript;
      end;
    end;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11.WriteTo(FileName : string; ID : integer) : integer;
var
  fs  : TFileStream;
  ss  : string;
  Buf : PByteArray;
  bpos: integer;
  ii  : integer;
  jj  : integer;
  ch  : TCharacter;
  // ------------------------------------
  procedure DumpScript(ScrNo : integer);
  var
    Cmd : TGenericCommand;
    ii : integer;
  begin
    for ii:=0 to ScriptSet[ScrNo].Count-1 do
    begin
      Cmd := ScriptSet[ScrNo][ii];
      if dword(Cmd.Size+bpos) > TopMem then exit;
      cmd.Save(Buf^,bpos);
      if (cmd is TSLV11Cmd_Stop) or (cmd is TSLV11Cmd_Restart) then break;
    end;
    Buf[bpos] := $aa; inc(bpos);
    Buf[bpos] := $55; inc(bpos);
  end;
  // ------------------------------------
begin
  result := 0;
  try
    fs := TFileStream.Create(FileName,fmCreate);
    GetMem(Buf,TopMem*2);
    try
      for ii:=0 to TopMem*2-1 do Buf[ii] := $ff;
      bpos := 0;
      ss := 'SLV11';
      fs.Write(ss[1],5);
      if ID = -1 then
      begin
        // Script Set
        ss := 'SET';
        fs.Write(ss[1],3);
        for ii:=0 to 31 do DumpScript(ii);
        fs.Write(Buf^,bpos);
      end
      else
      if ID >= 0 then
      begin
        // One Script
        ss := format('S%2.2X',[ID]);
        fs.Write(ss[1],3);
        DumpScript(ID);
        fs.Write(Buf^,bpos);
      end
      else
      begin
        // Char Table
        ss := 'CHT';
        fs.Write(ss[1],3);
        for ii:=0 to TopMem*2-1 do Buf[ii] := 0;
        for ii:=0 to 127 do
        begin
          Buf[bpos] := length(CharTable[ii].Data);
          inc(bpos);
        end;
        for ii:=0 to 127 do
        begin
          ch := CharTable[ii];
          for jj:=0 to length(ch.Data)-1 do
          begin
            Buf[bpos] := CharTable[ii].Data[jj];
            inc(bpos);
          end;
        end;
        fs.Write(Buf^,bpos);
      end;
      result := 1;
    finally
      FreeMem(Buf);
      fs.Free;
    end;
  except
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11.ReadFrom(FileName : string; ID : integer) : integer;
var
  fs  : TFileStream;
  Buf : PByteArray;
  ss  : string;
  bb  : byte;
  ii  : integer;
  jj  : integer;
  Index : integer;
  ch  : TCharacter;
  scr : TCommandSet;
  cmd : TGenericCommand;
  // ----------------------------------------------
  function LoadChar(Ch : TCharacter; var bpos : integer) : boolean;
  var
    ii : integer;
  begin
    result := false;
    if length(ch.Data) = 0 then exit;
    for ii:=0 to length(ch.Data)-1 do
    begin
      ch.Data[ii] := Buf[bpos];
      inc(bpos);
      if dword(bpos) >= TopMem then exit;
    end;
    result := true;
  end;
  // ------------------------------------------------
begin
  result := 0;
  if ID > 31 then exit;
  try
    fs := TFileStream.Create(FileName,fmOpenRead);
    GetMem(Buf,TopMem*2);
    try
      setlength(ss,8);
      if fs.Read(ss[1],8) < 8 then
      begin
        ShowMessage('Read Error');
        exit;
      end;
      if copy(ss,1,5) <> 'SLV11' then
      begin
        ShowMessage('This file is not for the currently selected device.');
        exit;
      end;
      jj := fs.Size-8;
      if dword(jj) > TopMem then jj := TopMem;
      if fs.Read(Buf^,jj) <> jj then
      begin
        ShowMessage('Read Error or File is too small');
        exit;
      end;
      ss := copy(ss,6,3);
      if ss = 'CHT' then
      begin
        CharTable.Clear;
        for Index:=0 to 127 do
        begin
          ch := CharTable[Index];
          setlength(ch.Data,Buf[Index]);
        end;
        Index := 128;
        ii := 0;
        while (Index < jj) and (ii < 128) do
        begin
          LoadChar(CharTable[ii],Index);
          inc(ii);
        end;
      end
      else
      if ss = 'SET' then
      begin
        ii := 0;
        ScriptSet.Clear;
        Index := 0;
        scr := TCommandSet.Create;
        try
          while (dword(Index) < (TopMem-1)) do
          begin
            if (Buf[Index] = $aa) and (Buf[Index+1] = $55) then
            begin
              inc(Index,2);
              if assigned(scr) then
              begin
                ScriptSet.Add(scr);
                inc(ii);
                if ii >= 32 then break;
              end;
              scr := TCommandSet.Create;
            end
            else
            if assigned(scr) then
            begin
              cmd := TSLV11_LoadCommand(Buf^,Index);
              scr.Add(cmd);
              if (cmd is TSLV11Cmd_Stop) or (cmd is TSLV11Cmd_Restart) then
              begin
                ScriptSet.Add(scr);
                scr := nil;
                inc(ii);
                if ii >= 32 then break;
              end;
            end
            else
              inc(Index);
          end;
        except
        end;
      end
      else
      begin
        bb := UnhexByte(copy(ss,2,2));
        if (ss[1] <> 'S') or (bb > 31) then
        begin
          ShowMessage('Can''t handle file of this type ('+ss+')');
          exit;
        end;
        ScriptSet[ID].Clear;
        Index := 0;
        scr := ScriptSet[ID];
        try
          while (dword(Index) < (TopMem-1)) do
          begin
            if (Buf[Index] = $aa) and (Buf[Index+1] = $55) then break;
            cmd := TSLV11_LoadCommand(Buf^,Index);
            scr.Add(cmd);
            if (cmd is TSLV11Cmd_Stop) or (cmd is TSLV11Cmd_Restart) then break;
          end;
        except
        end;
      end;
    finally
      FreeMem(Buf);
      fs.Free;
    end;
  except
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11.GetCharTblLimits : TPoint;
begin
  result.X := $00;
  result.Y := $7F;
end;

// -----------------------------------------------------------------------------

function TSLV11.GetMaxCharSize : integer;
begin
  result := 32;
end;

// -----------------------------------------------------------------------------

function TSLV11.CmdFromText(ACmdText : string) : TGenericCommand;
begin
  result := TSLV11_CreateCommand(ACmdText);
end;

// -----------------------------------------------------------------------------

function TSLV11.WriteTo(FileName : string; Scr : TCommandSet) : integer;
var
  fs  : TFileStream;
  ss  : string;
  Buf : PByteArray;
  bpos: integer;
  ii  : integer;
  jj  : integer;
  ch  : TCharacter;
  Cmd : TGenericCommand;
begin
  result := 0;
  try
    fs := TFileStream.Create(FileName,fmCreate);
    GetMem(Buf,TopMem*2);
    try
      for ii:=0 to TopMem*2-1 do Buf[ii] := $ff;
      bpos := 0;
      ss := 'SLV11';
      fs.Write(ss[1],5);
      // One Script
      ss := 'S00';
      fs.Write(ss[1],3);
      for ii:=0 to Scr.Count-1 do
      begin
        Cmd := Scr[ii];
        if dword(Cmd.Size+bpos) > TopMem then exit;
        cmd.Save(Buf^,bpos);
        if (cmd is TSLV11Cmd_Stop) or (cmd is TSLV11Cmd_Restart) then break;
      end;
      fs.Write(Buf^,bpos);
      result := 1;
    finally
      FreeMem(Buf);
      fs.Free;
    end;
  except
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11.ReadFrom(FileName : string; Scr : TCommandSet) : integer;
var
  fs  : TFileStream;
  Buf : PByteArray;
  ss  : string;
  bb  : byte;
  ii  : integer;
  jj  : integer;
  Index : integer;
  ch  : TCharacter;
  cmd : TGenericCommand;
begin
  result := 0;
  try
    fs := TFileStream.Create(FileName,fmOpenRead);
    GetMem(Buf,TopMem*2);
    try
      setlength(ss,8);
      if fs.Read(ss[1],8) < 8 then
      begin
        ShowMessage('Read Error');
        exit;
      end;
      if copy(ss,1,5) <> 'SLV11' then
      begin
        ShowMessage('This file is not for the currently selected device.');
        exit;
      end;
      jj := fs.Size-8;
      if dword(jj) > TopMem then jj := TopMem;
      if fs.Read(Buf^,jj) <> jj then
      begin
        ShowMessage('Read Error or File is too small');
        exit;
      end;
      ss := copy(ss,6,3);
      bb := UnhexByte(copy(ss,2,2));
      if (ss[1] <> 'S') or (bb > 31) then
      begin
        ShowMessage('Can''t handle file of this type ('+ss+')');
        exit;
      end;
      Scr.Clear;
      Index := 0;
      try
        while (dword(Index) < jj) do
        begin
          if (Buf[Index] = $aa) and (Buf[Index+1] = $55) then break;
          cmd := TSLV11_LoadCommand(Buf^,Index);
          scr.Add(Cmd);
        end;
      except
      end;
      result := 1;
    finally
      FreeMem(Buf);
      fs.Free;
    end;
  except
  end;
end;

// -----------------------------------------------------------------------------

end.

