unit SLV11Commands;

interface

uses
  GenericCommand, PCB, CharTable;

//   00000000 - Restart the script
type
  TSLV11Cmd_Restart = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   00000001 - Clear all
type
  TSLV11Cmd_ClearAll = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   00000010 - Set all
type
  TSLV11Cmd_SetAll = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   00000100 - Insert From Right
type
  TSLV11Cmd_InsFromRight = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   00000101 - Insert From Left
type
  TSLV11Cmd_InsFromLeft = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   010000xx - Set character spacing. xx = 0-3 empty columns between characters
type
  TSLV11Cmd_SetSpacing = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   0100010x - Set the direction for scrolling characters: 0 - right to left (default)
type
  TSLV11Cmd_SetDirection = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   0100011x - Char mirroring. 0 - straight, 1 - mirrored.
type
  TSLV11Cmd_SetMirroring = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   01101xxx - Set brightness
type
  TSLV11Cmd_SetBrightness = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   01110xxx - Scroll a row xxx left
type
  TSLV11Cmd_ScrollRowLeft = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   01111xxx - Scroll a row xxx right
type
  TSLV11Cmd_ScrollRowRight = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   10???xxx - Fill in the horizontal insert buffer, 4 bytes to follow
type
  TSLV11Cmd_FillHBuf = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   11000000 - Fill in the vertical insert buffer (next byte contains a value)
type
  TSLV11Cmd_FillVBuf = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   11000001 - Function call (next byte is a script number)
type
  TSLV11Cmd_Call = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   11000010 - GoTo (next byte is a script number)
type
  TSLV11Cmd_GoTo = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   11000011 - Stop execution
type
  TSLV11Cmd_Stop = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   11000100 - Insert from top
type
  TSLV11Cmd_InsFromTop = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   11000111 - Insert from bottom
type
  TSLV11Cmd_InsFromBottom = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   11001000 - Set the horizontal insert buffer
type
  TSLV11Cmd_SetHBuf = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   11001001 - Clear the horizontal insert buffer
type
  TSLV11Cmd_ClearHBuf = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
  end;

//   11001011 - Scroll a line. Next byte is a number of characters to follow
type
  TSLV11Cmd_ScrollMsg = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    procedure Reset; override;
    function GetEditType : integer; override;
  public
    CurrentChar : integer;
    CharCol : integer;
    CValue : integer;
  end;

//   11001100 - Delay. Next byte is a number of 0.002 sec intervals
type
  TSLV11Cmd_Delay = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    procedure Reset; override;
    function GetEditType : integer; override;
  public
    CValue : byte;
  end;

//   11001101 - Set the character output delay. Next byte is a number of 0.002 sec intervals
type
  TSLV11Cmd_CharDelay = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

//   11001110 - Big Delay. Next byte is a number of 0.02 sec intervals
type
  TSLV11Cmd_BigDelay = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    procedure Reset; override;
    function GetEditType : integer; override;
  public
    CValue : word;
  end;

//   111byyyy - Set a pixel to a "b" value next byte is xxxxxxxx
type
  TSLV11Cmd_SetPixel = class (TGenericCommand)
    constructor Create;
    function GetCmdText : string; override;
    function Load(var Data; var Index : integer) : integer; override;
    procedure Save(var Data; var Index : integer); override;
    function Size : integer; override;
    function Clone : TGenericCommand; override;
    function Compare(ACmd : TGenericCommand) : integer; override;
    function Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer; override;
    function GetEditType : integer; override;
  end;

function TSLV11_LoadCommand(var Data; var Index : integer) : TGenericCommand;
function TSLV11_CreateCommand(ACmdText : string) : TGenericCommand;

// =============================================================================
// =============================================================================
// =============================================================================

implementation

uses
  SysUtils, Classes, SerialIO, Constants;

// =============================================================================

function TSLV11_CreateCommand(ACmdText : string) : TGenericCommand;
var
  X,Y : byte;
  bb : byte;
  sl : TStringList;
  OnOff : string;
begin
  if ACmdText = '[RESTART]' then
    result := TSLV11Cmd_Restart.Create
  else
  if ACmdText = '[CLEARALL]' then
    result := TSLV11Cmd_ClearAll.Create
  else
  if ACmdText = '[SETALL]' then
    result := TSLV11Cmd_SetAll.Create
  else
  if ACmdText = '[INSFROMRIGHT]' then
    result := TSLV11Cmd_InsFromRight.Create
  else
  if ACmdText = '[INSFROMLEFT]' then
    result := TSLV11Cmd_InsFromLeft.Create
  else
  if ACmdText = '[STOP]' then
    result := TSLV11Cmd_Stop.Create
  else
  if ACmdText = '[INSFROMTOP]' then
    result := TSLV11Cmd_InsFromTop.Create
  else
  if ACmdText = '[INSFROMBOTTOM]' then
    result := TSLV11Cmd_InsFromBottom.Create
  else
  if ACmdText = '[SETHBUF]' then
    result := TSLV11Cmd_SetHBuf.Create
  else
  if ACmdText = '[CLEARHBUF]' then
    result := TSLV11Cmd_ClearHBuf.Create
  else
  if copy(ACmdText,1,13) = '[SETSPACING]=' then
  begin
    result := TSLV11Cmd_SetSpacing.Create;
    ACmdText := copy(ACmdText,14,255);
    while length(ACmdText) < 2 do ACmdText := '0'+ACmdText;
    bb := UnhexByte(copy(ACmdText,1,2));
    if bb > 3 then bb := 3;
    result.Parameters['Value'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,15) = '[SETDIRECTION]=' then
  begin
    result := TSLV11Cmd_SetDirection.Create;
    ACmdText := copy(ACmdText,16,255);
    if ACmdText = 'LTR' then
      result.Parameters['Value'] := 'LTR'
    else
      result.Parameters['Value'] := 'RTL';
  end
  else
  if copy(ACmdText,1,15) = '[SETMIRRORING]=' then
  begin
    result := TSLV11Cmd_SetDirection.Create;
    ACmdText := copy(ACmdText,16,255);
    if ACmdText = 'MIRRORED' then
      result.Parameters['Value'] := 'MIRRORED'
    else
      result.Parameters['Value'] := 'NORMAL';
  end
  else
  if copy(ACmdText,1,13) = '[BRIGHTNESS]=' then
  begin
    result := TSLV11Cmd_SetBrightness.Create;
    ACmdText := copy(ACmdText,14,255);
    while length(ACmdText) < 2 do ACmdText := '0'+ACmdText;
    bb := UnhexByte(copy(ACmdText,1,2));
    if bb > 7 then bb := 6;
    result.Parameters['Level'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,16) = '[SCROLLROWLEFT]=' then
  begin
    result := TSLV11Cmd_ScrollRowLeft.Create;
    ACmdText := copy(ACmdText,17,255);
    while length(ACmdText) < 2 do ACmdText := '0'+ACmdText;
    bb := UnhexByte(copy(ACmdText,1,2));
    if bb > 4 then bb := 4;
    result.Parameters['Row'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,17) = '[SCROLLROWRIGHT]=' then
  begin
    result := TSLV11Cmd_ScrollRowRight.Create;
    ACmdText := copy(ACmdText,18,255);
    while length(ACmdText) < 2 do ACmdText := '0'+ACmdText;
    bb := UnhexByte(copy(ACmdText,1,2));
    if bb > 4 then bb := 4;
    result.Parameters['Row'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,11) = '[FILLHBUF]=' then
  begin
    result := TSLV11Cmd_FillHBuf.Create;
    ACmdText := copy(ACmdText,12,255);
    while length(ACmdText) < 35 do ACmdText := ACmdText+'0';
    result.Parameters['Row'] := copy(ACmdText,1,35);
  end
  else
  if copy(ACmdText,1,11) = '[FILLVBUF]=' then
  begin
    result := TSLV11Cmd_FillVBuf.Create;
    ACmdText := copy(ACmdText,12,255);
    while length(ACmdText) < 8 do ACmdText := ACmdText+'0';
    result.Parameters['Value'] := copy(ACmdText,1,8);
  end
  else
  if copy(ACmdText,1,7) = '[CALL]=' then
  begin
    result := TSLV11Cmd_Call.Create;
    ACmdText := copy(ACmdText,8,255);
    while length(ACmdText) < 2 do ACmdText := '0'+ACmdText;
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Script'] := HexByte(bb and $1f);
  end
  else
  if copy(ACmdText,1,7) = '[GOTO]=' then
  begin
    result := TSLV11Cmd_GoTo.Create;
    ACmdText := copy(ACmdText,8,255);
    while length(ACmdText) < 2 do ACmdText := '0'+ACmdText;
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Script'] := HexByte(bb and $1f);
  end
  else
  if copy(ACmdText,1,12) = '[SCROLLMSG]=' then
  begin
    result := TSLV11Cmd_ScrollMsg.Create;
    ACmdText := copy(ACmdText,13,length(ACmdText));
    result.Parameters['Msg'] := copy(ACmdText,1,255);
  end
  else
  if copy(ACmdText,1,8) = '[DELAY]=' then
  begin
    result := TSLV11Cmd_Delay.Create;
    ACmdText := copy(ACmdText,9,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Value'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,12) = '[CHARDELAY]=' then
  begin
    result := TSLV11Cmd_CharDelay.Create;
    ACmdText := copy(ACmdText,13,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Value'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,11) = '[BIGDELAY]=' then
  begin
    result := TSLV11Cmd_BigDelay.Create;
    ACmdText := copy(ACmdText,12,255);
    while length(ACmdText) < 2 do ACmdText := ACmdText+'0';
    bb := UnhexByte(copy(ACmdText,1,2));
    result.Parameters['Value'] := HexByte(bb);
  end
  else
  if copy(ACmdText,1,11) = '[SETPIXEL]=' then
  begin
    result := TSLV11Cmd_SetPixel.Create;
    sl := TStringList.Create;
    try
      sl.CommaText := copy(ACmdText,12,255);
      X := 0; Y := 0; OnOff := 'ON';
      if sl.Count > 0 then X := UnhexByte(sl[0]);
      if sl.Count > 1 then Y := UnhexByte(sl[1]);
      if sl.Count > 2 then OnOff := sl[2];
      if X > 34 then X := 34;
      if Y > 4 then Y := 4;
      if (OnOff <> 'ON') and (OnOff <> 'OFF') then OnOff := 'ON';
      result.Parameters['X'] := HexByte(X);
      result.Parameters['Y'] := HexByte(Y);
      result.Parameters['OnOff'] := OnOff;
    finally
      sl.Free;
    end;
  end
  else
    result := TSLV11Cmd_Restart.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11_LoadCommand(var Data; var Index : integer) : TGenericCommand;
begin
  case TByteArray(Data)[Index] of
    $00 : result := TSLV11Cmd_Restart.Create;
    $01 : result := TSLV11Cmd_ClearAll.Create;
    $02 : result := TSLV11Cmd_SetAll.Create;
    $04 : result := TSLV11Cmd_InsFromRight.Create;
    $05 : result := TSLV11Cmd_InsFromLeft.Create;
    $c0 : result := TSLV11Cmd_FillVBuf.Create;
    $c1 : result := TSLV11Cmd_Call.Create;
    $c2 : result := TSLV11Cmd_GoTo.Create;
    $c3 : result := TSLV11Cmd_Stop.Create;
    $c4 : result := TSLV11Cmd_InsFromTop.Create;
    $c7 : result := TSLV11Cmd_InsFromBottom.Create;
    $c8 : result := TSLV11Cmd_SetHBuf.Create;
    $c9 : result := TSLV11Cmd_ClearHBuf.Create;
    $cb : result := TSLV11Cmd_ScrollMsg.Create;
    $cc : result := TSLV11Cmd_Delay.Create;
    $cd : result := TSLV11Cmd_CharDelay.Create;
    $ce : result := TSLV11Cmd_BigDelay.Create;
  else
    if (TByteArray(Data)[Index] and $fc) = $40 then
      result := TSLV11Cmd_SetSpacing.Create
    else
    if (TByteArray(Data)[Index] and $fe) = $44 then
      result := TSLV11Cmd_SetDirection.Create
    else
    if (TByteArray(Data)[Index] and $fe) = $46 then
      result := TSLV11Cmd_SetMirroring.Create
    else
    if (TByteArray(Data)[Index] and $f8) = $68 then
      result := TSLV11Cmd_SetBrightness.Create
    else
    if (TByteArray(Data)[Index] and $f8) = $70 then
      result := TSLV11Cmd_ScrollRowLeft.Create
    else
    if (TByteArray(Data)[Index] and $f8) = $78 then
      result := TSLV11Cmd_ScrollRowRight.Create
    else
    if (TByteArray(Data)[Index] and $c0) = $80 then
      result := TSLV11Cmd_FillHBuf.Create
    else
    if (TByteArray(Data)[Index] and $e0) = $e0 then
      result := TSLV11Cmd_SetPixel.Create
    else
      result := TSLV11Cmd_Restart.Create;
  end;
  result.Load(Data,Index);
end;

// ========== TSLV11Cmd_Restart ================================================

constructor TSLV11Cmd_Restart.Create;
begin
  inherited Create;
  ImgName := 'CI_RESTART';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command restarts the script''s so the execution continues from the first script''s command.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Restart.GetCmdText : string;
begin
  result := '[RESTART]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Restart.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_Restart.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $00; // 00000000 - Restart the script
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Restart.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Restart.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_Restart.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Restart.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_Restart then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Restart.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  // The following result forces the TCommandSet object to reset all the commands
  // and restart the execution flow from the first one
  result := -1;
end;

// ========== TSLV11Cmd_ClearAll ===============================================

constructor TSLV11Cmd_ClearAll.Create;
begin
  inherited Create;
  ImgName := 'CI_CLEARALL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command switches all the LEDs to the "OFF" state.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearAll.GetCmdText : string;
begin
  result := '[CLEARALL]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearAll.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_ClearAll.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $01; // 00000001
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearAll.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearAll.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_ClearAll.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearAll.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_ClearAll then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearAll.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : integer;
begin
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-1 do
      PCB[col,row].Active := false;
  // This command has finished
  result := 2;
end;

// ========== TSLV11Cmd_SetAll =================================================

constructor TSLV11Cmd_SetAll.Create;
begin
  inherited Create;
  ImgName := 'CI_SETALL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command switches all the LEDs to the "ON" state.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetAll.GetCmdText : string;
begin
  result := '[SETALL]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetAll.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_SetAll.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $02; // 00000010
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetAll.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetAll.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_SetAll.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetAll.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_SetAll then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetAll.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : integer;
begin
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-1 do
      PCB[col,row].Active := true;
  // This command has finished
  result := 2;
end;

// ========== TSLV11Cmd_InsFromRight ===========================================

constructor TSLV11Cmd_InsFromRight.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMRIGHT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display to the left and inserts a column, defined by te script''s data into the rightmost display column.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromRight.GetCmdText : string;
begin
  result := '[INSFROMRIGHT]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromRight.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_InsFromRight.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $04; // 00000100
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromRight.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromRight.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_InsFromRight.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromRight.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_InsFromRight then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromRight.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
  b : byte;
begin
  // Shift the display
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-2 do
      PCB[col,row].Active := PCB[col+1,row].Active;
  // Set the rightmost column
  b := 1;
  for row:=PCB.RowCount-1 downto 0 do
  begin
    PCB[PCB.ColCount-1,Row].Active := ((PCB.Tag5 and b) <> 0);
    b := b shl 1;
  end;
  // This command has finished
  result := 2;
end;

// ========== TSLV11Cmd_InsFromLeft ============================================

constructor TSLV11Cmd_InsFromLeft.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMLEFT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display to the right and inserts a column, defined by te script''s data into the leftmost display column.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromLeft.GetCmdText : string;
begin
  result := '[INSFROMLEFT]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromLeft.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_InsFromLeft.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $05; // 00000101
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromLeft.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromLeft.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_InsFromLeft.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromLeft.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_InsFromLeft then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromLeft.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
  b : byte;
begin
  // Shift the display
  for row:=0 to PCB.RowCount-1 do
    for col:=PCB.ColCount-1 downto 1 do
      PCB[col,row].Active := PCB[col-1,row].Active;
  // Set the leftmost column
  b := 1;
  for row:=PCB.RowCount-1 downto 0 do
  begin
    PCB[0,Row].Active := ((PCB.Tag5 and b) <> 0);
    b := b shl 1;
  end;
  // This command has finished
  result := 2;
end;

// ========== TSLV11Cmd_SetSpacing =============================================

constructor TSLV11Cmd_SetSpacing.Create;
begin
  inherited Create;
  ImgName := 'CI_SETSPACING';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets the number of empty columns between characters (0-3).';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetSpacing.GetCmdText : string;
begin
  result := '[SETSPACING]='+Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetSpacing.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Value'] := HexByte(TByteArray(Data)[Index] and $03);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_SetSpacing.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($40 or (UnhexByte(Parameters['Value']) and $03)); // 010000xx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetSpacing.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetSpacing.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_SetSpacing.Create;
  TSLV11Cmd_SetSpacing(result).Parameters['Value'] := Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetSpacing.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_SetSpacing then
  begin
    if ((ACmd as TSLV11Cmd_SetSpacing).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetSpacing.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  PCB.Tag3 := UnhexByte(Parameters['Value']);
  // This command has finished
  result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetSpacing.GetEditType : integer;
begin
  result := ED_SPACING;
end;

// ========== TSLV11Cmd_SetDirection ===========================================

constructor TSLV11Cmd_SetDirection.Create;
begin
  inherited Create;
  ImgName := 'CI_SETDIRECTION';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets the character''s line scrolling direction (right-to-left or left-to-right';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetDirection.GetCmdText : string;
begin
  result := '[SETDIRECTION]='+Parameters['Value'];  // We store this parameter as one of the values 'LTR' or 'RTL'
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetDirection.Load(var Data; var Index : integer) : integer;
begin
  if (TByteArray(Data)[Index] and $01) = 0 then
    Parameters['Value'] := 'RTL'
  else
    Parameters['Value'] := 'LTR';
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_SetDirection.Save(var Data; var Index : integer);
begin
  if Parameters['Value'] = 'LTR' then   // 0100010x
    TByteArray(Data)[Index] := $45
  else
    TByteArray(Data)[Index] := $44;
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetDirection.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetDirection.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_SetDirection.Create;
  TSLV11Cmd_SetDirection(result).Parameters['Value'] := Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetDirection.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_SetDirection then
  begin
    if ((ACmd as TSLV11Cmd_SetDirection).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetDirection.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  if Parameters['Value'] = 'LTR' then
    PCB.Tag6 := 1
  else
    PCB.Tag6 := 0;
  // This command has finished
  result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetDirection.GetEditType : integer;
begin
  result := ED_DIRECTION;
end;

// ========== TSLV11Cmd_SetMirroring ===========================================

constructor TSLV11Cmd_SetMirroring.Create;
begin
  inherited Create;
  ImgName := 'CI_MIRRORING';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets the character''s mirroring. Usually used with the "SetDirection" command.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetMirroring.GetCmdText : string;
begin
  result := '[SETMIRRORING]='+Parameters['Value'];  // We store this parameter as one of the values 'NORMAL' or 'MIRRORED'
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetMirroring.Load(var Data; var Index : integer) : integer;
begin
  if (TByteArray(Data)[Index] and $01) = 0 then
    Parameters['Value'] := 'NORMAL'
  else
    Parameters['Value'] := 'MIRRORED';
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_SetMirroring.Save(var Data; var Index : integer);
begin
  if Parameters['Value'] = 'MIRRORED' then   // 0100011x
    TByteArray(Data)[Index] := $47
  else
    TByteArray(Data)[Index] := $46;
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetMirroring.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetMirroring.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_SetMirroring.Create;
  TSLV11Cmd_SetMirroring(result).Parameters['Value'] := Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetMirroring.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_SetMirroring then
  begin
    if ((ACmd as TSLV11Cmd_SetMirroring).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetMirroring.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  if Parameters['Value'] = 'MIRRORED' then
    PCB.Tag7 := 1
  else
    PCB.Tag7 := 0;
  // This command has finished
  result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetMirroring.GetEditType : integer;
begin
  result := ED_MIRRORING;
end;


// ========== TSLV11Cmd_SetBrightness ==========================================

constructor TSLV11Cmd_SetBrightness.Create;
begin
  inherited Create;
  ImgName := 'CI_BRIGHTNESS';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets the brightness level for all the pixels.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetBrightness.GetCmdText : string;
begin
  result := '[BRIGHTNESS]='+Parameters['Level'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetBrightness.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Level'] := HexByte(TByteArray(Data)[Index] and $07);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_SetBrightness.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($68 or (UnhexByte(Parameters['Level']) and $07)); // 01101xxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetBrightness.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetBrightness.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_SetBrightness.Create;
  TSLV11Cmd_SetBrightness(result).Parameters['Level'] := Parameters['Level'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetBrightness.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_SetBrightness then
  begin
    if ((ACmd as TSLV11Cmd_SetBrightness).Parameters['Level'] = Parameters['Level']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetBrightness.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
begin
  for row:=0 to PCB.RowCount-1 do
    for col:=0 to PCB.ColCount-1 do
      PCB[col,row].Brightness := (UnhexByte(Parameters['Level']) and $07);
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetBrightness.GetEditType : integer;
begin
  result := ED_BRIGHTNESS;
end;

// ========== TSLV11Cmd_ScrollRowLeft ==========================================

constructor TSLV11Cmd_ScrollRowLeft.Create;
begin
  inherited Create;
  ImgName := 'CI_SCROLLROWLEFT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display row one pixel to the left.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowLeft.GetCmdText : string;
begin
  result := '[SCROLLROWLEFT]='+Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowLeft.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Row'] := HexByte(TByteArray(Data)[Index] and $07);
  if UnhexByte(Parameters['Row']) > 4 then Parameters['Row'] := '04';
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_ScrollRowLeft.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($70 or (UnhexByte(Parameters['Row']) and $07)); // 01110xxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowLeft.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowLeft.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_ScrollRowLeft.Create;
  TSLV11Cmd_ScrollRowLeft(result).Parameters['Row'] := Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowLeft.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_ScrollRowLeft then
  begin
    if ((ACmd as TSLV11Cmd_ScrollRowLeft).Parameters['Row'] = Parameters['Row']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowLeft.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col : integer;
  Row : byte;
begin
  Row := UnhexByte(Parameters['Row']);
  if assigned(PCB.LEDRows[Row]) then
  begin
    for col:=0 to PCB.ColCount-2 do
      PCB.LEDRows[Row][col].Active := PCB.LEDRows[Row][col+1].Active;
    PCB.LEDRows[Row][PCB.ColCount-1].Active := false;
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowLeft.GetEditType : integer;
begin
  result := ED_ROW_LEFT;
end;

// ========== TSLV11Cmd_ScrollRowRight =========================================

constructor TSLV11Cmd_ScrollRowRight.Create;
begin
  inherited Create;
  ImgName := 'CI_SCROLLROWRIGHT';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts a display row one pixel to the right.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowRight.GetCmdText : string;
begin
  result := '[SCROLLROWRIGHT]='+Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowRight.Load(var Data; var Index : integer) : integer;
begin
  Parameters['Row'] := HexByte(TByteArray(Data)[Index] and $07);
  if UnhexByte(Parameters['Row']) > 4 then Parameters['Row'] := '04';
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_ScrollRowRight.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($78 or (UnhexByte(Parameters['Row']) and $07)); // 01111xxx
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowRight.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowRight.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_ScrollRowRight.Create;
  TSLV11Cmd_ScrollRowRight(result).Parameters['Row'] := Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowRight.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_ScrollRowRight then
  begin
    if ((ACmd as TSLV11Cmd_ScrollRowRight).Parameters['Row'] = Parameters['Row']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowRight.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col : integer;
  Row : byte;
begin
  Row := UnhexByte(Parameters['Row']);
  if assigned(PCB.LEDRows[Row]) then
  begin
    for col:=PCB.ColCount-1 downto 1 do
      PCB.LEDRows[Row][col].Active := PCB.LEDRows[Row][col-1].Active;
    PCB.LEDRows[Row][0].Active := false;
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollRowRight.GetEditType : integer;
begin
  result := ED_ROW_RIGHT;
end;

// ========== TSLV11Cmd_FillHBuf ===============================================

constructor TSLV11Cmd_FillHBuf.Create;
begin
  inherited Create;
  ImgName := 'CI_FILLHBUF';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command fills in the horizontal insert buffer. To use with the "InsertFromTop" and "InsertFromBottom" commands.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillHBuf.GetCmdText : string;
begin
  result := '[FILLHBUF]='+Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillHBuf.Load(var Data; var Index : integer) : integer;
var
  ss : string;
begin
  ss := copy(BitStr(TByteArray(Data)[Index]),6,3);
  inc(Index);
  ss := ss+BitStr(TByteArray(Data)[Index]);
  inc(Index);
  ss := ss+BitStr(TByteArray(Data)[Index]);
  inc(Index);
  ss := ss+BitStr(TByteArray(Data)[Index]);
  inc(Index);
  Parameters['Row'] := ss+BitStr(TByteArray(Data)[Index]);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_FillHBuf.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := ($80 or (StrBit(copy(Parameters['Row'],1,3)) and $1f)); // 10???xxx
  inc(Index);
  TByteArray(Data)[Index] := StrBit(copy(Parameters['Row'],4,8));
  inc(Index);
  TByteArray(Data)[Index] := StrBit(copy(Parameters['Row'],12,8));
  inc(Index);
  TByteArray(Data)[Index] := StrBit(copy(Parameters['Row'],20,8));
  inc(Index);
  TByteArray(Data)[Index] := StrBit(copy(Parameters['Row'],28,8));
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillHBuf.Size : integer;
begin
  result := 5;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillHBuf.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_FillHBuf.Create;
  TSLV11Cmd_FillHBuf(result).Parameters['Row'] := Parameters['Row'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillHBuf.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_FillHBuf then
  begin
    if ((ACmd as TSLV11Cmd_FillHBuf).Parameters['Row'] = Parameters['Row']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillHBuf.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  PCB.Tag4 := Parameters['Row'];
  // This command has finished
  result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillHBuf.GetEditType : integer;
begin
  result := ED_HBUF;
end;

// ========== TSLV11Cmd_FillVBuf ===============================================

constructor TSLV11Cmd_FillVBuf.Create;
begin
  inherited Create;
  ImgName := 'CI_FILLVBUF';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command fills in the vertical insert buffer. To use with the "InsertFromRight" and "InsertFromLeft" commands.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillVBuf.GetCmdText : string;
begin
  result := '[FILLVBUF]='+Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillVBuf.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  Parameters['Value'] := BitStr(TByteArray(Data)[Index]);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_FillVBuf.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c0; // 11000000
  inc(Index);
  TByteArray(Data)[Index] := StrBit(Parameters['Value']);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillVBuf.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillVBuf.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_FillVBuf.Create;
  TSLV11Cmd_FillVBuf(result).Parameters['Value'] := Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillVBuf.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_FillVBuf then
  begin
    if ((ACmd as TSLV11Cmd_FillVBuf).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillVBuf.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  PCB.Tag5 := StrBit(Parameters['Value']);
  // This command has finished
  result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_FillVBuf.GetEditType : integer;
begin
  result := ED_VBUF;
end;

// ========== TSLV11Cmd_Call ===================================================

constructor TSLV11Cmd_Call.Create;
begin
  inherited Create;
  ImgName := 'CI_CALL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command is used to run another script and continue from this point after it finished.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Call.GetCmdText : string;
begin
  result := '[CALL]='+Parameters['Script'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Call.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  Parameters['Script'] := HexByte(TByteArray(Data)[Index]);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_Call.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c1; // 11000001
  inc(Index);
  TByteArray(Data)[Index] := UnhexByte(Parameters['Script']);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Call.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Call.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_Call.Create;
  TSLV11Cmd_Call(result).Parameters['Script'] := Parameters['Script'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Call.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_Call then
  begin
    if ((ACmd as TSLV11Cmd_Call).Parameters['Script'] = Parameters['Script']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Call.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  result := UnhexByte(Parameters['Script']) or CALL_MSK;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Call.GetEditType : integer;
begin
  result := ED_CALL;
end;

// ========== TSLV11Cmd_GoTo ===================================================

constructor TSLV11Cmd_GoTo.Create;
begin
  inherited Create;
  ImgName := 'CI_GOTO';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command is used to run another script instead of the current one.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_GoTo.GetCmdText : string;
begin
  result := '[GOTO]='+Parameters['Script'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_GoTo.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  Parameters['Script'] := HexByte(TByteArray(Data)[Index]);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_GoTo.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c2; // 11000010
  inc(Index);
  TByteArray(Data)[Index] := UnhexByte(Parameters['Script']);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_GoTo.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_GoTo.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_GoTo.Create;
  TSLV11Cmd_GoTo(result).Parameters['Script'] := Parameters['Script'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_GoTo.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_GoTo then
  begin
    if ((ACmd as TSLV11Cmd_GoTo).Parameters['Script'] = Parameters['Script']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_GoTo.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  result := UnhexByte(Parameters['Script']) or GOTO_MSK;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_GoTo.GetEditType : integer;
begin
  result := ED_GOTO;
end;

// ========== TSLV11Cmd_Stop ===================================================

constructor TSLV11Cmd_Stop.Create;
begin
  inherited Create;
  ImgName := 'CI_STOP';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command stops the script''s execution flow.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Stop.GetCmdText : string;
begin
  result := '[STOP]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Stop.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_Stop.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c3; // 11000011
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Stop.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Stop.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_Stop.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Stop.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_Stop then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Stop.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  // Script will be stuck on this command
  result := 1;
end;

// ========== TSLV11Cmd_InsFromTop =============================================

constructor TSLV11Cmd_InsFromTop.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMTOP';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts the image down one row and then sets the top row according to the content of the horizontal insert buffer.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromTop.GetCmdText : string;
begin
  result := '[INSFROMTOP]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromTop.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_InsFromTop.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c4; // 11000100
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromTop.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromTop.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_InsFromTop.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromTop.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_InsFromTop then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromTop.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : word;
begin
  for row:=PCB.RowCount-1 downto 1 do
    PCB.LEDRows[row].Assign(PCB.LEDRows[row-1]);
  for col:=0 to PCB.ColCount-1 do
    PCB[col,0].Active := (copy(PCB.Tag4,col+1,1) = '1');
  // This command has finished
  result := 2;
end;

// ========== TSLV11Cmd_InsFromBottom ==========================================

constructor TSLV11Cmd_InsFromBottom.Create;
begin
  inherited Create;
  ImgName := 'CI_INSFROMBOTTOM';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command shifts the image up one row and then sets the bottom row according to the content of the horizontal insert buffer.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromBottom.GetCmdText : string;
begin
  result := '[INSFROMBOTTOM]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromBottom.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_InsFromBottom.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c7; // 11000111
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromBottom.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromBottom.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_InsFromBottom.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromBottom.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_InsFromBottom then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_InsFromBottom.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  col,row : word;
begin
  for row:=0 to PCB.RowCount-2 do
    PCB.LEDRows[row].Assign(PCB.LEDRows[row+1]);
  for col:=0 to PCB.ColCount-1 do
    PCB[col,PCB.RowCount-1].Active := (copy(PCB.Tag4,col+1,1) = '1');
  // This command has finished
  result := 2;
end;

// ========== TSLV11Cmd_SetHBuf ================================================

constructor TSLV11Cmd_SetHBuf.Create;
begin
  inherited Create;
  ImgName := 'CI_SETHBUF';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets all the bits in the horizontal insert buffer. To use with the "InsertFromTop" and "InsertFromBottom" commands.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetHBuf.GetCmdText : string;
begin
  result := '[SETHBUF]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetHBuf.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_SetHBuf.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c8; // 11001000
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetHBuf.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetHBuf.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_SetHBuf.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetHBuf.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_SetHBuf then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetHBuf.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  ii : integer;
  ss : string;
begin
  ss := '';
  for ii:=0 to PCB.ColCount-1 do ss := ss+'1';
  PCB.Tag4 := ss;
  result := 0;
end;

// ========== TSLV11Cmd_ClearHBuf ==============================================

constructor TSLV11Cmd_ClearHBuf.Create;
begin
  inherited Create;
  ImgName := 'CI_CLEARHBUF';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command clears all the bits in the horizontal insert buffer. To use with the "InsertFromTop" and "InsertFromBottom" commands.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearHBuf.GetCmdText : string;
begin
  result := '[CLEARHBUF]';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearHBuf.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_ClearHBuf.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $c9; // 11001001
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearHBuf.Size : integer;
begin
  result := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearHBuf.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_ClearHBuf.Create;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearHBuf.Compare(ACmd : TGenericCommand) : integer;
begin
  if ACmd is TSLV11Cmd_ClearHBuf then
    result := 1
  else
    result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ClearHBuf.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  ii : integer;
  ss : string;
begin
  ss := '';
  for ii:=0 to PCB.ColCount-1 do ss := ss+'0';
  PCB.Tag4 := ss;
  result := 0;
end;

// ========== TSLV11Cmd_ScrollMsg ==============================================

constructor TSLV11Cmd_ScrollMsg.Create;
begin
  inherited Create;
  ImgName := 'CI_SCROLLMSG';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command scrolls a message.';
  Parameters['Msg'] := '';
  CurrentChar := 1;
  CharCol := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollMsg.GetCmdText : string;
begin
  result := '[SCROLLMSG]='+Parameters['Msg'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollMsg.Load(var Data; var Index : integer) : integer;
var
  len : byte;
  ss  : string;
begin
  inc(Index);
  len := TByteArray(Data)[Index];
  inc(Index);
  ss := '';
  while len > 0 do
  begin
    ss := ss+char(TByteArray(Data)[Index]);
    inc(Index);
    dec(len);
  end;
  CurrentChar := 1;
  CharCol := 0;
  Parameters['Msg'] := EscapeChars(ss);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_ScrollMsg.Save(var Data; var Index : integer);
var
  len : byte;
  ii  : byte;
  ss  : string;
begin
  ss := UnescapeChars(Parameters['Msg']);
  len := length(ss);
  TByteArray(Data)[Index] := $cb; // 11001011
  inc(Index);
  TByteArray(Data)[Index] := len;
  inc(Index);
  for ii:=1 to len do
  begin
    TByteArray(Data)[Index] := byte(ss[ii]);
    inc(Index);
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollMsg.Size : integer;
begin
  result := length(UnescapeChars(Parameters['Msg']))+2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollMsg.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_ScrollMsg.Create;
  TSLV11Cmd_ScrollMsg(result).Parameters['Msg'] := Parameters['Msg'];
  TSLV11Cmd_ScrollMsg(result).CurrentChar := 1;
  TSLV11Cmd_ScrollMsg(result).CharCol := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollMsg.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_ScrollMsg then
  begin
    if ((ACmd as TSLV11Cmd_ScrollMsg).Parameters['Msg'] = Parameters['Msg']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollMsg.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  row,col : integer;
  b1,b2 : byte;
  ss : string;
begin
  result := 3;
  ss := UnescapeChars(Parameters['Msg']);
  if CurrentChar > length(ss) then
  begin
    result := 0;
    exit;
  end;
  if not NoDelay then sleep(lo(PCB.Tag1)*2);
  if not assigned(CharTable[byte(ss[CurrentChar])]) or (CharCol >= length(CharTable[byte(ss[CurrentChar])].Data)) then
  begin
    b2 := 0;
    dec(CValue);
    if CValue < 0 then
    begin
      inc(CurrentChar);
      CharCol := 0;
      if CurrentChar > length(ss) then
      begin
        CurrentChar := 1;
        result := 2;
      end;
      exit;
    end;
  end
  else
  begin
    if PCB.Tag7 = 0 then
      b2 := CharTable[byte(ss[CurrentChar])].Data[CharCol]
    else
      b2 := CharTable[byte(ss[CurrentChar])].Data[length(CharTable[byte(ss[CurrentChar])].Data)-CharCol-1];
    inc(CharCol);
    if CharCol >= length(CharTable[byte(ss[CurrentChar])].Data) then
      CValue := PCB.Tag3;
  end;
  // Shift the display
  if PCB.Tag6 = 0 then
  begin
    for row:=0 to PCB.RowCount-1 do
      for col:=0 to PCB.ColCount-2 do
        PCB[col,row].Active := PCB[col+1,row].Active;
  end
  else
  begin
    for row:=0 to PCB.RowCount-1 do
      for col:=PCB.ColCount-1 downto 1 do
        PCB[col,row].Active := PCB[col-1,row].Active;
  end;
  // Set the rightmost column
  b1 := 1;
  for row:=PCB.RowCount-1 downto 0 do
  begin
    if PCB.Tag6 = 0 then
      PCB[PCB.ColCount-1,Row].Active := ((b2 and b1) <> 0)
    else
      PCB[0,Row].Active := ((b2 and b1) <> 0);
    b1 := b1 shl 1;
  end;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_ScrollMsg.Reset;
begin
  CharCol := 0;
  CurrentChar := 1;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_ScrollMsg.GetEditType : integer;
begin
  result := ED_SIMPLE_MSG_256;
end;

// ========== TSLV11Cmd_Delay ==================================================

constructor TSLV11Cmd_Delay.Create;
begin
  inherited Create;
  ImgName := 'CI_DELAY';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command pauses the script for the specified time.';
  Parameters['Value'] := '10';
  CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Delay.GetCmdText : string;
begin
  result := '[DELAY]='+Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Delay.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  Parameters['Value'] := HexByte(TByteArray(Data)[Index]);
  CValue := 0;
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_Delay.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $cc; // 11001100
  inc(Index);
  TByteArray(Data)[Index] := UnhexByte(Parameters['Value']);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Delay.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Delay.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_Delay.Create;
  TSLV11Cmd_Delay(result).Parameters['Value'] := Parameters['Value'];
  TSLV11Cmd_Delay(result).CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Delay.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_Delay then
  begin
    if ((ACmd as TSLV11Cmd_Delay).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Delay.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  bb : byte;
begin
  if not NoDelay then
  begin
    bb := UnhexByte(Parameters['Value']);
    sleep(bb*4);
  end;
  result := 0;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_Delay.Reset;
begin
  CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_Delay.GetEditType : integer;
begin
  result := ED_SET_DELAY_256;
end;

// ========== TSLV11Cmd_CharDelay ==============================================

constructor TSLV11Cmd_CharDelay.Create;
begin
  inherited Create;
  ImgName := 'CI_CHARDELAY';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets the delay time for the "ScrollMsg" commands.';
  Parameters['Value'] := '10';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_CharDelay.GetCmdText : string;
begin
  result := '[CHARDELAY]='+Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_CharDelay.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  Parameters['Value'] := HexByte(TByteArray(Data)[Index]);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_CharDelay.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $cd; // 11001101
  inc(Index);
  TByteArray(Data)[Index] := UnhexByte(Parameters['Value']);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_CharDelay.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_CharDelay.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_CharDelay.Create;
  TSLV11Cmd_CharDelay(result).Parameters['Value'] := Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_CharDelay.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_CharDelay then
  begin
    if ((ACmd as TSLV11Cmd_CharDelay).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_CharDelay.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  PCB.Tag1 := UnhexByte(Parameters['Value']);
  result := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_CharDelay.GetEditType : integer;
begin
  result := ED_SET_DELAY_256;
end;

// ========== TSLV11Cmd_BigDelay ===============================================

constructor TSLV11Cmd_BigDelay.Create;
begin
  inherited Create;
  ImgName := 'CI_BIGDELAY';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command pauses the script for the specified time.';
  Parameters['Value'] := '2';
  CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_BigDelay.GetCmdText : string;
begin
  result := '[BIGDELAY]='+Parameters['Value'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_BigDelay.Load(var Data; var Index : integer) : integer;
begin
  inc(Index);
  Parameters['Value'] := HexByte(TByteArray(Data)[Index]);
  CValue := 0;
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_BigDelay.Save(var Data; var Index : integer);
begin
  TByteArray(Data)[Index] := $ce; // 11001110
  inc(Index);
  TByteArray(Data)[Index] := UnhexByte(Parameters['Value']);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_BigDelay.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_BigDelay.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_BigDelay.Create;
  TSLV11Cmd_BigDelay(result).Parameters['Value'] := Parameters['Value'];
  TSLV11Cmd_BigDelay(result).CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_BigDelay.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_BigDelay then
  begin
    if ((ACmd as TSLV11Cmd_BigDelay).Parameters['Value'] = Parameters['Value']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_BigDelay.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
begin
  if NoDelay then
  begin
    result := 0;
    exit;
  end;
  if CValue = 0 then
    PCB.Tag2 := word(UnhexByte(Parameters['Value']));
  sleep(40);
  inc(CValue);
  if CValue >= PCB.Tag2 then
    result := 0
  else
    result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_BigDelay.Reset;
begin
  CValue := 0;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_BigDelay.GetEditType : integer;
begin
  result := ED_SET_DELAY_2560;
end;

// ========== TSLV11Cmd_SetPixel ===============================================

constructor TSLV11Cmd_SetPixel.Create;
begin
  inherited Create;
  ImgName := 'CI_SETPIXEL';
  if LoadImage = 0 then ImgName := '';
  Description := 'This command sets an individual pixel to the "ON" or "OFF" state.';
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetPixel.GetCmdText : string;
var
  X,Y : byte;
  S   : string;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  if Parameters['OnOff'] = 'OFF' then
    S := 'OFF'
  else
    S := 'ON';
  result := '[SETPIXEL]='+HexByte(X)+','+HexByte(Y)+','+S;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetPixel.Load(var Data; var Index : integer) : integer;
begin
  if (TByteArray(Data)[Index] and $10) <> 0 then
    Parameters['OnOff'] := 'ON'
  else
    Parameters['OnOff'] := 'OFF';
  Parameters['Y'] := HexByte(TByteArray(Data)[Index] and $0f);
  inc(Index);
  Parameters['X'] := HexByte(TByteArray(Data)[Index]);
  inc(Index);
  result := 1;
end;

// -----------------------------------------------------------------------------

procedure TSLV11Cmd_SetPixel.Save(var Data; var Index : integer);  //   111byyyy - Set a pixel to a "b" value next byte is xxxxxxxx
var
  bb : byte;
begin
  bb := $e0 or (UnhexByte(Parameters['Y']) and $0f);
  if Parameters['OnOff'] <> 'OFF' then bb := bb or $10;
  TByteArray(Data)[Index] := bb;
  inc(Index);
  TByteArray(Data)[Index] := UnhexByte(Parameters['X']);
  inc(Index);
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetPixel.Size : integer;
begin
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetPixel.Clone : TGenericCommand;
begin
  result := TSLV11Cmd_SetPixel.Create;
  TSLV11Cmd_SetPixel(result).Parameters['X'] := Parameters['X'];
  TSLV11Cmd_SetPixel(result).Parameters['Y'] := Parameters['Y'];
  TSLV11Cmd_SetPixel(result).Parameters['OnOff'] := Parameters['OnOff'];
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetPixel.Compare(ACmd : TGenericCommand) : integer;
begin
  result := 0;
  if ACmd is TSLV11Cmd_SetPixel then
  begin
    if ((ACmd as TSLV11Cmd_SetPixel).Parameters['X'] = Parameters['X']) and
       ((ACmd as TSLV11Cmd_SetPixel).Parameters['Y'] = Parameters['Y']) and
       ((ACmd as TSLV11Cmd_SetPixel).Parameters['OnOff'] = Parameters['OnOff']) then
      result := 1;
  end;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetPixel.Run(PCB : TPCB; CharTable : TCharTable; NoDelay : boolean) : integer;
var
  X,Y : byte;
begin
  X := UnhexByte(Parameters['X']);
  Y := UnhexByte(Parameters['Y']);
  if assigned(PCB[X,Y]) then
  begin
    if Parameters['OnOff'] = 'OFF' then
      PCB[X,Y].Active := false
    else
      PCB[X,Y].Active := true;
  end;
  // This command has finished
  result := 2;
end;

// -----------------------------------------------------------------------------

function TSLV11Cmd_SetPixel.GetEditType : integer;
begin
  result := ED_ONOFF_PIXEL;
end;

// -----------------------------------------------------------------------------

end.

