unit Terminal;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, SerialIO, Buttons;

const
  MSG_GOT_CHAR = WM_USER+2235;

type
  TfrmTerminal = class(TForm)
    Panel1: TPanel;
    mmIO: TMemo;
    BitBtn1: TBitBtn;
    sd: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure mmIOKeyPress(Sender: TObject; var Key: Char);
    procedure BitBtn1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    procedure SIOCallback(AEvent: integer; ASender: TSerialIO);
    procedure MsgGotChar(var Msg : TMessage); message MSG_GOT_CHAR;
  public
    SIO  : TSerialIO;
    sbuf : string;
    function OpenTerminal(APort : string) : integer;
  end;

var
  frmTerminal: TfrmTerminal;

implementation

{$R *.dfm}

function TfrmTerminal.OpenTerminal(APort : string) : integer;
var
  tout : integer;
begin
  result := 0;
  SIO := TSerialIO.Create(APort,SIOCallback);
  try
    tout := 0;
    while (tout < 4) and (SIO.Error = SIO_ERR_CLOSED) do
    begin
      sleep(50);
      inc(tout);
    end;
    if SIO.Error <> SIO_ERR_NO_ERROR then
    begin
      ShowMessage('Can''t open port "'+APort+'"');
      exit;
    end;
    ShowModal;
    result := 1;
  finally
    SIO.Terminate;
  end;
end;

procedure TfrmTerminal.SIOCallback(AEvent: integer; ASender: TSerialIO);
var
  sBuf : char;
  ww   : word;
begin
  if (AEvent = SIO_CB_EVENT_BYTE_RECEIVED) then
  begin
    if ASender.Recv(@sBuf,1) = 1 then
    begin
      if sbuf <> #0 then
      begin
        ww := byte(sbuf);
        PostMessage(handle,MSG_GOT_CHAR,ww,0);
      end;
    end;
  end;
end;

procedure TfrmTerminal.BitBtn1Click(Sender: TObject);
begin
  if sd.Execute then
  begin
    if FileExists(sd.FileName) then
    begin
      if Application.MessageBox('File exists, overwrite?','Warning!',MB_YESNO+MB_ICONEXCLAMATION) = idNo then exit;
      try DeleteFile(sd.FileName); except end;
    end;
    mmIO.Lines.SaveToFile(sd.FileName);
  end;
  mmIO.SetFocus;
end;

procedure TfrmTerminal.FormCreate(Sender: TObject);
begin
  sbuf := '';
end;

procedure TfrmTerminal.FormShow(Sender: TObject);
begin
  mmIO.SetFocus;
end;

procedure TfrmTerminal.mmIOKeyPress(Sender: TObject; var Key: Char);
begin
  SIO.Send(@Key,1);
end;

procedure TfrmTerminal.MsgGotChar(var Msg : TMessage);
begin
  sbuf := sbuf+char(lo(Msg.WParam));
  if length(sbuf) > $fff0 then
    sbuf := copy(sbuf,length(sbuf)-$ffef,$fff0);
  mmIO.Lines.Text := sbuf;
  mmIO.SelStart := length(sbuf)-1;
  mmIO.SelLength := 0;
end;

end.

