unit Timeline;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GenericCommand, GenericDevice, ExtCtrls, StdCtrls, Menus, PCB;

type
  TTimeLineCommand = class (TPanel)
    constructor Create(AOwner: TComponent); override;
  private
    pb : TPaintBox;
    FCmd : TGenericCommand;
    procedure SetCmd(Value : TGenericCommand);
    procedure PaintAButton(Sender : TObject);
  public
    PLeft : integer;
    Selected : boolean;
    property Cmd : TGenericCommand read FCmd write SetCmd;
  end;

type
  TfrmTimeLine = class(TForm)
    Hldr: TPanel;
    mnuCmpPopup: TPopupMenu;
    mnuEditCmd: TMenuItem;
    mnuDeleteCmd: TMenuItem;
    mnuCopy: TMenuItem;
    mnuPaste: TMenuItem;
    N2: TMenuItem;
    N1: TMenuItem;
    mnuSave: TMenuItem;
    mnuLoad: TMenuItem;
    sd: TSaveDialog;
    od: TOpenDialog;
    procedure FormDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure FormDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuEditCmdClick(Sender: TObject);
    procedure mnuDeleteCmdClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure mnuCopyClick(Sender: TObject);
    procedure mnuPasteClick(Sender: TObject);
    procedure mnuSaveClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure mnuLoadClick(Sender: TObject);
  private
    CS : TCommandSet;
    Dev : TGenericDevice;
    OldScrNo : integer;
    Autoresize : boolean;
    Undo : TUndo;
    LastSelected : TGenericCommand;
    function RollTo(Cmd : TGenericCommand) : TUndo;
    procedure EditCmd(Cmd : TGenericCommand);
  public
    procedure ShowTimeline(ADev : TGenericDevice; ACS : TCommandSet; ACaption : string);
    procedure CommandClick(Sender : TObject);
    procedure CommandDoubleClick(Sender : TObject);
    procedure ShowPopup(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure CreateCmdButtons;
  end;

var
  frmTimeLine: TfrmTimeLine;

implementation

{$R *.dfm}

uses
  Palette, main, Constants, RowEditor, PixelEditor, BrightnessEditor,
  ColEditor, MsgEditor, DelayEditor, RowShiftEditor, IniFiles, SerialIO,
  TimelineHelper, SpcEditor, DirEditor, MirrorEditor, CallEditor, Clipbrd;

// ========== TTimeLineCommand =================================================

constructor TTimeLineCommand.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 5;
  Height := 40;
  Text := '';
  Caption := '';
  BevelInner := bvNone;
  BevelOuter := bvRaised;
  BevelWidth := 2;
  BorderStyle := bsNone;
  Selected := false;

  pb := TPaintBox.Create(self);
  pb.Parent := self;
  pb.Align := alClient;
  pb.OnPaint := PaintAButton;
end;

// -----------------------------------------------------------------------------

procedure TTimeLineCommand.SetCmd(Value : TGenericCommand);
begin
  FCmd := Value;
  if assigned(FCmd) then
  begin
    Hint := Cmd.Description;
    ShowHint := true;
    Width := 40;
  end;
  Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TTimeLineCommand.PaintAButton(Sender : TObject);
begin
  if Selected and assigned(Cmd) then
    pb.Canvas.Brush.Color := RGB(255,80,80)
  else
    pb.Canvas.Brush.Color := clBtnFace;
  pb.Canvas.FillRect(pb.ClientRect);
  if assigned(Cmd) then
    pb.Canvas.Draw(2,2,Cmd.Image);
end;

// ========== TfrmTimeLine =====================================================

procedure TfrmTimeLine.ShowTimeline(ADev : TGenericDevice; ACS : TCommandSet; ACaption : string);
var
  cfg : TIniFile;
begin
  Caption := 'Script''s Time Line ['+ACaption+']';
  LastSelected := nil;
  CS := ACS;
  Dev := ADev;
  Autoresize := true;
  CreateCmdButtons;
  ControlStyle := ControlStyle + [csDisplayDragImage];
  cfg := TIniFile.Create(IniFN);
  try
    Left := UnhexWord(cfg.ReadString('Timeline','PositionX','014A'));
    Top := UnhexWord(cfg.ReadString('Timeline','PositionY','01F4'));
    Width := UnhexWord(cfg.ReadString('Timeline','Width','0190'));
  finally
    cfg.Free;
  end;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.CommandClick(Sender : TObject);
var
  ii,jj : integer;
  Cmd : TGenericCommand;
  Shift : boolean;
  Ctrl : boolean;
  LastSelIdx : integer;
begin
  LastSelIdx := -1;
  Cmd := nil;
  Shift := ((GetKeyState(VK_LSHIFT) and $80) <> 0) or ((GetKeyState(VK_RSHIFT) and $80) <> 0);
  Ctrl := ((GetKeyState(VK_LCONTROL) and $80) <> 0) or ((GetKeyState(VK_RCONTROL) and $80) <> 0);
  if Shift then
  begin
    // Find the index of the last-clicked command
    for ii:=0 to Hldr.ComponentCount-1 do
    begin
      if (Hldr.Components[ii] is TTimeLineCommand) and
         (assigned((Hldr.Components[ii] as TTimeLineCommand).Cmd)) and
         ((Hldr.Components[ii] as TTimeLineCommand).Cmd = LastSelected) then
      begin
        LastSelIdx := ii;
        break;
      end;
    end;
  end;
  LastSelected := nil;
  // Unselect commands if needed
  for ii:=0 to Hldr.ComponentCount-1 do
  begin
    if (Hldr.Components[ii] is TTimeLineCommand) and assigned((Hldr.Components[ii] as TTimeLineCommand).Cmd) then
    begin
      if (not Ctrl) then
      begin
        TTimeLineCommand(Hldr.Components[ii]).Selected := false;
        TTimeLineCommand(Hldr.Components[ii]).Invalidate;
      end;
    end;
  end;
  // Select commands
  for ii:=0 to Hldr.ComponentCount-1 do
  begin
    if (Hldr.Components[ii] is TTimeLineCommand) and assigned((Hldr.Components[ii] as TTimeLineCommand).Cmd) then
    begin
      if (Hldr.Components[ii] = Sender) or (TTimeLineCommand(Hldr.Components[ii]).pb = Sender) then
      begin
        TTimeLineCommand(Hldr.Components[ii]).Selected := true;
        Cmd := TTimeLineCommand(Hldr.Components[ii]).Cmd;
        LastSelected := Cmd;
        TTimeLineCommand(Hldr.Components[ii]).Invalidate;
        if Shift then
        begin
          if ii < LastSelIdx then
          begin
            for jj:=ii to LastSelIdx do
            begin
              TTimeLineCommand(Hldr.Components[jj]).Selected := true;
              TTimeLineCommand(Hldr.Components[jj]).Invalidate
            end;
          end
          else
          begin
            for jj:=LastSelIdx to ii do
            begin
              TTimeLineCommand(Hldr.Components[jj]).Selected := true;
              TTimeLineCommand(Hldr.Components[jj]).Invalidate
            end;
          end;
        end;
        break;
      end;
    end;
  end;
  if (not Ctrl) and (not Shift) and assigned(Cmd) then RollTo(Cmd);
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.CommandDoubleClick(Sender : TObject);
var
  ii : integer;
  Cmd : TGenericCommand;
begin
  LastSelected := nil;
  Cmd := nil;
  for ii:=0 to Hldr.ComponentCount-1 do
  begin
    if (Hldr.Components[ii] is TTimeLineCommand) and assigned((Hldr.Components[ii] as TTimeLineCommand).Cmd) then
    begin
      if (Hldr.Components[ii] = Sender) or (TTimeLineCommand(Hldr.Components[ii]).pb = Sender) then
      begin
        TTimeLineCommand(Hldr.Components[ii]).Selected := true;
        Cmd := TTimeLineCommand(Hldr.Components[ii]).Cmd;
        LastSelected := Cmd;
      end
      else
        TTimeLineCommand(Hldr.Components[ii]).Selected := false;
      TTimeLineCommand(Hldr.Components[ii]).Invalidate;
    end;
  end;
  if assigned(Cmd) then
  begin
    EditCmd(Cmd);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.ShowPopup(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ii : integer;
  pt : TPoint;
  Cmd : TGenericCommand;
  sel : integer;
begin
  if Button <> mbRight then exit;
  Cmd := nil;
  LastSelected := nil;
  mnuCmpPopup.Tag := -1;
  // Check if anything is selected
  sel := 0;
  for ii:=0 to Hldr.ComponentCount-1 do
  begin
    if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
    begin
      inc(sel);
    end;
  end;
  for ii:=0 to Hldr.ComponentCount-1 do
  begin
    if (TTimeLineCommand(Hldr.Components[ii]).pb = Sender) then
    begin
      mnuCmpPopup.Tag := ii;
      if sel > 1 then break;
      TTimeLineCommand(Hldr.Components[ii]).Selected := true;
      Cmd := TTimeLineCommand(Hldr.Components[ii]).Cmd;
      LastSelected := Cmd;
    end
    else
    if sel < 2 then
      TTimeLineCommand(Hldr.Components[ii]).Selected := false;
    TTimeLineCommand(Hldr.Components[ii]).Invalidate;
  end;
  if assigned(Cmd) then RollTo(Cmd);
  mnuEditCmd.Enabled := false;
  mnuDeleteCmd.Enabled := false;
  mnuCopy.Enabled := false;
  mnuPaste.Enabled := false;
  if mnuCmpPopup.Tag >= 0 then
  begin
    mnuEditCmd.Enabled := true;
    mnuDeleteCmd.Enabled := true;
    mnuCopy.Enabled := true;
    mnuPaste.Enabled := true;
  end;
  if sel > 1 then
  begin
    mnuEditCmd.Enabled := false;
    mnuDeleteCmd.Enabled := true;
    mnuCopy.Enabled := true;
    mnuPaste.Enabled := true;
  end;
  pt.X := X; pt.Y := Y;
  pt := TTimeLineCommand(Hldr.Components[mnuCmpPopup.Tag]).pb.ClientToScreen(pt);
  mnuCmpPopup.Popup(pt.X,pt.Y);
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
var
  ScrNo,ii,jj : integer;
  cmd : TTimeLineCommand;
begin
  if Sender is TPaintBox then
  begin
    inc(X,(Sender as TPaintBox).Left);
    inc(Y,(Sender as TPaintBox).Top);
    Sender := (Sender as TPaintBox).Parent;
  end;
  if (Sender is TPanel) and ((Sender as TPanel).Name <> 'Hldr') then
  begin
    inc(X,(Sender as TPanel).Left);
    inc(Y,(Sender as TPanel).Top);
//    Sender := (Sender as TPanel).Parent;
  end;
  ScrNo := (X-2) div 40;
  Accept := IsDragObject(Source) and (Y >= 0) and (Y < 40) and (ScrNo >= 0) and (ScrNo <= CS.Count);
  if Accept then
  begin
    if ScrNo <> OldScrNo then
    begin
      OldScrNo := ScrNo;
      for ii:=Hldr.ComponentCount-1 downto 0 do
      begin
        if Hldr.Components[ii] is TTimeLineCommand then
        begin
          cmd := TTimeLineCommand(Hldr.Components[ii]);
          jj := ScrNo*40;
          if (Cmd.PLeft+22) < (jj-4) then
            Cmd.Left := Cmd.PLeft
          else
          if (Cmd.PLeft+22) > (jj+4) then
            Cmd.Left := Cmd.PLeft+40;
          Cmd.Invalidate;
        end;
      end;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  ScrNo,ii,jj : integer;
  cmd : TTimeLineCommand;
  cmd2 : TGenericCommand;
begin
  LastSelected := nil;
  if Sender is TPaintBox then
  begin
    inc(X,(Sender as TPaintBox).Left);
    inc(Y,(Sender as TPaintBox).Top);
    Sender := (Sender as TPaintBox).Parent;
  end;
  if (Sender is TPanel) and ((Sender as TPanel).Name <> 'Hldr') then
  begin
    inc(X,(Sender as TPanel).Left);
    inc(Y,(Sender as TPanel).Top);
  end;
  ScrNo := (X-2) div 40;
  if IsDragObject(Source) and (Y >= 0) and (Y < 40) and (ScrNo >= 0) and (ScrNo <= CS.Count) then
  begin
    // Assign new permanent positions for the commands
    for ii:=Hldr.ComponentCount-1 downto 0 do
    begin
      if Hldr.Components[ii] is TTimeLineCommand then
      begin
        cmd := TTimeLineCommand(Hldr.Components[ii]);
        jj := ScrNo*40;
        if (Cmd.PLeft+22) < (jj-4) then
        begin
          Cmd.Left := Cmd.PLeft;
        end
        else
        if (Cmd.PLeft+22) > (jj+4) then
        begin
          Cmd.Left := Cmd.PLeft+40;
          Cmd.PLeft := Cmd.Left;
        end;
        Cmd.Invalidate;
      end;
    end;
    // Let's drop it right here...
    cmd2 := TDragCmd(Source).Cmd.Clone;
    CS.Insert(ScrNo,cmd2);
    CreateCmdButtons;
    EditCmd(cmd2);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormCreate(Sender: TObject);
begin
  DoubleBuffered := true;
  OldScrNo := -1;
  LastSelected := nil;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormShow(Sender: TObject);
begin
  OldScrNo := -1;
  LastSelected := nil;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormDeactivate(Sender: TObject);
begin
  OldScrNo := -1;
  LastSelected := nil;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.CreateCmdButtons;
var
  ii,jj : integer;
  Cmd : TTimeLineCommand;
  bw,ww : integer;
begin
  for ii:=Hldr.ComponentCount-1 downto 0 do
  begin
    if (Hldr.Components[ii] is TTimeLineCommand) then
      Hldr.Components[ii].Free;
  end;
  jj := 0;
  for ii:=0 to CS.Count-1 do
  begin
    Cmd := TTimeLineCommand.Create(Hldr);
    Cmd.Parent := Hldr;
    Cmd.Top := 0;
    Cmd.Left := 2+ii*40;
    Cmd.PLeft := 2+ii*40;
    jj := ii;
    Cmd.Cmd := CS[ii];
    Cmd.OnClick := CommandClick;
    Cmd.OnDblClick := CommandDoubleClick;
    Cmd.pb.OnClick := CommandClick;
    Cmd.pb.OnDblClick := CommandDoubleClick;
    Cmd.OnDragOver := OnDragOver;
    Cmd.OnDragDrop := OnDragDrop;
    Cmd.pb.OnDragOver := OnDragOver;
    Cmd.pb.OnDragDrop := OnDragDrop;
    Cmd.pb.OnMouseUp := ShowPopup;
  end;
  // Add one more as a spacer
  inc(jj);
  Cmd := TTimeLineCommand.Create(Hldr);
  Cmd.Parent := Hldr;
  Cmd.Top := 0;
  Cmd.Left := 2+jj*40;
  Cmd.PLeft := 2+jj*40;
  Cmd.Cmd := nil;
  Cmd.OnDragOver := OnDragOver;
  Cmd.OnDragDrop := OnDragDrop;
  Cmd.pb.OnDragOver := OnDragOver;
  Cmd.pb.OnDragDrop := OnDragDrop;
  Cmd.pb.OnMouseUp := ShowPopup;

  bw := 2+jj*40+Cmd.Width;
  ww := bw;
  if bw < 400 then ww := 400;
  if bw > 800 then ww := 800;
  if Autoresize then ClientWidth := ww;
  Hldr.ClientWidth := bw+40;
  if ((2+jj*80) > ClientWidth) then
    ClientHeight := 42
  else
   ClientHeight := 40;
  frmTimelineControl.UpdateControls;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  PostMessage(frmMain.Handle,MSG_CLOSE_EDITOR,0,0);
  frmMain.SavePositions(self);
  LastSelected := nil;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.mnuEditCmdClick(Sender: TObject);
var
  Cmd : TTimeLineCommand;
begin
  LastSelected := nil;
  if (mnuCmpPopup.Tag < 0) or (mnuCmpPopup.Tag >= Hldr.ComponentCount) then exit;
  if Hldr.Components[mnuCmpPopup.Tag] is TTimeLineCommand then
  begin
    Cmd := TTimeLineCommand(Hldr.Components[mnuCmpPopup.Tag]);
    if Cmd.Cmd.GetEditType = ED_NO_EDIT then
      ShowMessage('This command has no editable properties.')
    else
      EditCmd(Cmd.Cmd);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.mnuDeleteCmdClick(Sender: TObject);
var
  Cmd : TTimeLineCommand;
  Cmd1 : TGenericCommand;
  ii,jj : integer;
  sel : integer;
  CC : TClipboard;
  ss : string;
begin
  LastSelected := nil;
  if (mnuCmpPopup.Tag < 0) or (mnuCmpPopup.Tag >= Hldr.ComponentCount) then exit;
  CC := Clipboard;
  CC.Open;
  try
    // Check if anything is selected
    sel := 0;
    for ii:=0 to Hldr.ComponentCount-1 do
    begin
      if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
      begin
        inc(sel);
      end;
    end;
    if sel = 1 then
    begin
      if Hldr.Components[mnuCmpPopup.Tag] is TTimeLineCommand then
      begin
        Cmd := TTimeLineCommand(Hldr.Components[mnuCmpPopup.Tag]);
        Cmd1 := Cmd.Cmd;
        for ii:=0 to CS.Count-1 do
          if Cmd1 = CS[ii] then
          begin
            CC.Clear;
            CC.AsText := CS[ii].AsText;
            CS[ii].Free;
            CS.Delete(ii);
            CreateCmdButtons;
            exit;
          end;
      end;
    end
    else
    if sel > 1 then
    begin
      CC.Clear;
      ss := '';
      for ii:=0 to Hldr.ComponentCount-1 do
      begin
        if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
        begin
          for jj:=CS.Count-1 downto 0 do
          begin
            if TTimeLineCommand(Hldr.Components[ii]).Cmd = CS[jj] then
            begin
              if ss = '' then
                ss := CS[jj].AsText
              else
                ss := ss+#13#10+CS[jj].AsText;
              CS.Delete(jj);
              break;
            end;
          end;
        end;
      end;
      CreateCmdButtons;
    end;
  finally
    CC.Close;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.mnuCopyClick(Sender: TObject);
var
  Cmd : TTimeLineCommand;
  Cmd1 : TGenericCommand;
  ii,jj : integer;
  sel : integer;
  CC : TClipboard;
  ss : string;
begin
  LastSelected := nil;
  if (mnuCmpPopup.Tag < 0) or (mnuCmpPopup.Tag >= Hldr.ComponentCount) then exit;
  CC := Clipboard;
  CC.Open;
  try
    // Check if anything is selected
    sel := 0;
    for ii:=0 to Hldr.ComponentCount-1 do
    begin
      if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
      begin
        inc(sel);
      end;
    end;
    if sel = 1 then
    begin
      if Hldr.Components[mnuCmpPopup.Tag] is TTimeLineCommand then
      begin
        Cmd := TTimeLineCommand(Hldr.Components[mnuCmpPopup.Tag]);
        Cmd1 := Cmd.Cmd;
        for ii:=0 to CS.Count-1 do
          if Cmd1 = CS[ii] then
          begin
            CC.Clear;
            CC.AsText := CS[ii].AsText;
            exit;
          end;
      end;
    end
    else
    if sel > 1 then
    begin
      CC.Clear;
      ss := '';
      for ii:=0 to Hldr.ComponentCount-1 do
      begin
        if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
        begin
          for jj:=CS.Count-1 downto 0 do
          begin
            if TTimeLineCommand(Hldr.Components[ii]).Cmd = CS[jj] then
            begin
              if ss = '' then
                ss := CS[jj].AsText
              else
                ss := ss+#13#10+CS[jj].AsText;
              break;
            end;
          end;
        end;
      end;
      CC.AsText := ss;
    end;
  finally
    CC.Close;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.mnuPasteClick(Sender: TObject);
var
  Cmd : TTimeLineCommand;
  Cmd1 : TGenericCommand;
  ii,jj : integer;
  idx : integer;
  CC : TClipboard;
  sl : TStringList;
begin
  LastSelected := nil;
  if (mnuCmpPopup.Tag < 0) or (mnuCmpPopup.Tag >= Hldr.ComponentCount) then exit;
  if not (Hldr.Components[mnuCmpPopup.Tag] is TTimeLineCommand) then exit;
  CC := Clipboard;
  CC.Open;
  try
    Cmd := TTimeLineCommand(Hldr.Components[mnuCmpPopup.Tag]);
    Cmd1 := Cmd.Cmd;
    for ii:=0 to CS.Count-1 do
    begin
      if Cmd1 = CS[ii] then
      begin
        idx := ii;
        sl := TStringList.Create;
        try
          sl.Text := CC.AsText;
          for jj:=0 to sl.Count-1 do
          begin
            CS.Insert(Idx,Dev.CmdFromText(sl[jj]));
            inc(Idx);
          end;
        finally
          sl.Free;
        end;
        CreateCmdButtons;
        exit;
      end;
    end;
  finally
    CC.Close;
  end;
end;

// -----------------------------------------------------------------------------

function TfrmTimeLine.RollTo(Cmd : TGenericCommand) : TUndo;
begin
  Dev.PCB.Off;
  result := Dev.Run(true,CS,Cmd);
  frmMain.Repaint;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.EditCmd(Cmd : TGenericCommand);
var
  cfg : TIniFile;
  sTop,sLeft : string;
  iTop,iLeft : integer;
begin
  if not assigned(Dev) then exit;
  // Read the editor's position
  cfg := TIniFile.Create(IniFN);
  try
    sLeft := cfg.ReadString('Editors','PositionX','00FF');
    sTop := cfg.ReadString('Editors','PositionY','00FF');
    iLeft := UnhexWord(sLeft);
    iTop := UnhexWord(sTop);
  finally
    cfg.Free;
  end;
  Undo := RollTo(Cmd);
  case Cmd.GetEditType of
    ED_NO_EDIT:
    begin
      exit;
    end;
    ED_TOP_ROW:
    begin
      frmRowEditor.Left := iLeft;
      frmRowEditor.Top := iTop;
      if Dev.PCB.RowCount > 0 then
        frmRowEditor.OpenEditor(Cmd,0);
      Dev.Modified := true;
    end;
    ED_BOTTOM_ROW:
    begin
      frmRowEditor.Left := iLeft;
      frmRowEditor.Top := iTop;
      if Dev.PCB.RowCount > 0 then
        frmRowEditor.OpenEditor(Cmd,Dev.PCB.RowCount-1);
      Dev.Modified := true;
    end;
    ED_SET_PIXEL:
    begin
      frmPixelEditor.Left := iLeft;
      frmPixelEditor.Top := iTop;
      if (Dev.PCB.RowCount > 0) and (Dev.PCB.ColCount > 0) then
        frmPixelEditor.OpenEditor(Cmd,paOn,@Undo);
      Dev.Modified := true;
    end;
    ED_CLR_PIXEL:
    begin
      frmPixelEditor.Left := iLeft;
      frmPixelEditor.Top := iTop;
      if (Dev.PCB.RowCount > 0) and (Dev.PCB.ColCount > 0) then
        frmPixelEditor.OpenEditor(Cmd,paOff,@Undo);
      Dev.Modified := true;
    end;
    ED_BRIGHTNESS:
    begin
      frmBrightnessEditor.Left := iLeft;
      frmBrightnessEditor.Top := iTop;
      frmBrightnessEditor.OpenEditor(Cmd);
      Dev.Modified := true;
    end;
    ED_ROW_LEFT:
    begin
      frmRowShiftEditor.Left := iLeft;
      frmRowShiftEditor.Top := iTop;
      frmRowShiftEditor.OpenEditor(Cmd,true,@Undo);
      Dev.Modified := true;
    end;
    ED_ROW_RIGHT:
    begin
      frmRowShiftEditor.Left := iLeft;
      frmRowShiftEditor.Top := iTop;
      frmRowShiftEditor.OpenEditor(Cmd,false,@Undo);
      Dev.Modified := true;
    end;
    ED_INS_FROM_LEFT:
    begin
      frmColEditor.Left := iLeft;
      frmColEditor.Top := iTop;
      frmColEditor.OpenEditor(Cmd,0);
      Dev.Modified := true;
    end;
    ED_INS_FROM_RIGHT:
    begin
      frmColEditor.Left := iLeft;
      frmColEditor.Top := iTop;
      frmColEditor.OpenEditor(Cmd,Dev.PCB.ColCount-1);
      Dev.Modified := true;
    end;
    ED_SIMPLE_MSG:
    begin
      frmMsgEditor.Left := iLeft;
      frmMsgEditor.Top := iTop;
      frmMsgEditor.OpenEditor(Cmd,31);  // TODO : Display
      Dev.Modified := true;
    end;
    ED_SET_DELAY:
    begin
      frmDelayEditor.Left := iLeft;
      frmDelayEditor.Top := iTop;
      frmDelayEditor.OpenEditor(Cmd,$3f);
      Dev.Modified := true;
    end;
    ED_SPACING:
    begin
      frmSpcEditor.Left := iLeft;
      frmSpcEditor.Top := iTop;
      frmSpcEditor.OpenEditor(Cmd,3);  // TODO : Display
      Dev.Modified := true;
    end;
    ED_DIRECTION:
    begin
      frmDirEditor.Left := iLeft;
      frmDirEditor.Top := iTop;
      frmDirEditor.OpenEditor(Cmd);  // TODO : Display
      Dev.Modified := true;
    end;
    ED_MIRRORING:
    begin
      frmMirrorEditor.Left := iLeft;
      frmMirrorEditor.Top := iTop;
      frmMirrorEditor.OpenEditor(Cmd);  // TODO : Display
      Dev.Modified := true;
    end;
    ED_HBUF:
    begin
      frmRowEditor.Left := iLeft;
      frmRowEditor.Top := iTop;
      if Dev.PCB.RowCount > 0 then
        frmRowEditor.OpenEditor(Cmd,0);
      Dev.Modified := true;
    end;
    ED_VBUF:
    begin
      frmColEditor.Left := iLeft;
      frmColEditor.Top := iTop;
      frmColEditor.OpenEditor(Cmd,0);
      Dev.Modified := true;
    end;
    ED_CALL:
    begin
      frmCallEditor.Left := iLeft;
      frmCallEditor.Top := iTop;
      frmCallEditor.OpenEditor(Cmd);
      Dev.Modified := true;
    end;
    ED_GOTO:
    begin
      frmCallEditor.Left := iLeft;
      frmCallEditor.Top := iTop;
      frmCallEditor.OpenEditor(Cmd);
      Dev.Modified := true;
    end;
    ED_SIMPLE_MSG_256:
    begin
      frmMsgEditor.Left := iLeft;
      frmMsgEditor.Top := iTop;
      frmMsgEditor.OpenEditor(Cmd,255);  // TODO : Display
      Dev.Modified := true;
    end;
    ED_SET_DELAY_256:
    begin
      frmDelayEditor.Left := iLeft;
      frmDelayEditor.Top := iTop;
      frmDelayEditor.OpenEditor(Cmd,$ff,0.002);
      Dev.Modified := true;
    end;
    ED_SET_DELAY_2560:
    begin
      frmDelayEditor.Left := iLeft;
      frmDelayEditor.Top := iTop;
      frmDelayEditor.OpenEditor(Cmd,$ff,0.02);
      Dev.Modified := true;
    end;
    ED_ONOFF_PIXEL:
    begin
      frmPixelEditor.Left := iLeft;
      frmPixelEditor.Top := iTop;
      if (Dev.PCB.RowCount > 0) and (Dev.PCB.ColCount > 0) then
        frmPixelEditor.OpenEditor(Cmd,paFlip,@Undo);
      Dev.Modified := true;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormResize(Sender: TObject);
begin
  Autoresize := false;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.mnuSaveClick(Sender: TObject);
var
  ss : string;
begin
  if not assigned(Dev) then exit;
  sd.Filter := 'Single Script (*.sc1)|*.sc1';
  if not sd.Execute then exit;
  ss := uppercase(ExtractFileExt(sd.FileName));
  if ss = '' then sd.FileName := trim(sd.FileName)+'.sc1';
  if FileExists(sd.FileName) then
  begin
    if Application.MessageBox(pchar('A '+ss+' file "'+sd.FileName+'" already exists. Overwrite?'),'Warning!',MB_YESNO+MB_ICONHAND) = idNo then exit;
    try DeleteFile(sd.FileName); except end;
  end;
  Dev.WriteTo(sd.FileName,CS);
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.FormKeyPress(Sender: TObject; var Key: Char);
var
  Cmd : TTimeLineCommand;
  Cmd1 : TGenericCommand;
  ii,jj,kk : integer;
  idx : integer;
  CC : TClipboard;
  ss : string;
  sl : TStringList;
begin
  case Key of
     #3: // Ctrl-C;
    begin
      LastSelected := nil;
      CC := Clipboard;
      CC.Open;
      try
        CC.Clear;
        ss := '';
        for ii:=0 to Hldr.ComponentCount-1 do
        begin
          if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
          begin
            for jj:=CS.Count-1 downto 0 do
            begin
              if TTimeLineCommand(Hldr.Components[ii]).Cmd = CS[jj] then
              begin
                if ss = '' then
                  ss := CS[jj].AsText
                else
                  ss := ss+#13#10+CS[jj].AsText;
                break;
              end;
            end;
          end;
        end;
        CC.AsText := ss;
      finally
        CC.Close;
      end;
    end;
    #24: // Ctrl-X;
    begin
      LastSelected := nil;
      CC := Clipboard;
      CC.Open;
      try
        CC.Clear;
        ss := '';
        for ii:=0 to Hldr.ComponentCount-1 do
        begin
          if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
          begin
            for jj:=CS.Count-1 downto 0 do
            begin
              if TTimeLineCommand(Hldr.Components[ii]).Cmd = CS[jj] then
              begin
                if ss = '' then
                  ss := CS[jj].AsText
                else
                  ss := ss+#13#10+CS[jj].AsText;
                CS.Delete(jj);
                break;
              end;
            end;
          end;
        end;
        CreateCmdButtons;
      finally
        CC.Close;
      end;
    end;
    #22: // Ctrl-V;
    begin
      LastSelected := nil;
      for ii:=0 to Hldr.ComponentCount-1 do
      begin
        if (Hldr.Components[ii] is TTimeLineCommand) and TTimeLineCommand(Hldr.Components[ii]).Selected then
        begin
          Cmd := TTimeLineCommand(Hldr.Components[ii]);
          Cmd1 := Cmd.Cmd;
          for kk:=0 to CS.Count-1 do
          begin
            if Cmd1 = CS[kk] then
            begin
              idx := kk;
              sl := TStringList.Create;
              CC := Clipboard;
              CC.Open;
              try
                sl.Text := CC.AsText;
                for jj:=0 to sl.Count-1 do
                begin
                  CS.Insert(Idx,Dev.CmdFromText(sl[jj]));
                  inc(Idx);
                end;
              finally
                sl.Free;
                CC.Close;
              end;
              CreateCmdButtons;
              for jj:=0 to Hldr.ComponentCount-1 do
              begin
                if (Hldr.Components[jj] is TTimeLineCommand) and (TTimeLineCommand(Hldr.Components[jj]).Cmd = Cmd1) then
                begin
                  TTimeLineCommand(Hldr.Components[jj]).Selected := true;
                  break;
                end;
              end;
              exit;
            end;
          end;
        end;
      end;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimeLine.mnuLoadClick(Sender: TObject);
begin
  od.Filter := 'Script File (*.sc1)|*.sc1|Script Set (*.scr)|*.cht|All Files (*.*)|*.*';
  if not od.Execute then exit;
  if not FileExists(od.FileName) then
  begin
    ShowMessage('File "'+od.FileName+'" does not exist.');
    exit;
  end;
  if Dev.ReadFrom(od.FileName,CS) = 1 then
  begin
    CreateCmdButtons;
    Dev.Modified := true;
  end;
end;

end.

