unit TimelineHelper;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, GenericDevice, GenericCommand, Timeline, ExtCtrls, ScriptSet;

type
  TfrmTimelineControl = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    lbTotalMem: TLabel;
    lbUsedMem: TLabel;
    GroupBox2: TGroupBox;
    rbEdit: TRadioButton;
    rbRun: TRadioButton;
    Timer1: TTimer;
    procedure FormDeactivate(Sender: TObject);
    procedure rbEditClick(Sender: TObject);
    procedure rbRunClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    Dev : TGenericDevice;
    Scr : TCommandSet;
    ScrNo : integer;
    Timeline : TfrmTimeline;
    SavedScripts : TScriptSet;
    procedure StopRun;
    procedure StartRun;
  public
    procedure OpenForm(ADev : TGenericDevice; AScr : TCommandSet; AScrNo : integer; ATimeline : TfrmTimeline);
    procedure UpdateControls;
  end;

var
  frmTimelineControl: TfrmTimelineControl;

implementation

uses main;

{$R *.dfm}

// =============================================================================

procedure TfrmTimelineControl.OpenForm(ADev : TGenericDevice; AScr : TCommandSet; AScrNo : integer; ATimeline : TfrmTimeline);
begin
  Dev := ADev;
  Scr := AScr;
  ScrNo := AScrNo;
  Timeline := ATimeline;
  Left := Timeline.Left;
  Top := Timeline.Top+Timeline.Height+50;
  UpdateControls;
  SavedScripts := nil;
  Show;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.UpdateControls;
begin
  if not assigned(Dev) then exit;
  lbTotalMem.Caption := IntToStr(Dev.ScriptLimit[ScrNo]);
  lbUsedMem.Caption := IntToStr(Scr.Size);
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.FormDeactivate(Sender: TObject);
begin
  StopRun;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.rbEditClick(Sender: TObject);
begin
  if frmMain.mnuRun.Checked then StopRun;
  frmMain.mnuRun.Checked := false;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.rbRunClick(Sender: TObject);
begin
  if not frmMain.mnuRun.Checked then StartRun;
  frmMain.mnuRun.Checked := true;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.StopRun;
begin
  if not assigned(SavedScripts) then exit;
  try if assigned(frmMain.RunTimer) then frmMain.RunTimer.Terminate; except end;
  frmMain.RunTimer := nil;
  if assigned(Dev) then Dev.PCB.Off;
  if not rbEdit.Checked then rbEdit.Checked := true;
  if assigned(Dev) then
  begin
    Dev.ScriptSet.Delete(0);
    Dev.ScriptSet.Free;
    Dev.ScriptSet := SavedScripts;
    SavedScripts := nil;
    Dev.ScriptSet[Dev.ActiveScript].Reset;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.StartRun;
begin
  if assigned(SavedScripts) then exit;
  if assigned(Dev) then
  begin
    SavedScripts := Dev.ScriptSet;
    Dev.ScriptSet := TScriptSet.Create;
    Dev.ActiveScript := Dev.ScriptSet.Add(Scr);
    Dev.ScriptSet[Dev.ActiveScript].Reset;
    frmMain.RunTimer := TRunTimer.Create;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.Timer1Timer(Sender: TObject);
begin
  if assigned(Dev) then
  begin
    Dev.Run(false,nil,nil);
    frmMain.Invalidate;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.FormHide(Sender: TObject);
begin
  StopRun;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.FormCreate(Sender: TObject);
begin
  Dev := nil;
  SavedScripts := nil;
end;

// -----------------------------------------------------------------------------

procedure TfrmTimelineControl.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  StopRun;
  Dev := nil;
end;

// -----------------------------------------------------------------------------

end.
