unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, SerialIO, GenericDevice, ComCtrls, ExtCtrls, GenericCommand;

const
  MSG_CLOSE_EDITOR = WM_USER + 4444;
  MSG_INVALIDATE   = WM_USER + 4445;

type
  TRunTimer = class (TThread)
    constructor Create;
    procedure Execute; override;
  end;

type
  TfrmMain = class(TForm)
    MainMenu1: TMainMenu;
    mnuDevice: TMenuItem;
    mnuFile: TMenuItem;
    mnuEdit: TMenuItem;
    mnuHelp: TMenuItem;
    Discover1: TMenuItem;
    Force1: TMenuItem;
    Read1: TMenuItem;
    Write1: TMenuItem;
    Read2: TMenuItem;
    Write2: TMenuItem;
    EditFont1: TMenuItem;
    mnuEditScript: TMenuItem;
    About1: TMenuItem;
    sb: TStatusBar;
    mnuRun: TMenuItem;
    sd: TSaveDialog;
    od: TOpenDialog;
    procedure About1Click(Sender: TObject);
    procedure Discover1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure mnuRunClick(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Read1Click(Sender: TObject);
    procedure Write1Click(Sender: TObject);
    procedure Read2Click(Sender: TObject);
    procedure Write2Click(Sender: TObject);
    procedure Force1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure EditFont1Click(Sender: TObject);
  private
    procedure EditScript(Sender : TObject);
    procedure CloseEditor(var Msg : TMessage); message MSG_CLOSE_EDITOR;
    procedure MsgInvalidate(var Msg : TMessage); message MSG_INVALIDATE;
    procedure SaveEdits;
    procedure DisableMenu;
    procedure EnableMenu;
  public
    Discovered : TStringList;
    Forced : TStringList;
    Resp : string;
    Dev : TGenericDevice;
    RunTimer : TRunTimer;
    // ------------------
    EdCS : TCommandSet;
    EdScr : integer;
    // ------------------
    procedure SIOCallback(AEvent: integer; ASender: TSerialIO);
    procedure SetupDevice(APort : string; ADeviceID : string);
    procedure SetStatusBar(AIDString : string);
    procedure SavePositions(Sender : TForm);
    procedure RunTimeEvent;
  end;

var
  frmMain: TfrmMain;

procedure DisableAPP;
procedure EnableAPP;

implementation

uses
  about, selectdevice, SLV01, SLV05, SLV07, SLV11, Palette, Timeline,
  FontEditor, BrightnessEditor, RowEditor, RowShiftEditor, ColEditor,
  MsgEditor, PixelEditor, DelayEditor, IniFiles, Constants, TimelineHelper,
  PCB, AssignDevice;

{$R *.dfm}
{$R Images.res}

// =============================================================================

procedure DisableAPP;
begin
  frmTimeLine.Enabled := false;
  frmPalette.Enabled := false;
end;

// -----------------------------------------------------------------------------

procedure EnableAPP;
begin
  frmTimeLine.Enabled := true;
  frmPalette.Enabled := true;
end;

// -----------------------------------------------------------------------------

function CMDDifferent(Cmd1,Cmd2 : TGenericCommand) : integer;
begin
  if not assigned(Cmd1) or not assigned(Cmd2) or (Cmd1.Compare(Cmd2) = 1) then
    result := 0
  else
    result := 1;
end;

// -----------------------------------------------------------------------------

function CSDifferent(CS1,CS2 : TCommandSet) : integer;
var
  ii : integer;
begin
  result := 1;
  if not assigned(CS1) or not assigned(CS2) or (CS1.Count <> CS2.Count) then exit;
  for ii:=0 to CS1.Count do
    if CMDDifferent(CS1[ii],CS2[ii]) = 1 then exit;
  result := 0;
end;

// ========== TRunTimer ========================================================

constructor TRunTimer.Create;
begin
  inherited Create(true);
  FreeOnTerminate := true;
  resume;
end;

// -----------------------------------------------------------------------------

procedure TRunTimer.Execute;
begin
  while not Terminated do
  begin
    frmMain.RunTimeEvent;
  end;
end;

// =============================================================================

procedure TfrmMain.About1Click(Sender: TObject);
begin
  frmAbout.ShowModal;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.SIOCallback(AEvent: integer; ASender: TSerialIO);
var
  sBuf : char;
begin
  if (AEvent = SIO_CB_EVENT_BYTE_RECEIVED) then
  begin
    if ASender.Recv(@sBuf,1) = 1 then
    begin
      if sbuf <> #0 then Resp := Resp+sBuf;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.RunTimeEvent;
var
  u : TUndo;
begin
  if assigned(Dev) then
  begin
    u := Dev.Run(false,nil,nil);
    if u[0,0] then PostMessage(Handle,MSG_INVALIDATE,0,0);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.Discover1Click(Sender: TObject);
var
  sl : TStringList;
  ii,tout : integer;
  SIO : TSerialIO;
  ss : string;
  pfnt : boolean;
  // -------------------------------------------
  function WaitForEOL : boolean;
  var
    tout : integer;
  begin
    tout := 4;
    while (pos(#13,Resp) = 0) and (tout > 0) do
    begin
      dec(tout);
      sleep(50);
    end;
    result := (tout > 0);
  end;
  // -------------------------------------------
  function GetRespStr : string;
  var
    ii,jj : integer;
  begin
    setlength(result,length(Resp));
    ii := 1;
    jj := 1;
    while (ii <= length(Resp)) do
    begin
      if Resp[ii] = #13 then
      begin
        setlength(result,jj-1);
        Resp := copy(Resp,ii+1,length(Resp));
        exit;
      end
      else
      begin
        if byte(Resp[ii]) >= $20 then
        begin
          result[jj] := Resp[ii];
          inc(jj);
        end;
      end;
      inc(ii);
    end;
    result := '';
  end;
  // -------------------------------------------
begin
  pfnt := frmFontEditor.Visible;
  if pfnt then frmFontEditor.Hide;
  SaveEdits;
  Discovered.Clear;
  sl := TStringList.Create;
  try
    EnumerateComPorts(sl);
    for ii:=0 to sl.Count-1 do
    begin
      SIO := TSerialIO.Create(sl.Names[ii],SIOCallback);
      try
        tout := 0;
        while (tout < 4) and (SIO.Error = SIO_ERR_CLOSED) do
        begin
          sleep(50);
          inc(tout);
        end;
        if SIO.Error <> SIO_ERR_NO_ERROR then continue;
        resp := '';
        SIO.Send('?',1);
        if not WaitForEOL then continue;
        ss := GetRespStr;
        if (length(ss) < 16) then continue;
        if (copy(ss,1,7) = 'SLV0703') or (copy(ss,1,16) = 'Stop Light V 5.4') or (copy(ss,1,7) = 'SLV0504') or (copy(ss,1,7) = 'SLV1102') then
          Discovered.Add(sl.Names[ii]+'='+ss);
      finally
        SIO.Terminate;
        sleep(100);
      end;
    end;
    if Discovered.Count = 0 then
    begin
      ShowMessage('There are no CE devices connected to your system. If you are sure that there is one then '+
                  'it is probably in use by another program.');
      exit;
    end;
    ii := frmSelectDevice.SelectDevice(Discovered);
    if ii < 0 then exit;
    try
      if assigned(Dev) then Dev.Free;
    except
    end;
    Dev := nil;
    SetupDevice(Discovered.Names[ii],Discovered.ValueFromIndex[ii]);
    Invalidate;
    if assigned(Dev) then
    begin
//      Dev.ReadDevice;
      if pfnt then frmFontEditor.OpenEditor(Dev);
    end;
  finally
    sl.Free;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Discovered := TStringList.Create;
  Forced := TStringList.Create;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(Discovered);
  FreeAndNil(Forced);
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  ii : integer;
begin
  if assigned(Dev) then
  begin
    ii := Dev.Inputs.Highlight(Canvas,X,Y);
    if ii < 0 then
      sb.Panels[2].Text := ''
    else
      sb.Panels[2].Text := Dev.Inputs[ii].Name;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if assigned(Dev) then
  begin
    if Dev.Inputs.Click(X,Y) <> 0 then
    begin
      Invalidate;
      Dev.UpdateActiveScript;
    end;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormPaint(Sender: TObject);
var
  r : TRect;
begin
  r := ClientRect;
  inc(r.Left);
  inc(r.Top);
  dec(r.Bottom,sb.Height);
  if assigned(Dev) then Dev.DrawTo(Canvas,r);
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormResize(Sender: TObject);
begin
  Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormShow(Sender: TObject);
var
  cfg : TIniFile;
begin
  DoubleBuffered := true;
  EdCS := nil;
  EdScr := -1;
  RunTimer := nil;
  cfg := TIniFile.Create(IniFN);
  try
    Left := UnhexWord(cfg.ReadString('APP','PositionX','0180'));
    Top := UnhexWord(cfg.ReadString('APP','PositionY','0040'));
    Width := UnhexWord(cfg.ReadString('APP','Width','01F4'));
    Height := UnhexWord(cfg.ReadString('APP','Height','0190'));
  finally
    cfg.Free;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.mnuRunClick(Sender: TObject);
begin
  if mnuRun.Checked or assigned(EdCS) then
  begin
    mnuRun.Checked := false;
    try if assigned(RunTimer) then RunTimer.Terminate; except end;
    RunTimer := nil;
  end
  else
  begin
    mnuRun.Checked := true;
    RunTimer := TRunTimer.Create;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.Read1Click(Sender: TObject);
var
  pfnt : boolean;
begin
  if mnuRun.Checked then
  begin
    mnuRun.Checked := false;
    try if assigned(RunTimer) then RunTimer.Terminate; except end;
    RunTimer := nil;
  end;
  pfnt := frmFontEditor.Visible;
  if pfnt then frmFontEditor.Hide;
  SaveEdits;
  if frmPalette.Visible then frmPalette.Close;
  if frmTimeLine.Visible then frmTimeLine.Close;
  if frmTimelineControl.Visible then frmTimelineControl.Close;
  if assigned(Dev) then
  begin
    Dev.ReadDevice;
    if pfnt then frmFontEditor.OpenEditor(Dev);
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.SetupDevice(APort : string; ADeviceID : string);
  // -------------------------------------
  function NewMenuItem(Caption : string; Tag1 : integer; Tag2 : string; Action : TNotifyEvent; Check : boolean) : TMenuItem;
  begin
    result         := TMenuItem.Create(nil);
    result.Caption := Caption;
    result.Tag     := Tag1;
    result.Hint    := Tag2;
    result.OnClick := Action;
    result.Checked := Check;
  end;
  // -------------------------------------
begin
  mnuEditScript.Clear;
  if copy(ADeviceID,1,5) = 'SLV05' then
    Dev := TSLV05.Create(APort,ADeviceID)
  else
  if copy(ADeviceID,1,5) = 'SLV07' then
    Dev := TSLV07.Create(APort,ADeviceID)
  else
  if copy(ADeviceID,1,5) = 'SLV11' then
  begin
    Dev := TSLV11.Create(APort,ADeviceID);
    mnuEditScript.Add(NewMenuItem('Script 00000 (Parking Light - Accessory Power is Off)',0,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 00001 (Brake)',1,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 00010 (Left Turn)',2,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 00011 (Right Turn)',3,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 00100 (Emergency Signal)',4,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 00101 (Reverse)',5,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 00110 (Multipurpose Script)',6,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 00111 (Multipurpose Script)',7,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01000 (Multipurpose Script)',8,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01001 (Multipurpose Script)',9,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01010 (Multipurpose Script)',10,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01011 (Multipurpose Script)',11,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01100 (Multipurpose Script)',12,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01101 (Multipurpose Script)',13,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01110 (Multipurpose Script)',14,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 01111 (Running Light - Accessory Power is On)',15,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10000 (Multipurpose Script)',16,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10001 (Multipurpose Script)',17,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10010 (Multipurpose Script)',18,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10011 (Multipurpose Script)',19,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10100 (Multipurpose Script)',20,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10101 (Multipurpose Script)',21,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10110 (Multipurpose Script)',22,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 10111 (Multipurpose Script)',23,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11000 (Multipurpose Script)',24,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11001 (Multipurpose Script)',25,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11010 (Multipurpose Script)',26,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11011 (Multipurpose Script)',27,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11100 (Multipurpose Script)',28,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11101 (Multipurpose Script)',29,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11110 (Multipurpose Script)',30,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 11111 (Multipurpose Script)',31,'',EditScript,false));
  end
  else
  if copy(ADeviceID,1,16) = 'Stop Light V 5.4' then
  begin
    Dev := TSLV01.Create(APort,ADeviceID);
    mnuEditScript.Add(NewMenuItem('Script 000 (Running Light)',0,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 001 start (Brake)',2,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 001 end (Brake)',3,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 010 start (Right Turn)',4,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 010 end (Right Turn)',5,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 011 start (Right Turn and Brake)',6,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 011 end (Right Turn and Brake)',7,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 100 start (Left Turn)',8,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 100 end (Left Turn)',9,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 101 start (Left Turn and Brake)',10,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 101 end (Left Turn and Brake)',11,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 110 start',12,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 110 end',13,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 111 start',14,'',EditScript,false));
    mnuEditScript.Add(NewMenuItem('Script 111 end',15,'',EditScript,false));
  end
  else
    exit;
  SetStatusBar(Dev.IDString);
  Dev.Reset;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.SetStatusBar(AIDString : string);
var
  sid : string;
  ii : integer;
begin
  ii := pos(' ',AIDString);
  if ii = 0 then
  begin
    SB.Panels[0].Text := 'Unknown Device ID';
    SB.Panels[1].Text := '';
    exit;
  end;
  sid := copy(AIDString,1,ii-1);
  if copy(sid,1,5) = 'SLV05' then
  begin
    SB.Panels[0].Text := 'Motorcycle Tail Light (V.'+copy(sid,6,255)+', '+TS2DateStr(copy(AIDString,length(AIDString)-7,8))+')';
    SB.Panels[1].Text := sid;
  end
  else
  if copy(sid,1,5) = 'SLV07' then
  begin
    SB.Panels[0].Text := 'Bicycle Tail Light (V.'+copy(sid,6,255)+', '+TS2DateStr(copy(AIDString,length(AIDString)-7,8))+')';
    SB.Panels[1].Text := sid;
  end
  else
  if copy(sid,1,5) = 'SLV11' then
  begin
    SB.Panels[0].Text := 'Car Tail Light Controller (V.'+copy(sid,6,255)+', '+TS2DateStr(copy(AIDString,length(AIDString)-7,8))+')';
    SB.Panels[1].Text := sid;
  end
  else
  if copy(AIDString,1,16) = 'Stop Light V 5.4' then
  begin
    SB.Panels[0].Text := 'Motorcycle Tail Light (V.1)';
    SB.Panels[1].Text := 'SLV0501';
  end
  else
  begin
    SB.Panels[0].Text := 'Unknown Device ID';
    SB.Panels[1].Text := '';
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.Write1Click(Sender: TObject);
begin
  SaveEdits;
  if assigned(Dev) then Dev.WriteDevice;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.Read2Click(Sender: TObject);
begin
  SaveEdits;
  if not assigned(Dev) then exit;
  od.Filter := 'Script Set (*.scr)|*.scr|Character Table (*.cht)|*.cht|All Files (*.*)|*.*';
  if not od.Execute then exit;
  if not FileExists(od.FileName) then
  begin
    ShowMessage('File "'+od.FileName+'" does not exist.');
    exit;
  end;
  Dev.ReadFrom(od.FileName,-1);
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.Write2Click(Sender: TObject);
var
  ss : string;
begin
  SaveEdits;
  if not assigned(Dev) then exit;
  sd.Filter := 'Script Set (*.scr)|*.scr|Character Table (*.cht)|*.cht';
  if not sd.Execute then exit;
  ss := uppercase(ExtractFileExt(sd.FileName));
  if ss = '' then
  begin
    case sd.FilterIndex of
      1: sd.FileName := trim(sd.FileName)+'.scr';
      2: sd.FileName := trim(sd.FileName)+'.cht';
    end;
  end
  else
  if ss = 'SCR' then
    sd.FilterIndex := 1
  else
  if ss = 'CHT' then
    sd.FilterIndex := 3;
  case sd.FilterIndex of
    1: ss := 'Scrip Set';
    2: ss := 'Character Table';
  end;
  if FileExists(sd.FileName) then
  begin
    if Application.MessageBox(pchar('A '+ss+' file "'+sd.FileName+'" already exists. Overwrite?'),'Warning!',MB_YESNO+MB_ICONHAND) = idNo then exit;
    try DeleteFile(sd.FileName); except end;
  end;
  case sd.FilterIndex of
    1: Dev.WriteTo(sd.FileName,-1);
    2: Dev.WriteTo(sd.FileName,-2);
  end;
end;

// -----------------------------------------------------------------------------

procedure TFrmMain.EditScript(Sender : TObject);
var
  ScrNo : integer;
  ii : integer;
begin
  if not assigned(Dev) or not assigned(Sender) or not (Sender is TMenuItem) then exit;
  if mnuRun.Checked then
  begin
    try if assigned(RunTimer) then RunTimer.Terminate; except end;
    RunTimer := nil;
    Dev.ScriptSet[Dev.ActiveScript].ActiveCommand := 0;
    mnuRun.Checked := false;
  end;
  Dev.PCB.Off;
  ScrNo := (Sender as TMenuItem).Tag;
  if assigned(EdCS) then
  begin
    if ScrNo = EdScr then exit;
    if CSDifferent(EdCS,Dev.ScriptSet[EdScr]) = 1 then
    begin
      case Application.MessageBox('There is a script currently being edited. Do you want to save changes made to it?','Warning!',MB_YESNOCANCEL) of
        idCancel: exit;
        idYes:
        begin
          Dev.ScriptSet[EdScr].Clear;
          for ii:=0 to EdCS.Count-1 do
            Dev.ScriptSet[EdScr].Add(EdCS[ii].Clone);
        end;
      end;
    end;
    FreeAndNil(EdCS);
  end;
  EdScr := ScrNo;
  EdCS := TCommandSet.Create;
  for ii:=0 to Dev.ScriptSet[ScrNo].Count-1 do
    EdCS.Add(Dev.ScriptSet[ScrNo][ii].Clone);
  Dev.PCB.Off;
  Invalidate;
  frmPalette.OpenPalette(Dev.SupportedCommands);
  frmTimeLine.ShowTimeline(Dev,EdCS,(Sender as TMenuItem).Caption);
  frmTimelineControl.OpenForm(Dev,EdCS,EdScr,frmTimeline);
  DisableMenu;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.CloseEditor(var Msg : TMessage);
begin
  if not assigned(Dev) then exit;
  SaveEdits;
  if frmPalette.Visible then frmPalette.Close;
  if frmTimeLine.Visible then frmTimeLine.Close;
  if frmTimelineControl.Visible then frmTimelineControl.Close;
  EnableMenu;
  Dev.PCB.Off;
  Dev.ActiveScript := 0;
  Dev.Inputs.Reset;
  Dev.Modified := true;
  Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.MsgInvalidate(var Msg : TMessage);
begin
  Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.SaveEdits;
var
  ii : integer;
begin
  if not assigned(Dev) then exit;
  if assigned(EdCS) then
  begin
    if CSDifferent(EdCS,Dev.ScriptSet[EdScr]) = 1 then
    begin
      if Application.MessageBox('There is a script currently being edited. Do you want to save changes made to it?','Warning!',MB_YESNO) = idYes then
      begin
        Dev.Modified := true;
        Dev.ScriptSet[EdScr].Clear;
        for ii:=0 to EdCS.Count-1 do
          Dev.ScriptSet[EdScr].Add(EdCS[ii].Clone);
      end;
    end;
    FreeAndNil(EdCS);
  end;
  Dev.Modified := false;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.Force1Click(Sender: TObject);
var
  Port  : string;
  DevID : string;
begin
  SaveEdits;
  if frmAssignDevice.SelectDevice(Port,DevID) = 0 then exit;
  if assigned(Dev) then Dev.Free;
  SetupDevice(Port,DevID);
  Invalidate;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not assigned(Dev) then exit;
  if Dev.Modified then
  begin
    if Application.MessageBox('Some scripts and/or characters were changed. If you close the application now all the changes will be lost. Do you still want to exit?','Warning!',MB_YESNO) = idYes then exit;
    Action := caNone;
    exit;
  end;
  if assigned(EdCS) then
  begin
    if CSDifferent(EdCS,Dev.ScriptSet[EdScr]) = 1 then
    begin
      if Application.MessageBox('There is a script currently being edited. If you close the application now all the changes mage to that script will be lost. Do you still want to exit?','Warning!',MB_YESNO) = idYes then exit;
      Action := caNone;
      exit;
    end;
  end;
  SavePositions(self);
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.EditFont1Click(Sender: TObject);
begin
  if not assigned(Dev) then exit;
  frmFontEditor.OpenEditor(Dev);
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.SavePositions(Sender : TForm);
var
  cfg : TIniFile;
  ww : word;
begin
  // Read the editor's position
  cfg := TIniFile.Create(IniFN);
  try
    if (Sender is TfrmBrightnessEditor) or
       (Sender is TfrmRowEditor) or
       (Sender is TfrmRowShiftEditor) or
       (Sender is TfrmColEditor) or
       (Sender is TfrmMsgEditor) or
       (Sender is TfrmPixelEditor) or
       (Sender is TfrmDelayEditor) then
    begin
      ww := Sender.Top;
      cfg.WriteString('Editors','PositionY',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Left;
      cfg.WriteString('Editors','PositionX',HexByte(hi(ww))+HexByte(lo(ww)));
    end;
    if (Sender is TfrmTimeLine) then
    begin
      ww := Sender.Top;
      cfg.WriteString('Timeline','PositionY',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Left;
      cfg.WriteString('Timeline','PositionX',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Width;
      cfg.WriteString('Timeline','Width',HexByte(hi(ww))+HexByte(lo(ww)));
    end;
    if (Sender is TfrmPalette) then
    begin
      ww := Sender.Top;
      cfg.WriteString('Palette','PositionY',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Left;
      cfg.WriteString('Palette','PositionX',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Width;
      cfg.WriteString('Palette','Width',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Height;
      cfg.WriteString('Palette','Height',HexByte(hi(ww))+HexByte(lo(ww)));
    end;
    if (Sender is TfrmMain) then
    begin
      ww := Sender.Top;
      cfg.WriteString('APP','PositionY',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Left;
      cfg.WriteString('APP','PositionX',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Width;
      cfg.WriteString('APP','Width',HexByte(hi(ww))+HexByte(lo(ww)));
      ww := Sender.Height;
      cfg.WriteString('APP','Height',HexByte(hi(ww))+HexByte(lo(ww)));
    end;
  finally
    cfg.Free;
  end;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.DisableMenu;
begin
  mnuDevice.Enabled := false;
  mnuFile.Enabled := false;
  mnuEdit.Enabled := false;
  mnuHelp.Enabled := false;
end;

// -----------------------------------------------------------------------------

procedure TfrmMain.EnableMenu;
begin
  mnuDevice.Enabled := true;
  mnuFile.Enabled := true;
  mnuEdit.Enabled := true;
  mnuHelp.Enabled := true;
end;

// -----------------------------------------------------------------------------

end.

